/**  
 * @file csl_serdes_debug.h
 *
 * @brief 
 *  Header file for functional layer of CSL SERDES DEBUG. 
 *
 *  It contains the various enumerations, structure definitions and function 
 *  declarations
 *
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2013, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/** ============================================================================ 
 *
 * @defgroup CSL_SERDES_DEBUG SERDES DEBUG
 * @ingroup CSL_SERDES_API
 *
 * @section Introduction
 *
 * @subsection xxx Overview
 *
 * @subsection References
 *    
 * ============================================================================
 */   
#include <ti/csl/soc.h>
#include <ti/csl/csl.h>

#define CSL_SERDES_MAX_DEBUG_COEFF_RANGE 512

extern CSL_Uint64 CSL_tscRead();

typedef struct tbusdump
{
    uint32_t    taddr[155];
    uint32_t    tbus_data[155];
} tbus;

typedef struct csl_serdes_dbg_cmc1
{
    uint32_t    C1[CSL_SERDES_MAX_DEBUG_COEFF_RANGE];
    uint32_t    CM[CSL_SERDES_MAX_DEBUG_COEFF_RANGE];
    uint32_t    ber[CSL_SERDES_MAX_DEBUG_COEFF_RANGE];
} csl_serdes_dbg_cmc1;

typedef struct csl_serdes_dbg_boost_att
{
    uint32_t    att[CSL_SERDES_MAX_DEBUG_COEFF_RANGE];
    uint32_t    boost[CSL_SERDES_MAX_DEBUG_COEFF_RANGE];
    uint32_t    ber[CSL_SERDES_MAX_DEBUG_COEFF_RANGE];
} csl_serdes_dbg_boost_att;

CSL_IDEF_INLINE void CSL_SERDES_WRITE_32_MASK(uint32_t addr, uint32_t mask, uint32_t write_data)
{
    uint32_t read_data, data;

    read_data = (*(uint32_t *) (addr));
    data = (write_data & ~mask ) | (read_data & mask);
    (*(uint32_t *) (addr)) = data;
}


/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_cycleDelay
 *
 *   @b Description
 *   @n CPU Delay.
 *
 *   @b Arguments   count
 *
 *   <b> Return Value  </b>
 *   @n  None 
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_cycleDelay(50);

    @endverbatim
 * ===========================================================================
 */       
CSL_IDEF_INLINE void CSL_SERDES_cycleDelay (uint64_t count)
{
   uint64_t sat = 0;

  if (count <= 0)
    return;

  sat = CSL_tscRead() + (uint64_t)count;

  while (CSL_tscRead() < sat);

}

/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_TBUS_DUMP_PHYA
 *
 *   @b Description
 *   @n This API is used for dumping the TBUS data for PHYA.
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>
 *   @n  tbus 
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_TBUS_DUMP_PHYA(CSL_HYPERLINK_0_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */  
CSL_IDEF_INLINE tbus CSL_SERDES_TBUS_DUMP_PHYA(uint32_t base_addr)
{

    tbus tbusdump;
    uint32_t i,j;
    uint32_t count = 0;

    for(i = 0; i < 5; i++)
    {
        for (j = 0; j < 31; j++)
        {
            tbusdump.taddr[count] = (i << 5) | j;
            CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | (tbusdump.taddr[count] << 24)));
            tbusdump.tbus_data[count] = (*(uint32_t *) (base_addr + 0x00EC));
            tbusdump.tbus_data[count] = (tbusdump.tbus_data[count] & 0xFF000000) >> 24;
            tbusdump.tbus_data[count] = tbusdump.tbus_data[count] | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
            count++;
        }
    }
    return tbusdump;
}



/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_TBUS_DUMP_PHYB
 *
 *   @b Description
 *   @n This API is used for dumping the TBUS data for PHYB.
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>
 *   @n  tbus 
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_TBUS_DUMP_PHYB(CSL_XGE_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */  
CSL_IDEF_INLINE tbus CSL_SERDES_TBUS_DUMP_PHYB(uint32_t base_addr)
{

    tbus tbusdump;
    uint32_t i,j;
    uint32_t count = 0;

    for(i = 0; i < 5; i++)
    {
        for (j = 0; j < 31; j++)
        {
            tbusdump.taddr[count] = (i << 8) | j;
            CSL_SERDES_WRITE_32_MASK(base_addr + 0x00FC, 0x0000FFFF, (0x00000000 | (tbusdump.taddr[count] << 16)));
            tbusdump.tbus_data[count] = (*(uint32_t *) (base_addr + 0x00F8));
            tbusdump.tbus_data[count] = (tbusdump.tbus_data[count] & 0xFFFF0000) >> 16;
            count++;
        }
    }
    return tbusdump;
}


/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_RESET_CMULANES_PHYA
 *
 *   @b Description
 *   @n This API is used to reset the CMU for PHYA.
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>
 *   @n  None 
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_RESET_CMULANES_PHYA(CSL_HYPERLINK_0_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */  
CSL_IDEF_INLINE void CSL_SERDES_RESET_CMULANES_PHYA(uint32_t base_addr)
{

    uint32_t tbus_data;
    uint32_t lane_rate = 0;

    /* Assert cmu reset pin */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0010, 0x0FFFFFFF, 0x40000000);
    /* Deassert cmu reset pin */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0010, 0x0FFFFFFF, 0x50000000);
    /* Wait for cmu ok signal */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | (4 << 24)));
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data & 0x008 )!= 0x008)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }


    /* Assert lane reset pin */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0228, 0xC0007FFF, 0x21300000 | (lane_rate << 16));
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0428, 0xC0007FFF, 0x21300000 | (lane_rate << 16));
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0628, 0xC0007FFF, 0x21300000 | (lane_rate << 16));
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0828, 0xC0007FFF, 0x21300000 | (lane_rate << 16));
    /* Deassert lane reset pin */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0228, 0xC0007FFF, 0x21308000 | (lane_rate << 16));
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0428, 0xC0007FFF, 0x21308000 | (lane_rate << 16));
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0628, 0xC0007FFF, 0x21308000 | (lane_rate << 16));
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0828, 0xC0007FFF, 0x21308000 | (lane_rate << 16));


    /* Wait for lane ok signal */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | (0x21 << 24)));
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data & 0x008 )!= 0x008)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }
      
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | (0x41 << 24)));
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data & 0x008 )!= 0x008)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }
      
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | (0x61 << 24)));
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data & 0x008 )!= 0x008)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }
      
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | (0x81 << 24)));
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data & 0x008 )!= 0x008)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }

}

/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_RESET_CMULANES_PHYB
 *
 *   @b Description
 *   @n This API is used to reset the CMU for PHYB.
 *
 *   @b Arguments   base_addr
 *
 *   <b> Return Value  </b>
 *   @n  None 
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_RESET_CMULANES_PHYB(CSL_XGE_SERDES_CFG_REGS);

    @endverbatim
 * ===========================================================================
 */  
CSL_IDEF_INLINE void CSL_SERDES_RESET_CMULANES_PHYB(uint32_t base_addr)
{

    uint32_t tbus_data;

    /* Assert cmu reset pin */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0010, 0x0FFFFFFF, 0x40000000);
    /* Deassert cmu reset pin */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0010, 0x0FFFFFFF, 0x50000000);
    /* Wait for cmu ok signal */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x00FC, 0x0000FFFF, (0x00000000 | (4 << 16)));
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
    tbus_data = (tbus_data & 0xFFFF0000) >> 16;

    while ((tbus_data & 0x008 )!= 0x008)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
        tbus_data = (tbus_data & 0xFFFF0000) >> 16;
      }


    /* Assert lane reset pin */
      CSL_SERDES_WRITE_32_MASK(base_addr + 0x0228, 0xDFFFFFFF, 0x20000000);
      CSL_SERDES_WRITE_32_MASK(base_addr + 0x0428, 0xDFFFFFFF, 0x20000000);

    /* Deassert lane reset pin */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0228, 0xDFFFFFFF, (base_addr + 0x0228));
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0428, 0xDFFFFFFF, (base_addr + 0x0428));

    while((*(uint32_t *)(base_addr + 0xA00 + 0x1F8) & 0x60000000) != 0x60000000);

}


/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_UDPCLOCKTEST_PHYA
 *
 *   @b Description
 *   @n This API is used as a UDP Clock Test for PHY-A.
 *
 *   @b Arguments   base_addr, lane_num
 *
 *   <b> Return Value  </b>
 *   @n  None 
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYA needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_UDPCLOCKTEST_PHYA(CSL_HYPERLINK_0_SERDES_CFG_REGS, 0);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE void CSL_SERDES_UDPCLOCKTEST_PHYA(uint32_t base_addr, uint32_t lane_num)
{

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0030, 0x0002FFFF, 0x83C80000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0x87FC0000, 0x00017C42);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0038, 0x00000000, 0xAAAAAAAA);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x003C, 0x00000000, 0x000000AA);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0040, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0044, 0xF0000000, 0x00000000);
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFFDF, 0x00000020);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0000, 0xFF8FFF3F, 0x00000080);

    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFF7F, 0x00000080);
}

/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_UDPCLOCKTEST_PHYB
 *
 *   @b Description
 *   @n This API is used as a UDP Clock Test for PHY-B.
 *
 *   @b Arguments   base_addr, lane_num
 *
 *   <b> Return Value  </b>
 *   @n  None 
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYB needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_UDPCLOCKTEST_PHYB(CSL_XGE_SERDES_CFG_REGS, 0);

    @endverbatim
 * ===========================================================================
 */ 
CSL_IDEF_INLINE void CSL_SERDES_UDPCLOCKTEST_PHYB(uint32_t base_addr, uint32_t lane_num)
{

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0030, 0x0002FFFF, 0x83C80000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0x87FC0000, 0x00017C42);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0038, 0x00000000, 0xAAAAAAAA);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x003C, 0x00000000, 0x000000AA);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0040, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0044, 0xF0000000, 0x00000000);
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFFDF, 0x00000020);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0000, 0xFF8FFF3F, 0x00000080);

    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFF7F, 0x00000080);
}


/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_BERTEST_PHYA
 *
 *   @b Description
 *   @n This API is used for measuring the SERDES BER for PHY-A.
 *
 *   @b Arguments   base_addr, lane_num, testtime
 *
 *   <b> Return Value  </b>
 *   @n  uint32_t 
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYA API needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_BERTEST_PHYA(CSL_HYPERLINK_0_SERDES_CFG_REGS, 0, 160);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE uint32_t CSL_SERDES_BERTEST_PHYA(uint32_t base_addr, uint32_t lane_num, uint64_t testtime)
{

    uint32_t bist_chk_errors;
    uint32_t tbus_data;
    //tbus tbusdump;

    /* 2. Set the following registers for the transmit pattern generator: */
    /* bist_gen_cdn Lane 0x34 [5] 0x0 */
    /* bist_tx_clock_enable Lane 0x32 [6] 0x1 */
    /* bist_rx_clock_enable Lane 0x32 [7] 0x1 */
    /* bist_chk_data_mode Lane 0x32 [2] 0x1 */
    /* bist_gen_mode8b Lane 0x32 [0] 0x0 */
    /* bist_gen_word Lane 0x34 [6] 0x1 */
    /* bist_gen_en Lane 0x34 [7] 0x0 */
    /* bist_gen_send_pream Lane 0x37 [3] 0x0 */
    /* bist_chk_lfsr_length Lane 0x32 [5:3] 0x1 */
    /* bist_gen_inv_prbs Comlane 0x17 [1] 0x0 */
    /* bist_chk_inv_prbs Comlane 0x17 [0] 0x0 */
    /* bist_chk_pream0 Lane 0x34 [1:0], 0x33 [7:0] 0x283 */
    /* bist_chk_pream1 Lane 0x36 [1:0], 0x35 [7:0] 0x17C */
    /* bist_chk_udp Lane 0x3C-0x38 0x0 */
    /* bist_gen_en_low Lane 0x42-0x43 0x0 */
    /* bist_gen_en_high Lane 0x44-0x45 0x0 */
    /* bist_chk_insert_word Lane 0x41-0x3D 0x0 */
    /* bist_chk_insert_length Lane 0x34 [4:2] 0x0 */
    /* bist_gen_insert_count Lane 0x37 [6:4] 0x0 */
    /* bist_gen_insert_delay Lane 0x47 [3:0], 0x46 [7:0] 0x0 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0030, 0x0002FFFF, 0x83CC0000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0x87FC0000, 0x00017C42);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0038, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x003C, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0040, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0044, 0xF0000000, 0x00000000);

    /* 3. Wait a minimum of 20ns, then deassert bist_gen_cdn */
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFFDF, 0x00000020);

    /* 5. Set the following for each lane: */
    /* dmux_txa_sel Lane 0x00 [7:6] 0x2 */
    /* dmux_txb_sel Lane 0x02 [6:4] 0x0 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0000, 0xFF8FFF3F, 0x00000080);

    /* 6. Wait a minimum of 20ns, then assert bist_gen_en */
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFF7F, 0x00000080);

    /* 7. Wait a minimum of 2us for the analog transmit and receive paths to obtain bit lock on to the training */
    /* pattern. While this is occurring, set: */
    /* Register Location Address Value */
    /* bchk_en Lane 0x04 [3] 0x0 */
    /* bchk_clr Lane 0x04 [4] 0x1 */
    /* bchk_src Lane 0x04 [6:5] 0x2 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFF87, 0x00000050);
    
    /* wait for ln_stat[3:2] to go high */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | ( (((lane_num+1) << 5)+0x02) << 24)));

    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data & 0x060 )!= 0x060)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }
    
    CSL_SERDES_cycleDelay(300000);

    /* 8.  Wait an additional 20ns and then deassert bchk_clr by setting a 0 to lane 004 bit 4 */
    CSL_SERDES_cycleDelay(50);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFEF, 0x00000000);

    /* 9. Assert bchk_en by setting bit 3 of LANE 004 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFF7, 0x00000008);

    /* 10. The PHY will indicate that it has successfully detected PRBS when bist_chk_synch is asserted (read bit 10 of tbus data at address 0c) */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | ( (((lane_num+1) << 5)+0xC) << 24)));
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data & 0x400 )!= 0x400)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }


    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFEF, 0x00000080);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFEF, 0x00000000);

    /* 11. Waiting 320us to measure BER */
    CSL_SERDES_cycleDelay(testtime);

    /* 12.  Deassert bchk_en by clearing bit 3 of LANE 004 and then bist_gen_en by clearing bit 7 of lane 34 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFF7, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFF7F, 0x00000000);

    /* 13. After 50ns check bist_chk_synch */
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | ( (((lane_num+1) << 5)+0xC) << 24)));
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
    while ((tbus_data | 0xBFF ) != 0xBFF)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
        tbus_data = (tbus_data & 0xFF000000) >> 24;
        tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);
      }

    /* 14. Veriy that bist_chk_errors is 0; tbus address is 0x0C and 0x0D, bit locations are */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | ( (((lane_num+1) << 5)+0xC) << 24)));
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);

    bist_chk_errors = (tbus_data & 0x3FF) << 6;

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0008, 0x00FFFFFF, (0x00000000 | ( (((lane_num+1) << 5)+0xD) << 24)));
    tbus_data = (*(uint32_t *) (base_addr + 0x00EC));
    tbus_data = (tbus_data & 0xFF000000) >> 24;
    tbus_data = tbus_data | (((*(uint32_t *) (base_addr + 0x00FC)) >> 16) & 0x00000F00);

    bist_chk_errors = bist_chk_errors + ((tbus_data & 0xFC0) >> 6);

    /* 15. To return to normal mode set dmux_txa_sel and dmux_txb_sel to 0 by setting bits 7 of lane 0 and 20-22 of lane 0 repsectively */
    /* dmux_txa_sel Lane 0x00 [7:6] 0x0 */
    /* dmux_txb_sel Lane 0x02 [6:4] 0x0 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0000, 0xFF8FFF3F, 0x00000000);

    return bist_chk_errors;
}

/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_BERTEST_PHYB
 *
 *   @b Description
 *   @n This API is used for measuring the SERDES BER for PHY-B.
 *
 *   @b Arguments   base_addr, lane_num, testtime
 *
 *   <b> Return Value  </b>
 *   @n  uint32_t 
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYB API needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_BERTEST_PHYB(CSL_XGE_SERDES_CFG_REGS, 0, 160);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE uint32_t CSL_SERDES_BERTEST_PHYB(uint32_t base_addr, uint32_t lane_num, uint64_t testtime)
{

    uint32_t bist_chk_errors;
    uint32_t bist_chk_errors_msb, bist_chk_errors_lsb;
    uint32_t tbus_data;
    //tbus tbusdump;

    /* 2. Set the following registers for the transmit pattern generator: */
    /* bist_gen_cdn Lane 0x34 [5] 0x0 */
    /* bist_tx_clock_enable Lane 0x32 [6] 0x1 */
    /* bist_rx_clock_enable Lane 0x32 [7] 0x1 */
    /* bist_chk_data_mode Lane 0x32 [2] 0x1 */
    /* bist_gen_mode8b Lane 0x32 [0] 0x0 */
    /* bist_gen_word Lane 0x34 [6] 0x1 */
    /* bist_gen_en Lane 0x34 [7] 0x0 */
    /* bist_gen_send_pream Lane 0x37 [3] 0x0 */
    /* bist_chk_lfsr_length Lane 0x32 [5:3] 0x1 */
    /* bist_gen_inv_prbs Comlane 0x17 [1] 0x0 */
    /* bist_chk_inv_prbs Comlane 0x17 [0] 0x0 */
    /* bist_chk_pream0 Lane 0x34 [1:0], 0x33 [7:0] 0x283 */
    /* bist_chk_pream1 Lane 0x36 [1:0], 0x35 [7:0] 0x17C */
    /* bist_chk_udp Lane 0x3C-0x38 0x0 */
    /* bist_gen_en_low Lane 0x42-0x43 0x0 */
    /* bist_gen_en_high Lane 0x44-0x45 0x0 */
    /* bist_chk_insert_word Lane 0x41-0x3D 0x0 */
    /* bist_chk_insert_length Lane 0x34 [4:2] 0x0 */
    /* bist_gen_insert_count Lane 0x37 [6:4] 0x0 */
    /* bist_gen_insert_delay Lane 0x47 [3:0], 0x46 [7:0] 0x0 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0030, 0x0000FFFF, 0x83DD0000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0x87FC0000, 0x00017C42);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0038, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x003C, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0040, 0x00000000, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0044, 0xF0000000, 0x00000000);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0xA00 + 0x14, 0xFCFFFFFF, 0x00000000);

    /* 3. Wait a minimum of 20ns, then deassert bist_gen_cdn */
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFFDF, 0x00000020);

    /* 5. Set the following for each lane: */
    /* dmux_txa_sel Lane 0x00 [7:6] 0x2 */
    /* dmux_txb_sel Lane 0x02 [6:4] 0x0 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0000, 0xFF8FFF3F, 0x00000080);

    /* 6. Wait a minimum of 20ns, then assert bist_gen_en */
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFF7F, 0x00000080);

    /* 7. Wait a minimum of 2us for the analog transmit and receive paths to obtain bit lock on to the training */
    /* pattern. While this is occurring, set: */
    /* Register Location Address Value */
    /* bchk_en Lane 0x04 [3] 0x0 */
    /* bchk_clr Lane 0x04 [4] 0x1 */
    /* bchk_src Lane 0x04 [6:5] 0x2 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFF87, 0x00000070);
    
    CSL_SERDES_cycleDelay(300000);

    /* 8.  Wait an additional 20ns and then deassert bchk_clr by setting a 0 to lane 004 bit 4 */
    CSL_SERDES_cycleDelay(50);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFEF, 0x00000000);

    /* 9. Assert bchk_en by setting bit 3 of LANE 004 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFF7, 0x00000008);

    /* 10. The PHY will indicate that it has successfully detected PRBS when bist_chk_synch is asserted (read bit 10 of tbus data at address 0c) */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x00FC, 0xF800FFFF, 0x020C0000);
    
    tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
    tbus_data = (tbus_data & 0xFFFF0000) >> 16;

    while ((tbus_data & 0x400 )!= 0x400)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
        tbus_data = (tbus_data & 0xFFFF0000) >> 16;
      }


    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFEF, 0x00000080);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFEF, 0x00000000);

    /* 11. Waiting 320us to measure BER */
    CSL_SERDES_cycleDelay(testtime);

    /* 12.  Deassert bchk_en by clearing bit 3 of LANE 004 and then bist_gen_en by clearing bit 7 of lane 34 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0004, 0xFFFFFFF7, 0x00000000);
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0034, 0xFFFFFF7F, 0x00000000);

    /* 13. After 50ns check bist_chk_synch */
    CSL_SERDES_cycleDelay(50);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x00FC, 0xF800FFFF, 0x020C0000);

    tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
    tbus_data = (tbus_data & 0xFFFF0000) >> 16;

    while ((tbus_data | 0xBFF ) != 0xBFF)
      {
        tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
        tbus_data = (tbus_data & 0xFFFF0000) >> 16;
      }

    /* 14. Veriy that bist_chk_errors is 0; tbus address is 0x0C and 0x0D, bit locations are */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x00FC, 0xF800FFFF, 0x020C0000);
    tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
    bist_chk_errors_msb = (tbus_data & 0x03FF0000) >> 10;

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x00FC, 0xF800FFFF, 0x020D0000);

    tbus_data = (*(uint32_t *) (base_addr + 0x00F8));
    bist_chk_errors_lsb = (tbus_data & 0x0FC00000) >> 22;

    bist_chk_errors = bist_chk_errors_msb | bist_chk_errors_lsb;

    /* 15. To return to normal mode set dmux_txa_sel and dmux_txb_sel to 0 by setting bits 7 of lane 0 and 20-22 of lane 0 repsectively */
    /* dmux_txa_sel Lane 0x00 [7:6] 0x0 */
    /* dmux_txb_sel Lane 0x02 [6:4] 0x0 */
    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0000, 0xFF8FFF7F, 0x00000000);

    return bist_chk_errors;
}


/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_SWEEP_CM_C1_PHYA
 *
 *   @b Description
 *   @n This API is used for sweeping the BER across C1, CM coefficients.
 *
 *   @b Arguments   *pDbgCMC1, base_addr, lane_num, testtime, C1coeff_start, C1coeff_end, CMcoeff_start, CMcoeff_end
 *
 *   <b> Return Value  </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYA API needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_SWEEP_CM_C1(&pDbgCMC1, CSL_HYPERLINK_0_SERDES_CFG_REGS, 0, 160, 6, 33, 0, 3);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_SERDES_SWEEP_CM_C1_PHYA(csl_serdes_dbg_cmc1 *pDbgCMC1, uint32_t base_addr, uint32_t lane_num, uint32_t testtime, uint32_t C1coeff_start, uint32_t C1coeff_end, uint32_t CMcoeff_start, uint32_t CMcoeff_end)
{
    uint32_t i,j, count = 0;

    for (i = C1coeff_start; i < C1coeff_end; i++)
    {
        /* set c1 to i */
        CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFFFFE0, (0x00000000 | i));
        for (j = CMcoeff_start; j < CMcoeff_end; j++)
        {
            /* set c2 to j */
            CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFF0FFF, (0x00000000 | (j << 12)));

            CSL_SERDES_RESET_CMULANES_PHYA(base_addr);

            pDbgCMC1->C1[count] = i;
            pDbgCMC1->CM[count] = j;
            pDbgCMC1->ber[count] = CSL_SERDES_BERTEST_PHYA(base_addr, lane_num, testtime);
            count++;
        }
    }
}

/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_SWEEP_CM_C1_PHYB
 *
 *   @b Description
 *   @n This API is used for sweeping the BER across C1, CM coefficients.
 *
 *   @b Arguments   *pDbgCMC1, base_addr, lane_num, testtime, C1coeff_start, C1coeff_end, CMcoeff_start, CMcoeff_end
 *
 *   <b> Return Value  </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYB API needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_SWEEP_CM_C1_PHYB(&pDbgCMC1, CSL_XGE_SERDES_CFG_REGS, 0, 160, 6, 33, 0, 3);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_SERDES_SWEEP_CM_C1_PHYB(csl_serdes_dbg_cmc1 *pDbgCMC1, uint32_t base_addr, uint32_t lane_num, uint32_t testtime, uint32_t C1coeff_start, uint32_t C1coeff_end, uint32_t CMcoeff_start, uint32_t CMcoeff_end)
{
    uint32_t i,j, count = 0;

    for (i = C1coeff_start; i < C1coeff_end; i++)
    {
        /* set c1 to i */
        CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFFFFE0, (0x00000000 | i));
        for (j = CMcoeff_start; j < CMcoeff_end; j++)
        {
            /* set c2 to j */
            CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFF0FFF, (0x00000000 | (j << 12)));

            CSL_SERDES_RESET_CMULANES_PHYB(base_addr);

            pDbgCMC1->C1[count] = i;
            pDbgCMC1->CM[count] = j;
            pDbgCMC1->ber[count] = CSL_SERDES_BERTEST_PHYB(base_addr, lane_num, testtime);
            count++;
        }
    }
}


/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_SWEEP_BOOST_ATT_PHYA
 *
 *   @b Description
 *   @n This API is used for sweeping the BER across Boost, Attenuation coefficients.
 *
 *   @b Arguments   *pDbgBstAtt, base_addr, lane_num, testtime, att_start, att_end, boost_start, boost_end
 *
 *   <b> Return Value  </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYA API needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
    CSL_SERDES_SWEEP_BOOST_ATT_PHYA(&pDbgBstAtt, CSL_HYPERLINK_0_SERDES_CFG_REGS, 0, 160, 5, 6, 7, 9);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_SERDES_SWEEP_BOOST_ATT_PHYA(csl_serdes_dbg_boost_att *pDbgBstAtt, uint32_t base_addr, uint32_t lane_num, uint32_t testtime, uint32_t att_start, uint32_t att_end, uint32_t boost_start, uint32_t boost_end)
{

    uint32_t i,j, count=0;

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0a84, 0xFFFFF8FF, 0x00000000);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFFFFE0, 0x00000009);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFF0FFF, 0x00000000);

    /* set att to i */
    for (i = att_start; i < att_end; i++)
     {
        CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x008C, 0xFFFFF0FF, (0x00000000 | (i << 8)));

        /* set rate3_boost_start to j */
        for (j = boost_start; j < boost_end; j++)
        {
                CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x008C, 0xFFFF0FFF, (0x00000000 | (j << 12)));

                CSL_SERDES_RESET_CMULANES_PHYA(base_addr);

                pDbgBstAtt->att[count] = i;
                pDbgBstAtt->boost[count] = j;
                pDbgBstAtt->ber[count] = CSL_SERDES_BERTEST_PHYA(base_addr, lane_num, testtime);
                count++;
                }
         }
}

/** @addtogroup CSL_SERDES_DEBUG
 @{ */
/** ============================================================================
 *   @n@b CSL_SERDES_SWEEP_BOOST_ATT_PHYB
 *
 *   @b Description
 *   @n This API is used for sweeping the BER across Boost, Attenuation coefficients.
 *
 *   @b Arguments   *pDbgBstAtt, base_addr, lane_num, testtime, att_start, att_end, boost_start, boost_end
 *
 *   <b> Return Value  </b>
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  CSL_SERDES_RESET_CMULANES_PHYB API needs to be called before this API is called.
 *
 *   <b> Post Condition </b>
 *   @n  None
 *
 *   @b Reads
 *   @n None 
 *
 *  <b> Usage Constraints: </b>
 *  @n  None
 *
 *   @b Example
 *   @verbatim
 
     CSL_SERDES_SWEEP_BOOST_ATT_PHYB(&pDbgBstAtt, CSL_XGE_SERDES_CFG_REGS, 0, 160, 5, 6, 7, 9);

    @endverbatim
 * ===========================================================================
 */
CSL_IDEF_INLINE void CSL_SERDES_SWEEP_BOOST_ATT_PHYB(csl_serdes_dbg_boost_att *pDbgBstAtt, uint32_t base_addr, uint32_t lane_num, uint32_t testtime, uint32_t att_start, uint32_t att_end, uint32_t boost_start, uint32_t boost_end)
{

    uint32_t i,j, count=0;

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x0a84, 0xFFFFF8FF, 0x00000000);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFFFFE0, 0x00000009);

    CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x0008, 0xFFFF0FFF, 0x00000000);

    /* set att to i */
    for (i = att_start; i < att_end; i++)
     {
        CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x008C, 0xFFFFF0FF, (0x00000000 | (i << 8)));

        /* set rate3_boost_start to j */
        for (j = boost_start; j < boost_end; j++)
        {
                CSL_SERDES_WRITE_32_MASK(base_addr + 0x200*(lane_num + 1) + 0x008C, 0xFFFF0FFF, (0x00000000 | (j << 12)));

                CSL_SERDES_RESET_CMULANES_PHYB(base_addr);

                pDbgBstAtt->att[count] = i;
                pDbgBstAtt->boost[count] = j;
                pDbgBstAtt->ber[count] = CSL_SERDES_BERTEST_PHYB(base_addr, lane_num, testtime);
                count++;
                }
         }
}


/* @} */

