/******************************************************************************
 * FILE PURPOSE:  SA LLD Common Interface APIs
 ******************************************************************************
 * FILE NAME:   salldcom.c  
 *
 * DESCRIPTION: SA LLD Common Interface API implementations 
 *
 * FUNCTION               DESCRIPTION
 * ========               ===========
 *
 * REVISION HISTORY:
 *
 * (C) Copyright 2009-2012, Texas Instruments Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* System level header files */
#include <stdlib.h>
#include <string.h>

/* SALLD header files */
#include <ti/drv/sa/salld.h>
#include <ti/drv/sa/sa_osal.h>
#include <ti/csl/cslr_cp_ace.h>
//#include <ti/csl/cslr_device.h>
#include "src/salldloc.h"

/* Define SA related base address */
/* SA control register */
//static CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)CSL_PA_SS_CFG_CP_ACE_CFG_REGS;

/* Global Variable which describes the SA LLD Version Information */
const char sa_lld_version_str[] = SA_LLD_VERSION_STR ":" __DATE__  ":" __TIME__;

/* SA PHP System Statistics */
#define SA_SYS_STATS_BASE             0x200

/* Offsets to system statistics block */
#define SA_SYS_STATS_ERR_OFFSET         0x00     /* PHP 0 and 1 */
#define SA_SYS_STATS_ESP_OFFSET         0x40     /* PHP 0 only */
#define SA_SYS_STATS_AH_OFFSET          0x80     /* PHP 0 only */
#define SA_SYS_STATS_SRTP_OFFSET        0x40     /* PHP 1 only */
#define SA_SYS_STATS_AC_OFFSET          0x80     /* PHP 1 only */

/* System Error Statistic */
/* Packet Instance Buffers not available */
#define SA_SYS_ERR_NOMEM_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_ERR_OFFSET + 0)/4)  
/* Context Cache related errors */     
#define SA_SYS_ERR_CTX_OFFSET         ((SA_SYS_STATS_BASE + SA_SYS_STATS_ERR_OFFSET + 4)/4)       
/* SA processing engine related error */
#define SA_SYS_ERR_ENGINE_OFFSET      ((SA_SYS_STATS_BASE + SA_SYS_STATS_ERR_OFFSET + 8)/4)  
/* Packet Procotol related error such as Invalid IP types */     
#define SA_SYS_ERR_PROTO_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_ERR_OFFSET + 12)/4) 

/* IPSEC ESP system statistics */
/* Number of replay failures with old packets */
#define SA_SYS_ESP_REPLAY_OLD_OFFSET  ((SA_SYS_STATS_BASE + SA_SYS_STATS_ESP_OFFSET + 0)/4)
/* Number of replay failures with duplicated packets */
#define SA_SYS_ESP_REPLAY_DUP_OFFSET  ((SA_SYS_STATS_BASE + SA_SYS_STATS_ESP_OFFSET + 4)/4)
/* Number of Authentication failure */
#define SA_SYS_ESP_AUTH_FAIL_OFFSET   ((SA_SYS_STATS_BASE + SA_SYS_STATS_ESP_OFFSET + 8)/4)
/* Total Number of Packets encrypted */ 
#define SA_SYS_ESP_ENC_HI_OFFSET      ((SA_SYS_STATS_BASE + SA_SYS_STATS_ESP_OFFSET + 12)/4)
#define SA_SYS_ESP_ENC_LO_OFFSET      ((SA_SYS_STATS_BASE + SA_SYS_STATS_ESP_OFFSET + 16)/4)
/* Total Number of Packets decrypted */ 
#define SA_SYS_ESP_DEC_HI_OFFSET      ((SA_SYS_STATS_BASE + SA_SYS_STATS_ESP_OFFSET + 20)/4)
#define SA_SYS_ESP_DEC_LO_OFFSET      ((SA_SYS_STATS_BASE + SA_SYS_STATS_ESP_OFFSET + 24)/4)

#define SA_SYS_ESP_OFFSET   SA_SYS_ESP_REPLAY_OLD_OFFSET  


/* IPSEC AH system statistics */
/* Number of replay failures with old packets */
#define SA_SYS_AH_REPLAY_OLD_OFFSET   ((SA_SYS_STATS_BASE + SA_SYS_STATS_AH_OFFSET + 0)/4)
/* Number of replay failures with duplicated packets */
#define SA_SYS_AH_REPLAY_DUP_OFFSET   ((SA_SYS_STATS_BASE + SA_SYS_STATS_AH_OFFSET + 4)/4)
/* Number of Authentication failure */
#define SA_SYS_AH_AUTH_FAIL_OFFSET    ((SA_SYS_STATS_BASE + SA_SYS_STATS_AH_OFFSET + 8)/4)
/* Total Number of Packets encrypted */ 
#define SA_SYS_AH_ENC_HI_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AH_OFFSET + 12)/4)
#define SA_SYS_AH_ENC_LO_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AH_OFFSET + 16)/4)
/* Total Number of Packets decrypted */ 
#define SA_SYS_AH_DEC_HI_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AH_OFFSET + 20)/4)
#define SA_SYS_AH_DEC_LO_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AH_OFFSET + 24)/4)

#define SA_SYS_AH_OFFSET   SA_SYS_AH_REPLAY_OLD_OFFSET


/* SRTP system statistics */
/* Number of replay failures with old packets */
#define SA_SYS_SRTP_REPLAY_OLD_OFFSET ((SA_SYS_STATS_BASE + SA_SYS_STATS_SRTP_OFFSET + 0)/4)
/* Number of replay failures with duplicated packets */
#define SA_SYS_SRTP_REPLAY_DUP_OFFSET ((SA_SYS_STATS_BASE + SA_SYS_STATS_SRTP_OFFSET + 4)/4)
/* Number of Authentication failure */
#define SA_SYS_SRTP_AUTH_FAIL_OFFSET  ((SA_SYS_STATS_BASE + SA_SYS_STATS_SRTP_OFFSET + 8)/4)
/* Total Number of Packets encrypted */ 
#define SA_SYS_SRTP_ENC_HI_OFFSET     ((SA_SYS_STATS_BASE + SA_SYS_STATS_SRTP_OFFSET + 12)/4)
#define SA_SYS_SRTP_ENC_LO_OFFSET     ((SA_SYS_STATS_BASE + SA_SYS_STATS_SRTP_OFFSET + 16)/4)
/* Total Number of Packets decrypted */ 
#define SA_SYS_SRTP_DEC_HI_OFFSET     ((SA_SYS_STATS_BASE + SA_SYS_STATS_SRTP_OFFSET + 20)/4)
#define SA_SYS_SRTP_DEC_LO_OFFSET     ((SA_SYS_STATS_BASE + SA_SYS_STATS_SRTP_OFFSET + 24)/4)

#define SA_SYS_SRTP_OFFSET   SA_SYS_SRTP_REPLAY_OLD_OFFSET


/* Air Ciphering system statistics */
/* Number of Authentication failure */
#define SA_SYS_AC_AUTH_FAIL_OFFSET    ((SA_SYS_STATS_BASE + SA_SYS_STATS_AC_OFFSET + 0)/4)
/* Total Number of Packets encrypted */ 
#define SA_SYS_AC_ENC_HI_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AC_OFFSET + 4)/4)
#define SA_SYS_AC_ENC_LO_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AC_OFFSET + 8)/4)
/* Total Number of Packets decrypted */ 
#define SA_SYS_AC_DEC_HI_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AC_OFFSET + 12)/4)
#define SA_SYS_AC_DEC_LO_OFFSET       ((SA_SYS_STATS_BASE + SA_SYS_STATS_AC_OFFSET + 16)/4)

#define SA_SYS_AC_OFFSET   SA_SYS_AC_AUTH_FAIL_OFFSET


/* SA PHP PDSP Version Number */ 
#define SA_PHP_VERSION_BASE           0x2f0
    
#define SA_PHP_VERSION_OFFSET         ((SA_PHP_VERSION_BASE + 0)/4)  

/* SA TRNG related definitions */
#define SA_RNG_STARTUP_CYCLES_DEFAULT     (1 << 8) 
#define SA_RNG_STARTUP_CYCLES_MIN         (1 << 8)
#define SA_RNG_STARTUP_CYCLES_MAX         (1 << 24)
#define SA_RNG_STARTUP_CYCLES_SHIFT       8 

#define SA_RNG_MIN_REFILL_CYCLES_DEFAULT  (1 << 6)     
#define SA_RNG_MIN_REFILL_CYCLES_MIN      (1 << 6)
#define SA_RNG_MIN_REFILL_CYCLES_MAX      (1 << 14)
#define SA_RNG_MIN_REFILL_CYCLES_SHIFT    6

#define SA_RNG_MAX_REFILL_CYCLES_DEFAULT  (1 << 24) 
#define SA_RNG_MAX_REFILL_CYCLES_MIN      (1 << 8)
#define SA_RNG_MAX_REFILL_CYCLES_MAX      (1 << 24)
#define SA_RNG_MAX_REFILL_CYCLES_SHIFT    8

#define SA_RNG_CLOCK_DIVIDER_DEFAULT       1    
#define SA_RNG_CLOCK_DIVIDER_MIN           1
#define SA_RNG_CLOCK_DIVIDER_MAX          16 
#define SA_RNG_CLOCK_DIVIDER_SHIFT         0

/* SA PKA related definitions */
#define SA_PKA_RAM_BASE                  0x00022000
#define SA_PKA_AINDEX                    0
#define SA_PKA_BINDEX                    1
#define SA_PKA_CINDEX                    2
#define SA_PKA_DINDEX                    3

/* PKA A, B, C, B offsets in 32-bit words */
#define SA_PKA_A_OFFSET                  0x00
#define SA_PKA_B_OFFSET                  0x80
#define SA_PKA_C_OFFSET                  0x100
#define SA_PKA_D_OFFSET                  0x180

/* PKA A, B, C, B base in bytes */
#define SA_PKA_A_BASE                    (SA_PKA_RAM_BASE + (SA_PKA_A_OFFSET << 2))
#define SA_PKA_B_BASE                    (SA_PKA_RAM_BASE + (SA_PKA_B_OFFSET << 2))
#define SA_PKA_C_BASE                    (SA_PKA_RAM_BASE + (SA_PKA_C_OFFSET << 2))
#define SA_PKA_D_BASE                    (SA_PKA_RAM_BASE + (SA_PKA_D_OFFSET << 2))

/***********************************************************************************************
 * FUNCTION PURPOSE: Convert SA TRNG cycle parameters
 ***********************************************************************************************
 * DESCRIPTION: Convert SA TRNG cycle parameters based on min, max, default and divider
 ***********************************************************************************************/
static uint16_t sap_rng_conv_cycle_params(uint32_t param, uint32_t max, uint32_t min, 
                                          uint32_t default_value, uint16_t shift)
{
    if (param == 0)
    {
        param = default_value;
    }
    else if (param > max)
    {
        param = max;
    }
    else if (param < min)
    {
        param = min;
    }

    return ((uint16_t)(param >> shift));
}


/***********************************************************************************************
 * FUNCTION PURPOSE: Zero check
 ***********************************************************************************************
 * DESCRIPTION: Verify whether a PKA input vector is zero
 *
 * Return: TRUE: zero
 *         FALSE: non-zero
 *          
 ***********************************************************************************************/
static int sap_pka_is_zero(uint32_t* data, uint16_t size) 
{
    int i;
    
    for ( i = 0; i < size; i++)
    {
        if(data[i])
            return(FALSE);
    }
    return(TRUE);
}


/******************************************************************************
 * FUNCTION PURPOSE:  Obtains memory buffer requirements for an SALLD
 *                    instance
 ******************************************************************************
 * DESCRIPTION: This function obtains memory buffer requirements for an SALLD
 *         instance. the memory buffer requirements are in terms of the size and 
 *         alignment array which must not be changed by external software.  
 *         Worst case memory size requirements are computed using parameters 
 *         provided by Sa_SizeCfg_t. 
 *
 * int16_t  Sa_getBufferReq (
 *              Sa_SizeCfg_t *sizeCfg,   - configuration information to be used in
 *                                         estimating the worst case buffer sizes
 *              int          *sizes      - size requirement array
 *              int          *aligns)    - alignemnt requirement array  
 *
 *  Assumptions:
 *      --All the memory is from heap.
 *      --There is no special requirement for memory.
 *****************************************************************************/
int16_t  Sa_getBufferReq (Sa_SizeCfg_t *sizeCfg, int sizes[], int aligns[])
{
  int alignment = max(8, sizeCfg->cacheLineSize);

  sizes[0] = SALLD_ROUND_UP(sizeof(salldObj_t), alignment);
  if (sizeCfg->ctrlBitMap & sa_SIZE_CONFIG_CREATE_SHADOW_INST)
  {
    sizes[0] *= 2;  /* Double the instance size to include the shadow */
  }
  aligns[0] = alignment;
  
  return(sa_ERR_OK); 
}  /* Sa_getBufferReq */

#define SA_CSL_SASS_ENABLE_ALL      (CSL_CP_ACE_CMD_STATUS_ENCSS_EN_MASK    | \
                                     CSL_CP_ACE_CMD_STATUS_AUTHSS_EN_MASK   | \
                                     CSL_CP_ACE_CMD_STATUS_AIRSS_EN_MASK    | \
                                     CSL_CP_ACE_CMD_STATUS_TRNG_EN_MASK     | \
                                     CSL_CP_ACE_CMD_STATUS_PHP1SS_EN_MASK   | \
                                     CSL_CP_ACE_CMD_STATUS_PHP2SS_EN_MASK   | \
                                     CSL_CP_ACE_CMD_STATUS_CTXCACH_EN_MASK  | \
                                     CSL_CP_ACE_CMD_STATUS_PA_IN_PORT_EN_MASK       | \
                                     CSL_CP_ACE_CMD_STATUS_CDMA_IN_PORT_EN_MASK     | \
                                     CSL_CP_ACE_CMD_STATUS_PA_OUT_PORT_EN_MASK      | \
                                     CSL_CP_ACE_CMD_STATUS_CDMA_OUT_PORT_EN_MASK      \
                                     )

/******************************************************************************
 * FUNCTION PURPOSE: Create and initialize an SALLD instance 
 ******************************************************************************
 * DESCRIPTION: This function initialize an SALLD instance and its
 *         corresponding instance structure based on stystem configuration data 
 *         such as the the call-out table and etc.
 *
 * int16_t Sa_create (
 *          Sa_Config_t *cfg,      - a pointer to configuration structure
 *          void*        bases,    - Array of the memory buffer base addresses
 *          Sa_Handle   *pHandle)  - store Instance handle which is the pointer
 *                                   to an initialized instance structures
 *
 * Return values:  sa_ERR_OK
 *                 sa_ERR_GEN
 *                 sa_ERR_PARAMS
 *                 sa_ERR_NOMEM 
 *                 sa_ERR_INV_BUF
 *                 sa_ERR_INV_PROTO_TYPE 
 *
 *****************************************************************************/
int16_t Sa_create (Sa_Config_t *cfg, void* bases[], Sa_Handle *pHandle)
{
  int  i;
  int  instSize, alignment;
  Bool fShadowInst;
  
  salldObj_t *inst = (salldObj_t *)bases[SALLD_BUFN];
  salldObj_t *shadowInst = NULL;
  CSL_Cp_aceRegs*  pSaRegs;

  /* Test instance pointer */
  if (pHandle == NULL || inst == NULL || cfg == NULL || cfg->sizeConfig == NULL)
    return(sa_ERR_PARAMS);
  
  alignment = max(8, cfg->sizeConfig->cacheLineSize);
  instSize = SALLD_ROUND_UP(sizeof(salldObj_t), alignment);  
    
  if (cfg->sizeConfig->ctrlBitMap & sa_SIZE_CONFIG_CREATE_SHADOW_INST)
  {
    fShadowInst = TRUE;
    instSize += instSize; /* Double instance size for shadow instance */ 
  } 
  else
  {
    fShadowInst = FALSE;
  }
    
  /* Invalidate the Cache Contents */
  Sa_osalBeginMemAccess(inst, instSize);  
  memset(inst, 0, sizeof(salldObj_t));
  
  /* Allocate all dynamic buffers (base address != NULL ?)   */
  for (i = 0; i < SALLD_NBUF; i++) {
    if (bases[i] == NULL) {
      return (sa_ERR_NOMEM);
    }
    inst->memBaseOffsets[i] = sa_CONV_ADDR_TO_OFFSET(cfg->instPoolBaseAddr, bases[i]);  
  }

  /* Verify all mandatory call-out APIs are implemented */
  if ((cfg->callTable == NULL)                    ||
      (cfg->callTable->ScAlloc == NULL)           ||
      (cfg->callTable->ScFree == NULL)            ||
      (cfg->callTable->ChanRegister == NULL)      ||
      (cfg->callTable->ChanUnRegister == NULL)    ||
      (cfg->callTable->ChanSendNullPkt == NULL))  {
  
    return(sa_ERR_PARAMS);   
  }   

  /* Default Instance Initialization */
  inst->magic          = SALLD_INST_MAGIC_WORD;
  inst->ID             = cfg->ID;
  salldLObj.baseAddr   = (void *) cfg->baseAddr;

  inst->stateBitfield  = 0;    /* clear all states  */
  SALLD_SET_STATE_ENABLE(inst, 1);
  /* Record the call-out table */
  SALLD_SET_LOC_INST_READY(&salldLObj, 1);
  salldLObj.callOutFuncs = *cfg->callTable;
  /* Copy special security context buffer pointer if necessary */
  salldLObj.intBuf = cfg->intBuf;
  
  /* Return the instance handle */
  *pHandle = (Sa_Handle)sa_CONV_ADDR_TO_OFFSET(cfg->instPoolBaseAddr, inst);

  /* Record the base addresses in the local object */
  salldLObj.instPoolBaseAddr = cfg->instPoolBaseAddr;
  salldLObj.scPoolBaseAddr   = cfg->scPoolBaseAddr;
  
  /* Enable all the sud-modules expect PKA in the SA */
  pSaRegs = (CSL_Cp_aceRegs*)cfg->baseAddr;
  pSaRegs->MMR.CMD_STATUS |= SA_CSL_SASS_ENABLE_ALL;

  /* Populate the shadow Instance which will be used at another processer with different Endian mode if required */
  if (fShadowInst)
  {
    /* Shadow instance is created just after system instance in the memory
       hence the shadowoffset = system offset + size of system structure */
    inst->shadowInstOffset = (uint32_t) inst->memBaseOffsets[SALLD_BUFN] + sizeof (salldObj_t);         
    shadowInst = (salldObj_t *)(sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, inst->shadowInstOffset));		
    
    memset(shadowInst, 0, sizeof(salldObj_t));
    shadowInst->magic = salld_swiz32(SALLD_INST_MAGIC_WORD);
    shadowInst->ID = SALLD_SWIZ(inst->ID);
    shadowInst->stateBitfield = SALLD_SWIZ(inst->stateBitfield);
    shadowInst->shadowInstOffset = SALLD_SWIZ(inst->shadowInstOffset);
    
    /* Note: the local instance can not be used by the processors in different endian mode */
    /* The callout table can not be shared among ARM and DSP cores */
  }
  
  /* Once the SA LLD has been created we need to writeback the contents of its instance data */ 
  Sa_osalEndMemAccess(inst, instSize);  
  
  return(sa_ERR_OK);
} /*  Sa_create  */

/******************************************************************************
 * FUNCTION PURPOSE: Activate the SALLD local instance 
 ******************************************************************************
 * DESCRIPTION: This function activates the SALLD local instance. This function 
 *              should be called once at each core which shares the SA LLD handle
 *              with the master core where the SA LLD instance was created.  
 *
 * int16_t Sa_start (
 *
 *          Sa_Handle handle,      - SALLD channel identifier
 *          Sa_Config_t *cfg)      - a pointer to configuration structure
 *
 * Return values:  sa_ERR_OK
 *                 sa_ERR_GEN
 *                 sa_ERR_PARAMS
 *
 *****************************************************************************/
int16_t Sa_start (Sa_Handle handle, Sa_Config_t *cfg)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(cfg->instPoolBaseAddr, handle);
  uint32_t mtCsKey;
  int16_t ret = sa_ERR_OK;

  /* Test instance pointer */
  if (inst == NULL || cfg == NULL)
    return(sa_ERR_PARAMS);
    
  /* Verify all mandatory call-out APIs are implemented */
  if ((cfg->callTable == NULL)                    ||
      (cfg->callTable->ScAlloc == NULL)           ||
      (cfg->callTable->ScFree == NULL)            ||
      (cfg->callTable->ChanRegister == NULL)      ||
      (cfg->callTable->ChanUnRegister == NULL)    ||
      (cfg->callTable->ChanSendNullPkt == NULL))  {
  
    return(sa_ERR_PARAMS);   
  }   
  /* Record the call-out table */
  /* CRITICAL Section Start: The global instance is a shared resource which needs to be
   * protected from the following:
   *  a) Multiple Cores acccess
   */
  Sa_osalMtCsEnter(&mtCsKey);
      
  /* Invalidate the Cache Contents */
  Sa_osalBeginMemAccess(inst, sizeof(salldObj_t));
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        ret = sa_ERR_INV_ENDIAN_MODE;
    else
        ret = sa_ERR_INV_HANDLE;    
  }  
  else
  {  
    salldLObj.callOutFuncs = *cfg->callTable;
    /* Copy special security context buffer pointer if necessary */
    salldLObj.intBuf = cfg->intBuf;	

	/* Record the base addresses in the local object */
    salldLObj.instPoolBaseAddr = cfg->instPoolBaseAddr;
    salldLObj.scPoolBaseAddr   = cfg->scPoolBaseAddr;
    salldLObj.baseAddr         = (void *) cfg->baseAddr;	
  }  
  
  /* Writeback the instance updates */
  Sa_osalEndMemAccess(inst, sizeof(salldObj_t));  
  
  /* Critical Section End */
  Sa_osalMtCsExit(mtCsKey);
  
  return(ret);
} /*  Sa_start  */


/******************************************************************************
 * FUNCTION PURPOSE:  Delete an SALLD instance 
 ******************************************************************************
 * DESCRIPTION: This function clears the SALLD instance and returns the base  
 *         address of all buffers previously allocated for the SA LLD instance.
 *
 *  int16_t  Sa_close  (
 *              Sa_Handle handle,      - SALLD channel identifier
 *              void*     bases[])     - Output array of memory buffer base addresses
 *
 * Return values:  sa_ERR_OK
 *
 ******************************************************************************/
int16_t  Sa_close  (Sa_Handle handle, void* bases[])
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  int i;
  salldObj_t *shadowInst = NULL;

  /* Non zero shadow instance offset indicates the presense of shadow instance */
  if (inst->shadowInstOffset)
  	 shadowInst = (salldObj_t *)(sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, inst->shadowInstOffset));  
  
  /* Invalidate the Cache Contents */
  Sa_osalBeginMemAccess(inst, sizeof(salldObj_t));  
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  inst->stateBitfield = 0;  /* indicate SALLD instance is closed */
  inst->magic = 0;
  
  if (shadowInst)
  {
    shadowInst->magic = 0;
  }
  
  for (i = 0; i < SALLD_NBUF; i++) {
    bases[i] = (void *) sa_CONV_OFFSET_TO_ADDR (salldLObj.instPoolBaseAddr, inst->memBaseOffsets[i]);  
  }
  
  /* Writeback the instance updates */
  Sa_osalEndMemAccess(inst, sizeof(salldObj_t));  
  
  return(sa_ERR_OK);  

} /* Sa_close */

/******************************************************************************
 * FUNCTION PURPOSE:  SALLD Get Sysyem Stats 
 ******************************************************************************
 * DESCRIPTION: This function obtains SALLD system statistics
 *
 *  int16_t  Sa_getSysStats (
 *   Sa_Handle        handle      - SALLD instance handle
 *   Sa_SysStats_t    *stats)     - a pointer to system statistics
 *
 * Return values:  sa_ERR_OK
 *                 sa_ERR_PARAMS
 *
 ******************************************************************************/
int16_t Sa_getSysStats(Sa_Handle handle, Sa_SysStats_t *stats) 
{

  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  

  if (!stats)
  {
    return(sa_ERR_PARAMS);
  }
  
  stats->err.errNoMem  = pSaRegs->PDSP_SRAM[0].DATA[SA_SYS_ERR_NOMEM_OFFSET] +
                         pSaRegs->PDSP_SRAM[1].DATA[SA_SYS_ERR_NOMEM_OFFSET];
  stats->err.errCtx    = pSaRegs->PDSP_SRAM[0].DATA[SA_SYS_ERR_CTX_OFFSET] +
                         pSaRegs->PDSP_SRAM[1].DATA[SA_SYS_ERR_CTX_OFFSET];
  stats->err.errEngine = pSaRegs->PDSP_SRAM[0].DATA[SA_SYS_ERR_ENGINE_OFFSET] +
                         pSaRegs->PDSP_SRAM[1].DATA[SA_SYS_ERR_ENGINE_OFFSET];
  stats->err.errProto  = pSaRegs->PDSP_SRAM[0].DATA[SA_SYS_ERR_PROTO_OFFSET] +
                         pSaRegs->PDSP_SRAM[1].DATA[SA_SYS_ERR_PROTO_OFFSET];
                         
  memcpy(&stats->esp, (void *)&pSaRegs->PDSP_SRAM[0].DATA[SA_SYS_ESP_OFFSET], sizeof(Sa_IpsecSysStats_t));                       
  memcpy(&stats->ah,  (void *)&pSaRegs->PDSP_SRAM[0].DATA[SA_SYS_AH_OFFSET], sizeof(Sa_IpsecSysStats_t));                       
  memcpy(&stats->srtp,(void *)&pSaRegs->PDSP_SRAM[1].DATA[SA_SYS_SRTP_OFFSET], sizeof(Sa_SrtpSysStats_t));                       
  memcpy(&stats->ac,  (void *)&pSaRegs->PDSP_SRAM[1].DATA[SA_SYS_AC_OFFSET], sizeof(Sa_AcSysStats_t));                       
                     
  return(sa_ERR_OK);  
}

/******************************************************************************
 * FUNCTION PURPOSE: Create and initialize the TRNG module 
 ******************************************************************************
 * DESCRIPTION: The function is called to initialize and configure the TRNG 
 *          (True Random Number Generator) module inside SA
 *
 * int16_t Sa_rngInit (
 *          Sa_Handle            handle  - salld instance handle
 *          Sa_RngConfigParams_t *cfg)   - a pointer to configuration structure
 *
 * Return values:  sa_ERR_OK
 *                 sa_ERR_PARAMS
 *
 *****************************************************************************/
int16_t Sa_rngInit (Sa_Handle handle, Sa_RngConfigParams_t* pCfg)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;

  CSL_Cp_aceTrngRegs* pRngRegs = &pSaRegs->TRNG;  
  uint32_t mtKey;
  uint16_t startup_cycles, min_refill_cycles, max_refill_cycles, clock_divider;
  
  if(pCfg == NULL)
    return(sa_ERR_PARAMS);
    
  /* Convert the configuration parameters */
  startup_cycles = sap_rng_conv_cycle_params(pCfg->startupCycles, SA_RNG_STARTUP_CYCLES_MAX,
                                             SA_RNG_STARTUP_CYCLES_MIN, SA_RNG_STARTUP_CYCLES_DEFAULT,
                                             SA_RNG_STARTUP_CYCLES_SHIFT);
  min_refill_cycles = sap_rng_conv_cycle_params(pCfg->minRefillCycles, SA_RNG_MIN_REFILL_CYCLES_MAX,
                                                SA_RNG_MIN_REFILL_CYCLES_MIN, SA_RNG_MIN_REFILL_CYCLES_DEFAULT,
                                                SA_RNG_MIN_REFILL_CYCLES_SHIFT);
                                           
  max_refill_cycles = sap_rng_conv_cycle_params(pCfg->maxRefillCycles, SA_RNG_MAX_REFILL_CYCLES_MAX,
                                                SA_RNG_MAX_REFILL_CYCLES_MIN, SA_RNG_MAX_REFILL_CYCLES_DEFAULT,
                                                SA_RNG_MAX_REFILL_CYCLES_SHIFT);
                                                
  clock_divider = sap_rng_conv_cycle_params(pCfg->clockDiv, SA_RNG_CLOCK_DIVIDER_MAX,
                                            SA_RNG_CLOCK_DIVIDER_MIN, SA_RNG_CLOCK_DIVIDER_DEFAULT,
                                            SA_RNG_CLOCK_DIVIDER_SHIFT);
  Sa_osalMtCsEnter(&mtKey); 
  
  /* Invalidate the Cache Contents */
  Sa_osalBeginMemAccess(inst, sizeof(salldObj_t));  
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    Sa_osalMtCsExit(mtKey);
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  /*
   * Initialize the TRNG only if it has not been initialized or
   * the sa_RNG_CTRL_REINIT flag is set
   */                                                     
  if(SALLD_TEST_STATE_RNG(inst) && !(pCfg->ctrlBitfield & sa_RNG_CTRL_REINIT))
  {
    Sa_osalMtCsExit(mtKey);
    return (sa_ERR_OK);
  } 
  
  /* Enable the RNG module */
  pSaRegs->MMR.CMD_STATUS |= CSL_CP_ACE_CMD_STATUS_TRNG_EN_MASK;
  
  /* Configure the RNG module */
  pRngRegs->TRNG_CONTROL = 0; /* Disable RNG */
  pRngRegs->TRNG_CONTROL = startup_cycles << CSL_CP_ACE_TRNG_CONTROL_STARTUP_CYCLES_SHIFT; 
  pRngRegs->TRNG_CONFIG = (min_refill_cycles << CSL_CP_ACE_TRNG_CONFIG_MIN_REFILL_CYCLES_SHIFT) |
                          ((clock_divider - 1) << CSL_CP_ACE_TRNG_CONFIG_SAMPLE_DIV_SHIFT)      |
                          (max_refill_cycles << CSL_CP_ACE_TRNG_CONFIG_MAX_REFILL_CYCLES_SHIFT);
  if (pCfg->ctrlBitfield & sa_RNG_CTRL_ENABLE_INT)
  {
    pRngRegs->TRNG_INTMASK |= CSL_CP_ACE_TRNG_INTMASK_READY_MASK;  
  } 
  else
  {
    pRngRegs->TRNG_INTMASK &= ~CSL_CP_ACE_TRNG_INTMASK_READY_MASK;  
  
  }                       
  
  /* Enable the RNG module */
  pRngRegs->TRNG_CONTROL |= CSL_CP_ACE_TRNG_CONTROL_ENABLE_TRNG_MASK;

  SALLD_SET_STATE_RNG(inst, 1);  

  /* Writeback the instance updates */
  Sa_osalEndMemAccess(inst, sizeof(salldObj_t));  

  Sa_osalMtCsExit(mtKey);
  return (sa_ERR_OK);

}

/******************************************************************************
 * FUNCTION PURPOSE:  SALLD Get a 64-bit true random number 
 ******************************************************************************
 * DESCRIPTION: This function is called to get a 64-bit true random number.
 *              It is a non-blocking function call with return value indicates
 *              whetehr the random number is available 
 *
 *  int16_t  Sa_getRandonNum (
 *   Sa_Handle        handle      - SALLD instance handle
 *   Bool             f_isr       - Flag to indicate whether it is called from ISR 
 *   Sa_RngData_t*    rnd)        - a pointer to 64-bit random number
 *
 * Return values:  sa_ERR_OK
 *                 sa_ERR_PARAMS 
 *                 sa_ERR_MOUDLE_BUSY
 *                 sa_ERR_MODULE_UNAVAIL 
 *
 ******************************************************************************/
int16_t Sa_getRandomNum (Sa_Handle handle, uint16_t f_isr, Sa_RngData_t* rnd)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;

  CSL_Cp_aceTrngRegs* pRngRegs = &pSaRegs->TRNG;  
  uint32_t mtKey;
  int16_t  retCode = sa_ERR_OK;
  
  if (!rnd)
  {
    return(sa_ERR_PARAMS);
  }
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  Sa_osalMtCsEnter(&mtKey); 
  
  if(!SALLD_TEST_STATE_RNG(inst))
  {
    retCode = sa_ERR_MODULE_UNAVAIL;
  } 
  else if (!(pRngRegs->TRNG_STATUS & CSL_CP_ACE_TRNG_STATUS_READY_MASK))
  {
    retCode = sa_ERR_MODULE_BUSY;
  }
  else
  {
    rnd->hi = pRngRegs->TRNG_OUTPUT_H;   /* read randon number */
    rnd->lo = pRngRegs->TRNG_OUTPUT_L;
  
    pRngRegs->TRNG_INTACK = CSL_CP_ACE_TRNG_INTACK_READY_MASK; /* interrupt acknowledge */
  }
  
  Sa_osalMtCsExit(mtKey);
  
  return(retCode);  
}

/******************************************************************************
 * FUNCTION PURPOSE:  Deactivate the TRNG module (optional) 
 ******************************************************************************
 * DESCRIPTION: This function deactivates the TRNG module and clears its
 *              corresponding LLD internal state 
 *
 *  int16_t  Sa_rngClose  (
 *              Sa_Handle handle)      - SALLD instance
 *
 * Return values:  sa_ERR_OK
 *
 ******************************************************************************/
int16_t  Sa_rngClose  (Sa_Handle handle)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;
 
  CSL_Cp_aceTrngRegs* pRngRegs = &pSaRegs->TRNG;  
  uint32_t mtKey;
  
  Sa_osalMtCsEnter(&mtKey); 

  /* Invalidate the Cache Contents */
  Sa_osalBeginMemAccess(inst, sizeof(salldObj_t));  
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    Sa_osalMtCsExit(mtKey);
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  if (SALLD_TEST_STATE_RNG(inst))
  {
    
    pRngRegs->TRNG_CONTROL = 0; /* Disable RNG */

    SALLD_SET_STATE_RNG(inst, 0);  
    
    /* Writeback the instance updates */
    Sa_osalEndMemAccess(inst, sizeof(salldObj_t));  
    
    /* Disable the RNG module */
    pSaRegs->MMR.CMD_STATUS &= ~CSL_CP_ACE_CMD_STATUS_TRNG_EN_MASK;
    
  }
  
  Sa_osalMtCsExit(mtKey);
  
  return(sa_ERR_OK);  

} /* Sa_rngClose */

/******************************************************************************
 * FUNCTION PURPOSE: Create and initialize the PKA module 
 ******************************************************************************
 * DESCRIPTION: The function is called to initialize the PKA 
 *              (Public Key Accelerator) module inside SA
 *
 * int16_t Sa_pkaInit (
 *          Sa_Handle            handle) - salld instance handle
 *
 * Return values:  sa_ERR_OK
 *
 *****************************************************************************/
int16_t Sa_pkaInit (Sa_Handle handle)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;
 
  CSL_Cp_acePkaRegs* pPkaRegs = &pSaRegs->PKA;  
  
  uint32_t mtKey;
  
  Sa_osalMtCsEnter(&mtKey); 
  
  Sa_osalBeginMemAccess(inst, sizeof(salldObj_t));  
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    Sa_osalMtCsExit(mtKey);
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  
  if(!SALLD_TEST_STATE_PKA(inst))
  {
    /* Enable the PKA module */
    pSaRegs->MMR.CMD_STATUS |= CSL_CP_ACE_CMD_STATUS_PKA_EN_MASK;
    
    SALLD_SET_STATE_PKA(inst, 1); 
    
    /* Pre-set ABCD register */
    pPkaRegs->PKA_ABCDPTR[SA_PKA_AINDEX] = SA_PKA_A_OFFSET;
    pPkaRegs->PKA_ABCDPTR[SA_PKA_BINDEX] = SA_PKA_B_OFFSET;
    pPkaRegs->PKA_ABCDPTR[SA_PKA_CINDEX] = SA_PKA_C_OFFSET;
    pPkaRegs->PKA_ABCDPTR[SA_PKA_DINDEX] = SA_PKA_D_OFFSET;
    
    /* Writeback the instance updates */
    Sa_osalEndMemAccess(inst, sizeof(salldObj_t));  
     
  } 

  Sa_osalMtCsExit(mtKey);
  
  return (sa_ERR_OK);

}

/******************************************************************************
 * FUNCTION PURPOSE:  Perform large vector arithmetic operation via PKA 
 ******************************************************************************
 * DESCRIPTION: This function is called to triggers a large vector arithmetic 
 *              operation through the PKA module.
 *              It is considered as a blocking function call since it will wait 
 *              for the PKA module to complete the arithmetic operation. 
 *              However, it only takes a few cycles for PKA to complete any operation. 
 *              This function also returns with error code immediately if 
 *              the PKA is still in the process to perform the previous operation
 *              or when timeout occurs.
 *
 *  int16_t Sa_pkaOperation (
 *   Sa_Handle        handle      - SALLD instance handle
 *   Sa_PkaReqInfo_t  pkaReqInfo) - Pointer to the PKA operation request information structure
 *
 * Return values:  sa_ERR_OK
 *                 sa_ERR_PARAMS 
 *                 sa_ERR_MOUDLE_BUSY
 *                 sa_ERR_MODULE_UNAVAIL 
 *                 sa_ERR_PKA_TIMEOUT
 *
 ******************************************************************************/
int16_t Sa_pkaOperation (Sa_Handle handle, Sa_PkaReqInfo_t* pPkaReqInfo)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  uint32_t  baseAddr = (uint32_t) salldLObj.baseAddr;
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)baseAddr;
  CSL_Cp_acePkaRegs* pPkaRegs = &pSaRegs->PKA;  
  uint32_t op_code, *pB, *pC;
  int16_t ret_code = sa_ERR_PARAMS;
  uint32_t mtKey;
  int i;
  
  
  if (!pPkaReqInfo)
  {
    return(sa_ERR_PARAMS);
  }
  
  if ((pPkaReqInfo->operation != sa_PKA_OP_COMP) && (!pPkaReqInfo->resultPtr))
  {
    return(sa_ERR_PARAMS);
  }
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  Sa_osalMtCsEnter(&mtKey); 
  
  if(!SALLD_TEST_STATE_PKA(inst))
  {
    ret_code = sa_ERR_MODULE_UNAVAIL;
    
  } 
  else if (pPkaRegs->PKA_FUNCTION & CSL_CP_ACE_PKA_FUNCTION_RUN_MASK)
  {
    ret_code = sa_ERR_MODULE_BUSY;
  }
  else
  {
    
    /* 
    * Perform the following common operations
    * - Copy data for oprand A and B
    * - Configure registers for oprand A & B
    */
    if ((!pPkaReqInfo->oprandSize[SA_PKA_AINDEX]) || (!pPkaReqInfo->oprandPtr[SA_PKA_AINDEX]))
    {
        goto Sa_pkaOperation_end;
    }  
    
    if (pPkaReqInfo->oprandSize[SA_PKA_AINDEX] > sa_MAX_PKA_OPERAND_SIZE)
    {
       goto Sa_pkaOperation_end;
    } 
        
    memcpy((void *)(baseAddr + SA_PKA_A_BASE),  
           pPkaReqInfo->oprandPtr[SA_PKA_AINDEX], pPkaReqInfo->oprandSize[SA_PKA_AINDEX]<<2);
    pPkaRegs->PKA_ABLENGTH[SA_PKA_AINDEX] = pPkaReqInfo->oprandSize[SA_PKA_AINDEX];
    
    if (pPkaReqInfo->oprandSize[SA_PKA_BINDEX])
    {
        if (!pPkaReqInfo->oprandPtr[SA_PKA_BINDEX])
        {
            goto Sa_pkaOperation_end;
        }    
        
        if (pPkaReqInfo->oprandSize[SA_PKA_BINDEX] > sa_MAX_PKA_OPERAND_SIZE)
        {
        goto Sa_pkaOperation_end;
        } 
    
        memcpy((void *)(baseAddr + SA_PKA_B_BASE), 
               pPkaReqInfo->oprandPtr[SA_PKA_BINDEX], pPkaReqInfo->oprandSize[SA_PKA_BINDEX]<<2);
        pPkaRegs->PKA_ABLENGTH[SA_PKA_BINDEX] = pPkaReqInfo->oprandSize[SA_PKA_BINDEX];
    }
    
    switch (pPkaReqInfo->operation)
    {
        case sa_PKA_OP_ADD:
            /* Range check: Alength > 0; Blength > 0 */
            if (!pPkaReqInfo->oprandSize[SA_PKA_BINDEX])
            {
                goto Sa_pkaOperation_end;
            }    
        
            op_code = CSL_CP_ACE_PKA_FUNCTION_ADD_MASK |
                      CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;
        
        case sa_PKA_OP_SUB:
            /* Range check: Alength > 0; Blength > 0; Alength >= Blength */
            if ((!pPkaReqInfo->oprandSize[SA_PKA_BINDEX]) || 
                (pPkaReqInfo->oprandSize[SA_PKA_AINDEX] < pPkaReqInfo->oprandSize[SA_PKA_BINDEX]))
            {
                goto Sa_pkaOperation_end;
            }    
        
            op_code = CSL_CP_ACE_PKA_FUNCTION_SUBSTRACT_MASK |
                    CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;

        case sa_PKA_OP_MUL:
            /* Range check: Alength > 0; Blength > 0 */
            if (!pPkaReqInfo->oprandSize[SA_PKA_BINDEX])
            {
                goto Sa_pkaOperation_end;
            } 
               
            op_code = CSL_CP_ACE_PKA_FUNCTION_MULTIPLY_MASK |
                    CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;
        
        case sa_PKA_OP_DIV:
            /* 
             * Alength > 1; Blength > 1
             * B != 0  
             * Alength >= Blength
             * B[Blength - 1] != 0 ==> B != 0
             */
            if ((pPkaReqInfo->oprandSize[SA_PKA_AINDEX] <= 1) ||
                (pPkaReqInfo->oprandSize[SA_PKA_BINDEX] <= 1) || 
                (pPkaReqInfo->oprandSize[SA_PKA_AINDEX] < pPkaReqInfo->oprandSize[SA_PKA_BINDEX]) ||
                (pPkaReqInfo->oprandPtr[SA_PKA_BINDEX][pPkaReqInfo->oprandSize[SA_PKA_BINDEX] - 1] == 0) ||
                (!pPkaReqInfo->remPtr))
            {
                goto Sa_pkaOperation_end;
            }    
             
            op_code = CSL_CP_ACE_PKA_FUNCTION_DIVIDE_MASK |
                    CSL_CP_ACE_PKA_FUNCTION_RUN_MASK; 
            break;
        
        case sa_PKA_OP_RSHIFT:
            pPkaRegs->PKA_SHIFT = pPkaReqInfo->numShiftBits;
            op_code = CSL_CP_ACE_PKA_FUNCTION_RSHIFT_MASK |
                    CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;
        
        case sa_PKA_OP_LSHIFT:
            pPkaRegs->PKA_SHIFT = pPkaReqInfo->numShiftBits;
            op_code = CSL_CP_ACE_PKA_FUNCTION_LSHIFT_MASK |
                    CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;
        
        case sa_PKA_OP_COMP:
            /* Alength == Blength */
            if (pPkaReqInfo->oprandSize[SA_PKA_AINDEX] != pPkaReqInfo->oprandSize[SA_PKA_BINDEX])
            {
                goto Sa_pkaOperation_end;
            }
            op_code = CSL_CP_ACE_PKA_FUNCTION_COMPARE_MASK |
                    CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;
        
        case sa_PKA_OP_COPY:
            op_code = CSL_CP_ACE_PKA_FUNCTION_COPY_MASK |
                    CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;
        
        case sa_PKA_OP_EXP2:
        case sa_PKA_OP_EXP4:
            /* 
             * Alength > 0; Blength > 1
             * A != 0; C != 0  (TBD: non-zero check)
             * Alength < Blength
             * B:odd, B[Length - 1] != 0
             * B > C 
             */
            if ((pPkaReqInfo->oprandSize[SA_PKA_BINDEX] <= 1) || 
                (pPkaReqInfo->oprandSize[SA_PKA_AINDEX] >= pPkaReqInfo->oprandSize[SA_PKA_BINDEX]) ||
                (!(pPkaReqInfo->oprandPtr[SA_PKA_BINDEX][0] & 0x1))                                      ||
                (pPkaReqInfo->oprandPtr[SA_PKA_BINDEX][pPkaReqInfo->oprandSize[SA_PKA_BINDEX] - 1] == 0) ||
                (!pPkaReqInfo->oprandPtr[SA_PKA_CINDEX]))
            {
                goto Sa_pkaOperation_end;
            }   
            
            if (sap_pka_is_zero(pPkaReqInfo->oprandPtr[SA_PKA_AINDEX], pPkaReqInfo->oprandSize[SA_PKA_AINDEX]))
            {
                goto Sa_pkaOperation_end;
            } 
            
            if (sap_pka_is_zero(pPkaReqInfo->oprandPtr[SA_PKA_CINDEX], pPkaReqInfo->oprandSize[SA_PKA_BINDEX]))
            {
                goto Sa_pkaOperation_end;
            }
            
            if ((pPkaReqInfo->operation == sa_PKA_OP_EXP4) && (pPkaReqInfo->oprandSize[SA_PKA_BINDEX] > sa_MAX_PKA_OPERAND_SIZE_EXP4))
            {
                goto Sa_pkaOperation_end;
            } 
            
            pB = pPkaReqInfo->oprandPtr[SA_PKA_BINDEX];
            pC = pPkaReqInfo->oprandPtr[SA_PKA_CINDEX];
            for ( i = pPkaReqInfo->oprandSize[SA_PKA_BINDEX] - 1; i > 0; i--)
            {
                if (pC[i] > pB[i])
                {
                    goto Sa_pkaOperation_end;
                }
                else if (pC[i] < pB[i])
                {
                    break;
                }
            } 
            memcpy((void *)(baseAddr + SA_PKA_C_BASE), 
                   pPkaReqInfo->oprandPtr[SA_PKA_CINDEX], pPkaReqInfo->oprandSize[SA_PKA_BINDEX]<<2);
             
        
            op_code = ((pPkaReqInfo->operation == sa_PKA_OP_EXP2)?CSL_CP_ACE_PKA_FUNCTION_EXP2_MASK:CSL_CP_ACE_PKA_FUNCTION_EXP4_MASK) |
                      CSL_CP_ACE_PKA_FUNCTION_RUN_MASK;
            break;
        
        default:
            goto Sa_pkaOperation_end;
    }
  
    /* Initialize the Operation */
    pPkaRegs->PKA_FUNCTION  = op_code;
  
    /* Wait for the operation to be complete or timeout occurs */
    for (i = 0; i < 1000000; i++)
    {
        if (!(pPkaRegs->PKA_FUNCTION & CSL_CP_ACE_PKA_FUNCTION_RUN_MASK))
            break;
    }
  
    /* figure out how to calculate the size of operation result */
    if (i >= 1000000)
    {
        ret_code = sa_ERR_PKA_TIMEOUT;
    }
    else
    {
        /* Process the operation result */
        ret_code = sa_ERR_OK;
        switch (pPkaReqInfo->operation)
        {
            case sa_PKA_OP_ADD:
            case sa_PKA_OP_SUB:
            case sa_PKA_OP_MUL:
            case sa_PKA_OP_RSHIFT:
            case sa_PKA_OP_LSHIFT:
            case sa_PKA_OP_COPY:
                pPkaReqInfo->resultSize = (pPkaRegs->PKA_MSW & CSL_CP_ACE_PKA_MSW_RES_ZERO_MASK)?0:
                                           (uint16_t)(pPkaRegs->PKA_MSW - SA_PKA_C_OFFSET + 1);  
                memcpy(pPkaReqInfo->resultPtr, 
                       (void *)(baseAddr + SA_PKA_C_BASE), 
                       pPkaReqInfo->resultSize << 2);
                break;
    
            case sa_PKA_OP_DIV:
                pPkaReqInfo->resultSize = (pPkaRegs->PKA_MSW & CSL_CP_ACE_PKA_MSW_RES_ZERO_MASK)?0:
                                            (uint16_t)(pPkaRegs->PKA_MSW - SA_PKA_D_OFFSET + 1);  
                pPkaReqInfo->remSize = (pPkaRegs->PKA_DIVMSW & CSL_CP_ACE_PKA_DIVMSW_RES_ZERO_MASK)?0:
                                            (uint16_t)(pPkaRegs->PKA_DIVMSW - SA_PKA_C_OFFSET + 1);  
                memcpy(pPkaReqInfo->remPtr, 
                       (void *)(baseAddr + SA_PKA_C_BASE), 
                       pPkaReqInfo->remSize << 2);
                memcpy(pPkaReqInfo->resultPtr, 
                       (void *)(baseAddr + SA_PKA_D_BASE), 
                       pPkaReqInfo->resultSize << 2);
                break;
    
            case sa_PKA_OP_COMP:
                pPkaReqInfo->cmpResult =  (Sa_PkaCompResults_t)pPkaRegs->PKA_COMPARE;
                break;
    
            case sa_PKA_OP_EXP2:
            case sa_PKA_OP_EXP4:
                pPkaReqInfo->resultSize = (pPkaRegs->PKA_MSW & CSL_CP_ACE_PKA_MSW_RES_ZERO_MASK)?0:
                                            (uint16_t)(pPkaRegs->PKA_MSW - SA_PKA_D_OFFSET + 1);  
                memcpy(pPkaReqInfo->resultPtr, 
                       (void *)(baseAddr + SA_PKA_D_BASE), 
                       pPkaReqInfo->resultSize << 2);
                break;
        
            default:
                break;    
        }
  
    }
  }
  
Sa_pkaOperation_end:
  
  Sa_osalMtCsExit(mtKey);
  
  return(ret_code);  
}

/******************************************************************************
 * FUNCTION PURPOSE:  Deactivate the PKA module (optional) 
 ******************************************************************************
 * DESCRIPTION: This function deactivates the PKA module and clears its
 *              corresponding LLD internal state 
 *
 *  int16_t  Sa_pkaClose  (
 *              Sa_Handle handle)      - SALLD instance
 *
 * Return values:  sa_ERR_OK
 *
 ******************************************************************************/
int16_t  Sa_pkaClose  (Sa_Handle handle)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;
 
  uint32_t mtKey;
  
  Sa_osalMtCsEnter(&mtKey); 
  
  Sa_osalBeginMemAccess(inst, sizeof(salldObj_t)); 
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    Sa_osalMtCsExit(mtKey);
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  if (SALLD_TEST_STATE_PKA(inst))
  {
    
    SALLD_SET_STATE_PKA(inst, 0);  
    
    /* Disable the RNG module */
    pSaRegs->MMR.CMD_STATUS &= ~CSL_CP_ACE_CMD_STATUS_PKA_EN_MASK;
    
    /* Writeback the instance updates */
    Sa_osalEndMemAccess(inst, sizeof(salldObj_t));  
  }
  
  Sa_osalMtCsExit(mtKey);
  
  return(sa_ERR_OK);  

} /* Sa_pkaClose */


/******************************************************************************
 * FUNCTION PURPOSE:  Obtain the SA system ID 
 ******************************************************************************
 * DESCRIPTION: This function returns the SA system ID associated with the 
 *      specified handle. 
 *         SALLD instance.
 *
 *****************************************************************************/
uint32_t  Sa_getID (Sa_Handle handle)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
    
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
    return(0); 
  }
  else
  {
    return (inst->ID);
  }  
}
 
/******************************************************************************
 * FUNCTION PURPOSE:  Obtain the SA shadow system handle 
 ******************************************************************************
 * DESCRIPTION: This function returns the shaddow system handle. The shaddow system 
 *           handle is set to NULL if it does not exist.
 *
 *****************************************************************************/
int16_t  Sa_getShadowHandle  (Sa_Handle handle, Sa_Handle *shandle)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);

  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }
  
  *shandle = (Sa_Handle)inst->shadowInstOffset; 
  
  return (sa_ERR_OK); 

}


/****************************************************************************************
 * FUNCTION PURPOSE: Enable a PDSP
 ****************************************************************************************
 * DESCRIPTION:  The PDSP is put into the run state if it is currently in the reset state.
 *               If taken out of reset the function waits for the PDSP to indicate 
 *               that it is ready. 
 *
 *               Return value:
 *                  SALLD_PDSP_ALREADY_ACTIVE - the PDSP was already out of reset
 *                  SALLD_PDSP_RESET_RELEASED - the PDSP was in reset and successfully taken out of reset
 *                  SALLD_PDSP_NO_RESTART - the PDSP was taken out of reset but timed out on the handshake
 ****************************************************************************************/

#define SALLD_PDSP_ALREADY_ACTIVE      0
#define SALLD_PDSP_RESET_RELEASED      1
#define SALLD_PDSP_NO_RESTART          2

static int salld_pdsp_run (CSL_Cp_acePdsp_control_statusRegs *pRegs)
{
  /* Check for enabled PDSP */
  if ( (pRegs->PDSP_CONTROL & CSL_CP_ACE_PDSP_CONTROL_ENABLE_MASK) == CSL_CP_ACE_PDSP_CONTROL_ENABLE_MASK)
    return (SALLD_PDSP_ALREADY_ACTIVE);

  /* Enable the PDSP */
  pRegs->PDSP_CONTROL |= CSL_CP_ACE_PDSP_CONTROL_ENABLE_MASK;

  return (SALLD_PDSP_RESET_RELEASED);

} /* salld_pdsp_run */

/*****************************************************************************************
 * FUNCTION PURPOSE: Set the state of the Sub-System
 *****************************************************************************************
 * DESCRIPTION: The Sub-System state can be set or queried.
 *****************************************************************************************/
Sa_State_t Sa_resetControl (Sa_Handle handle, Sa_State_t newState)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;
 
  Sa_State_t  ret           = sa_STATE_ENABLE;
  int i;         
  uint32_t mtKey;
  
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
    return (sa_STATE_INVALID_REQUEST);
  }  
  
  Sa_osalMtCsEnter(&mtKey); 
                                

  if (newState == sa_STATE_RESET)  {

    /* Disable and reset PDSPs */
    for (i = 0; i < 2; i++)  {
      pSaRegs->PDSP_CONTROL_STATUS[i].PDSP_CONTROL =  0;
    }

    Sa_osalMtCsExit(mtKey);
    return (sa_STATE_RESET);

  }

  if (newState == sa_STATE_ENABLE)  {
    int         res;

    /* Do nothing if a pdsp is already out of reset. If any PDSPs are out of reset
     * a global init is not performed */
    for (i = 0; i < 2; i++)  {

      res = salld_pdsp_run (&pSaRegs->PDSP_CONTROL_STATUS[i]);
    
      if (res == SALLD_PDSP_NO_RESTART)  {
        ret = sa_STATE_ENABLE_FAILED;
      }
    }

    Sa_osalMtCsExit(mtKey);
    return (ret);

  }

  if (newState == sa_STATE_QUERY)  {

    uint16_t enable  = FALSE;
    uint16_t reset   = FALSE;

    for (i = 0; i < 2; i++)  {
      if ( (pSaRegs->PDSP_CONTROL_STATUS[i].PDSP_CONTROL & CSL_CP_ACE_PDSP_CONTROL_ENABLE_MASK) == 
                     CSL_CP_ACE_PDSP_CONTROL_ENABLE_MASK )
        enable = TRUE;
      else
        reset = TRUE;
    }

    if ((reset == TRUE) && (enable == TRUE))
      ret = sa_STATE_INCONSISTENT; 
    else if (reset == TRUE)
       ret = sa_STATE_RESET;
    else
       ret = sa_STATE_ENABLE;
       
    Sa_osalMtCsExit(mtKey);
    return (ret);

  }

  Sa_osalMtCsExit(mtKey);
  return (sa_STATE_INVALID_REQUEST);

} /* Sa_resetControl */

/***********************************************************************************************
 * FUNCTION PURPOSE: Download a PDSP image
 ***********************************************************************************************
 * DESCRIPTION: A PDSP image is downloaded. The PDSP remains in reset.
 ***********************************************************************************************/
int16_t Sa_downloadImage (Sa_Handle handle, int modId, void* image, int sizeBytes)
{

  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;
 
  uint32_t mtKey;
  
  /* Verify the specified PDSP is valid */
  if ((modId < 0)  || (modId >= 2))
    return (sa_ERR_PARAMS);

  /* Verify the image size is in range */
  if ((sizeBytes < 0)  || (sizeBytes >= 0x4000))
    return (sa_ERR_PARAMS);
    
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
  
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
    
  Sa_osalMtCsEnter(&mtKey); 

  /* Make sure the PDSP is disabled */
  if ( (pSaRegs->PDSP_CONTROL_STATUS[modId].PDSP_CONTROL & CSL_CP_ACE_PDSP_CONTROL_ENABLE_MASK)  == 
                CSL_CP_ACE_PDSP_CONTROL_ENABLE_MASK)
  {
    Sa_osalMtCsExit(mtKey);
    return (sa_STATE_INVALID);
  }
  /* Copy the image */
  memcpy ((void *)pSaRegs->PDSP_IRAM[modId].DATA, image, sizeBytes);

  Sa_osalMtCsExit(mtKey);
  return (sa_ERR_OK);

} /* Sa_downloadImage */

/***********************************************************************************************
 * FUNCTION PURPOSE: Inquire PDSP Version Number
 ***********************************************************************************************
 * DESCRIPTION: Extract the SA PDSP  version number.
 ***********************************************************************************************/
int16_t Sa_getPDSPVersion (Sa_Handle handle, int modId, uint32_t *pVersion)
{
  salldObj_t *inst = (salldObj_t *)sa_CONV_OFFSET_TO_ADDR(salldLObj.instPoolBaseAddr, handle);
  CSL_Cp_aceRegs*  pSaRegs = (CSL_Cp_aceRegs*)salldLObj.baseAddr;
   
  /* Verify the specified PDSP is valid */
  if ((modId < 0)  || (modId >= 2))
    return (sa_ERR_PARAMS);
    
  if (inst->magic != SALLD_INST_MAGIC_WORD)
  {
    if(inst->magic == salld_swiz32(SALLD_INST_MAGIC_WORD))
        return(sa_ERR_INV_ENDIAN_MODE);
    else
        return(sa_ERR_INV_HANDLE);    
  }  
  
  *pVersion = pSaRegs->PDSP_SRAM[modId].DATA[SA_PHP_VERSION_OFFSET];
  
  return (sa_ERR_OK);

} /* Sa_getPDSPVersion */


/***********************************************************************************************
 * FUNCTION PURPOSE: Inquire Version Number
 ***********************************************************************************************
 * DESCRIPTION: Return the SA LLD version number.
 ***********************************************************************************************/
uint32_t Sa_getVersion (void)
{
    return SA_LLD_VERSION_ID;
}

/***********************************************************************************************
 * FUNCTION PURPOSE: Inquire Version String
 ***********************************************************************************************
 * DESCRIPTION: Return the SA LLD version string.
 ***********************************************************************************************/
const char* Sa_getVersionStr (void)
{
    return sa_lld_version_str;
}



  




