/*
 *
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/



#include "../unittest.h"
#include "../testconn.h"
#include "../salldsim/salldcfg.h"

/* Data Mode Basic Functional test.
 * This test is designed to verify the basic opertions of Data Mode
 * by creating multiple Data Mode channels with different configurations
 * and performing packet verification tests.
 * The goal is to cover all the normal program flows in both SA LLD and the
 * PDSP firmware inside the SA sub-system.
 *
 * Test Procedures:
 * 
 * - Create multiple Data Mode connections including pairs of SA LLD Data Mode
 *   channels with configurations specified below
 * - Perform multi-packet, multi-connection data verification test
 *   - For each pkt loop
 *     - Generate payload with specific data pattern and variable length
 *     - For each pair of connections
 *       - Format the raw packet with the connection header and payload
 *       - Perform the following protocol-specific operations 
 *         - Invoke salld_sendData() API for Data Mode channel
 *       - Prepare and send packet to SA for Encryption (and Authentication)
 *       - Receive encrypted packet from SA sub-system
 *       - Perform the following protocol-specific operations
 *          - Invoke salld_sendData() API for the corresponding Data Mode channel
 *       - Forward the packet to SA for Decryption (and Authentication) 
 *       - Preceive the decrypted packet from the SA sub-system
 *       - Perform payload verification              
 *   - End of the test loop
 * - Query all the SALLD channel statistics
 * - Close all the SALLD channels
 * 
 * a0 is a pointer to the test framework structure
 * a1 is a pointer to the saTest_t structure for this test, as initialized in testMain.
 * 
 */

/*
 * Data Mode channel configuration array
 */
static sauDataModeConfig_t  testDmCfg[] =
{
    /* Cipher Mode, Auth Mode, encKeySize, macKeySize, macSize, aadSize, enc flag, enc1st flag */
    {sa_CipherMode_AES_CTR, sa_AuthMode_HMAC_SHA1,      16, 20,  8,  0,  TRUE, TRUE},
    {sa_CipherMode_AES_CTR, sa_AuthMode_HMAC_SHA1,      16, 20,  8,  0,  FALSE, FALSE},
    {sa_CipherMode_GCM,     sa_AuthMode_NULL,           16,  0,  8, 12,  TRUE, TRUE},
    {sa_CipherMode_GCM,     sa_AuthMode_NULL,           16,  0,  8, 12,  FALSE, TRUE},
    {sa_CipherMode_NULL,    sa_AuthMode_GMAC,           0,  24, 12,  8,  FALSE, FALSE},
    {sa_CipherMode_NULL,    sa_AuthMode_GMAC,           0,  24, 12,  8,  FALSE, FALSE},
    {sa_CipherMode_NULL,    sa_AuthMode_GMAC_AH,        0,  16, 12,  0,  FALSE, FALSE},
    {sa_CipherMode_NULL,    sa_AuthMode_GMAC_AH,        0,  16, 12,  0,  FALSE, FALSE},
    {sa_CipherMode_CCM,     sa_AuthMode_NULL,           32,  0, 16, 12,  TRUE, TRUE},
    {sa_CipherMode_CCM,     sa_AuthMode_NULL,           32,  0, 16, 12,  FALSE, TRUE},
    {sa_CipherMode_AES_CBC, sa_AuthMode_HMAC_MD5,       32, 20, 12,  0,  TRUE, TRUE},
    {sa_CipherMode_AES_CBC, sa_AuthMode_HMAC_MD5,       32, 20, 12,  0,  FALSE, FALSE},
    {sa_CipherMode_DES_CBC, sa_AuthMode_HMAC_SHA2_224,   8, 32, 16,  0,  TRUE, TRUE},
    {sa_CipherMode_DES_CBC, sa_AuthMode_HMAC_SHA2_224,   8, 32, 16,  0,  FALSE, FALSE},
    {sa_CipherMode_3DES_CBC, sa_AuthMode_HMAC_SHA2_256,  24, 32, 16,  0,  TRUE, TRUE},
    {sa_CipherMode_3DES_CBC, sa_AuthMode_HMAC_SHA2_256,  24, 32, 16,  0,  FALSE, FALSE},
    #ifdef SA_3GPP_SUPPORT
    {sa_CipherMode_SNOW3G_F8,  sa_AuthMode_NULL,        16, 0,  0,  0,   FALSE, TRUE},
    {sa_CipherMode_SNOW3G_F8,  sa_AuthMode_NULL,        16, 0,  0,  0,   FALSE, TRUE},
    {sa_CipherMode_KASUMI_F8,    sa_AuthMode_KASUMI_F9, 16, 16,  4,  0,  FALSE, FALSE},
    {sa_CipherMode_KASUMI_F8,    sa_AuthMode_KASUMI_F9, 16, 16,  4,  0,  FALSE, TRUE},
    #endif
    {sa_CipherMode_NULL,    sa_AuthMode_CMAC,           0,  16, 12,  0,  TRUE, TRUE},
    {sa_CipherMode_NULL,    sa_AuthMode_CMAC,           0,  16, 12,  0,  FALSE, FALSE},
    {sa_CipherMode_NULL,    sa_AuthMode_MD5,            0,  16, 12,  0,  TRUE, TRUE},
    {sa_CipherMode_NULL,    sa_AuthMode_MD5,            0,  16, 12,  0,  FALSE, FALSE},
    #ifdef SA_3GPP_SUPPORT
    {sa_CipherMode_NULL,    sa_AuthMode_KASUMI_F9,      0,  16,  4,  0,  FALSE, FALSE},
    {sa_CipherMode_NULL,    sa_AuthMode_KASUMI_F9,      0,  16,  4,  0,  FALSE, FALSE},
    #endif
    {sa_CipherMode_NULL,    sa_AuthMode_NULL,           0,   0,  0,  0,  TRUE,  TRUE},
    {sa_CipherMode_NULL,    sa_AuthMode_NULL,           0,   0,  0,  0,  FALSE, FALSE},
};

/* SA Data Mode Basic Functional test */
Void saDataModeTest (UArg a0, UArg a1)
{

 	tFramework_t  *tf  = (tFramework_t *)a0;
 	saTest_t      *pat = (saTest_t *)a1;
    sauHdrHandle_t *pDmHdl;
 	Int  i;
    UInt16 hdrSize = 12;  
    
    /* Initialize protocol addresses */
    
    /* Create test channels */
    for ( i = 0; i <  sizeof(testDmCfg)/sizeof(sauDataModeConfig_t) ; i += 2, hdrSize += 2)
    {
        /* Create a pair of Data Mode channels */
        if ((pDmHdl = sauCreateDm(NULL, hdrSize, &testDmCfg[i])) == NULL)
        {
            saTestRecoverAndExit (tf, pat, SA_TEST_FAILED);  /* no return */
        }
        
        /* Create connection */
        if (sauCreateConnection(tf, pat, pDmHdl) == NULL)
        {
            saTestRecoverAndExit (tf, pat, SA_TEST_FAILED);  /* no return */
        }
        
        if ((pDmHdl = sauCreateDm(NULL, hdrSize, &testDmCfg[i+1])) == NULL)
        {
            saTestRecoverAndExit (tf, pat, SA_TEST_FAILED);  /* no return */
        }
        
        /* Create connection */
        if (sauCreateConnection(tf, pat, pDmHdl) == NULL)
        {
            saTestRecoverAndExit (tf, pat, SA_TEST_FAILED);  /* no return */
        }
    }
    
    /* Packet Tests */
    sauConnDmPktTest(tf, pat, 
                     numTestPkt1,     /* Number of packets per connection */
                     80,              /* initial payload length */
                     3,               /* payload llength increment step (3)*/
                     SAU_PAYLOAD_INC8 /* payload pattern */
                     );
                     
    sauConnDmPktTest(tf, pat, 
                     numTestPkt2,     /* Number of packets per connection */
                     300,             /* initial payload length */
                     6,               /* payload llength increment step */
                     SAU_PAYLOAD_INC8 /* payload pattern */
                     );
                   
    /* Get Channel Statistics */
    salldSim_get_all_chan_stats();
    
 	saTestRecoverAndExit (tf, pat, SA_TEST_PASSED);  /* no return */
}
	
	
	
		
	


  		
