/**
 *   @file  WatchdogTimer.c
 *
 *   @brief   
 *      The file implements the WatchdogTimer module.
 *
 *  \par
 *  NOTE:
 *      (C) Copyright 2012 Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**  @mainpage Watchdog Timer Module
 *
 *   @section intro  Introduction
 *
 *  The Watchdog Timer Module provides configuration and use of the watchdog timer mode
 *  for the DSP local timers.  The watchdog mode is a 64-bit timer mode for the local timers.
 *  When configured in the watchdog mode the timer will count up to a configured 64-bit
 *  period value.  The count is incremented for each DSP clock tick.  Periodically the DSP must
 *  "service" the watchdog timer by writing a sequence of values to the timer control registers.
 *  When the watchdog timer is serviced correctly the count will reset to zero and then resume
 *  counting up to the timer period value.  If for any reason the count value reaches the timer
 *  period a watchdog timer event will be triggered.  The actions taken by the triggered event
 *  are based on the watchdog timer configurations in the timer control registers and the device
 *  reset block.
 *
 *  In general, a device's timer n maps to DSP core n's local timer.  The DSP local timers are
 *  the only timers in the system capable of operating in the watchdog timer mode.  This means
 *  that each DSP core has only one timer capable of acting as its watchdog timer.  Please keep
 *  this in mind when using this module.  Any applications integrated with this module must not
 *  make use of the DSP local timer as it will taken by this module to be used as the watchdog
 *  timer.  Module configuration parameters are specifically provided to handle reconfiguring
 *  the timer used by BIOS at startup for the Clock module.
 *
 *  Watchdog Timer Mapping:
 *    DSP Timer 0 -> Core 0 Local/Watchdog Timer <br>
 *    DSP Timer 1 -> Core 1 Local/Watchdog Timer <br>
 *    DSP Timer 2 -> Core 2 Local/Watchdog Timer <br>
 *    DSP Timer 3 -> Core 3 Local/Watchdog Timer <br>
 *    DSP Timer n -> Core n Local/Watchdog Timer <br>
 *
 *  DSPs can be configured to take different actions if its configured watchdog timer were to
 *  expire.  These actions can be configured via the configuration structure passed to
 *  the Watchdog Timer module's _Init function.  The watchdog timer can be configured to 
 *  generate local hard and soft resets when the watchdog expires.  It can also be configured
 *  to generate an NMI or an NMI followed by a local reset after a configurable number of 
 *  clock cycles.  A major functionality of the watchdog timer expiration actions is the
 *  generation of a System event that can be registered as a system exception in the 
 *  INTC exception combiner.  This allows the BIOS Exception module to handle an exception
 *  generated by the expiration of the watchdog timer.  Since the BIOS Exception module
 *  is capable of handling the watchdog timer expiration exception other fault management
 *  capabilities, like core dumps, can be executed when the watchdog timer expires.
 *
 *  For Watchdog Timer Module API information please see the WatchdogTimer.xdc file.
 *
 *  For device information on the watchdog timer operation please see: <br>
 *  SPRUGV5A - Keystone Architecture Timer64P User Guide  
 */

/** @defgroup WATCHDOGTIMER_API   DSP Watchdog Timer
 */
 
/**************************************************************************
 *************************** Include Files ********************************
 **************************************************************************/

/* Standard Include Files. */
#include <c6x.h>

/* BIOS/XDC Include Files. */
#include <xdc/std.h>
#include <ti/sysbios/family/c64p/Exception.h>

/* WatchdogTimer Includes */
#include <ti/instrumentation/wdtimer/WatchdogTimer.h>
#include "package/internal/WatchdogTimer.xdc.h"

/**
@defgroup WATCHDOGTIMER_FUNCTION                DSP Watchdog Timer Functions
@ingroup WATCHDOGTIMER_API
*/
/**
@defgroup WATCHDOGTIMER_DATA_STRUCTURE          DSP Watchdog Timer Data Structures
@ingroup WATCHDOGTIMER_API
*/

/**************************************************************************
 ************************** Global Variables ******************************
 **************************************************************************/

/** @addtogroup WATCHDOGTIMER_DATA_STRUCTURE
 @{ */

/**
 * @brief
 *  Base address for all Watchdog Timer registers.  An offset based on the DSP core number
 *  will be added to the register base address at initialization time to access each core's 
 *  watchdog timer registers
 */
#define WatchdogTimer_REG_BASE 0x02200000

/**
 * @brief
 * Register field write macro
 */
#define WatchdogTimer_REG_FIELD_WRITE(reg, mask, offset, val) \
                   reg = ((reg & ((uint32_t) (~(mask << offset)))) | ((val & mask) << offset))
                   

/**
 * @brief
 *  Watchdog timer's register overlay
 */
volatile WatchdogTimer_regOverlay *wdRegOverlay;

/**
@}
*/



/**********************************************************************
 ********************* Internal Functions *********************
 **********************************************************************/
 
/**
 *  @b Description
 *  @n  
 *      This function configures the corepac reset action to take if the watchdog
 *      timer expires.
 *
 *  @param[in]  omode
 *      RSTMUX register output mode configured
 *  @param[in]  delay
 *      RSTMUX register configured delay between NMI and local reset
 */
void configureResetMuxReg(WatchdogTimer_RstmuxOmode omode, 
                                                     WatchdogTimer_RstmuxDelay delay)
{
    volatile uint32_t *rstmux = (volatile uint32_t *) (WatchdogTimer_RSTMUXnBASE);
    uint32_t omodeVal = (uint32_t) omode;
    uint32_t delayVal = (uint32_t) delay;
    
    /* Set the output mode making sure to offset the RSTMUXn register pointer based on the core number */
    WatchdogTimer_REG_FIELD_WRITE(rstmux[DNUM], WatchdogTimer_RSTMUXn_OMODE_MASK, 
                                                               WatchdogTimer_RSTMUXn_OMODE_SHIFT, omodeVal);
    

    /* Set the delay field if NMI followed by a local reset is selected */
    if (omode == WatchdogTimer_RstmuxOmode_NMI_INPUT_PLUS_LOCAL_RESET_INPUT)
    {
        /* Offset the RSTMUXn register pointer based on the core number */
        WatchdogTimer_REG_FIELD_WRITE(rstmux[DNUM], WatchdogTimer_RSTMUXn_DELAY_MASK, 
                                                                   WatchdogTimer_RSTMUXn_DELAY_SHIFT, delayVal);
    }

    return;
}


/**********************************************************************
 ********************* External Functions *********************
 **********************************************************************/

/** @addtogroup WATCHDOGTIMER_FUNCTION
 @{ */

/**
 *  @b Description
 *  @n 
 *      This function is called to configure and start the watchdog timer for a DSP core.
 *      The input configuration controls watchdog running conditions, timeout conditions,
 *      and timeout actions.
 *
 *  @param[in]  wdCfg
 *      Watchdog timer configuration structure.  For more information on the configuration
 *      parameters please see WatchdogTimer.xdc
 *
 *  @retval
 *      WatchdogTimer_Status - For more information on status return values please see
 *                                           WatchdogTimer.xdc
 */
WatchdogTimer_Status WatchdogTimer_Init (WatchdogTimer_InitCfg *wdCfg)
{
    volatile uint32_t *rstctrl = (volatile uint32_t *)WatchdogTimer_RSTCTRL;
    volatile uint32_t *rstcfg = (volatile uint32_t *)WatchdogTimer_RSTCFG;

    /* Set the Watchdog timer register overlay address based on DSP core number */
    wdRegOverlay = (volatile WatchdogTimer_regOverlay *) (WatchdogTimer_REG_BASE + (0x10000 * DNUM));

    /* Configure how the reset block handles the watchdog timer timeout event */
    configureResetMuxReg(wdCfg->rstOmode, wdCfg->rstDelay);

    /* Check the RSTCFG reset type value */
    if ((wdCfg->wdResetType != WatchdogTimer_RstcfgResetType_HARD_RESET) &&
        (wdCfg->wdResetType != WatchdogTimer_RstcfgResetType_SOFT_RESET))
    {
        return (WatchdogTimer_Status_ERROR_INVALID_RSTCFG_VAL);
    }
    
    /* Configure the type of reset to occur if the watchdog timer expires */
    WatchdogTimer_REG_FIELD_WRITE(*rstctrl, WatchdogTimer_RSTCTRL_KEY_MASK, 
                                                               WatchdogTimer_RSTCTRL_KEY_SHIFT, 
                                                               (uint32_t) WatchdogTimer_RSTCTRL_KEY_VALUE);
    /* Configure the core's watchdog timer reset type */
    WatchdogTimer_REG_FIELD_WRITE(*rstcfg, WatchdogTimer_RSTCFG_WDTYPE_MASK, 
                                                               DNUM, (uint32_t) wdCfg->wdResetType);

    if (wdCfg->wdRegisterTimeoutExc)
    {
        /* Register the TINTLn GEM event as an exception causing event */
        Exception_evtEvtClear(WatchdogTimer_TINTLDSPINT);
        Exception_evtExpMaskEnable(WatchdogTimer_TINTLDSPINT);
    }
      
    /* Enable interrupt generation for watchdog mode */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->INTCTLSTAT, 
                                   WatchdogTimer_INTCTRLSTAT_PRDINTEN_MASK, 
                                   WatchdogTimer_INTCTRLSTAT_PRDINTEN_LO_SHIFT, 
                                   (uint32_t) WatchdogTimer_INTCTRLSTAT_PRDINTEN_ENABLE_INT);
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->INTCTLSTAT, 
                                   WatchdogTimer_INTCTRLSTAT_PRDINTEN_MASK, 
                                   WatchdogTimer_INTCTRLSTAT_PRDINTEN_HI_SHIFT, 
                                   (uint32_t) WatchdogTimer_INTCTRLSTAT_PRDINTEN_ENABLE_INT);

    /* Move the timer from disabled to initial state (watchdog mode is still disabled)
      * TIMLORS = not in reset
      * TIMHIRS = not in reset
      * TIMMODE = 64-bit watchdog timer mode */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->TGCR, 
                                   WatchdogTimer_TGCR_TIMLORS_MASK, 
                                   WatchdogTimer_TGCR_TIMLORS_SHIFT, 
                                   (uint32_t) WatchdogTimer_TGCR_TIMLORS_NOT_IN_RESET);
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->TGCR, 
                                   WatchdogTimer_TGCR_TIMHIRS_MASK, 
                                   WatchdogTimer_TGCR_TIMHIRS_SHIFT, 
                                   (uint32_t) WatchdogTimer_TGCR_TIMHIRS_NOT_IN_RESET);      
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->TGCR, 
                                   WatchdogTimer_TGCR_TIMMODE_MASK, 
                                   WatchdogTimer_TGCR_TIMMODE_SHIFT, 
                                   (uint32_t) WatchdogTimer_TGCR_TIMMODE_64BIT_WD_MODE);      

    /* Configure the watchdog timer period */
    wdRegOverlay->PRDLO = wdCfg->wdPeriodLo;
    wdRegOverlay->PRDHI = wdCfg->wdPeriodHi;

    /* Clear the count registers to zero */
    wdRegOverlay->CNTLO = 0;
    wdRegOverlay->CNTHI = 0;

    /* Enable timer mode: Only write ENAMODE_LO */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->TCR, 
                                   WatchdogTimer_TCR_ENAMODE_LO_MASK, 
                                   WatchdogTimer_TCR_ENAMODE_LO_SHIFT, 
                                   (uint32_t) WatchdogTimer_TCR_ENAMODE_LO_CONTINUOUS_INC);      

    /* Enable watchdog timer mode, moves the watchdog timer into the pre-active state */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR, 
                                   WatchdogTimer_WDTCR_WDEN_MASK, 
                                   WatchdogTimer_WDTCR_WDEN_SHIFT, 
                                   (uint32_t) WatchdogTimer_WDTCR_WATCHDOG_ENABLE);     
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR,
                                   WatchdogTimer_WDTCR_WDKEY_MASK, 
                                   WatchdogTimer_WDTCR_WDKEY_SHIFT, 
                                   (uint32_t) WatchdogTimer_WDTCR_WDKEY_KEY1);         
    WatchdogTimer_module->wdTimerState = WatchdogTimer_wdStates_PRE_ACTIVE;

    /* Activate the timer */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR, 
                                   WatchdogTimer_WDTCR_WDKEY_MASK, 
                                   WatchdogTimer_WDTCR_WDKEY_SHIFT, 
                                   (uint32_t) WatchdogTimer_WDTCR_WDKEY_KEY2);      
    WatchdogTimer_module->wdTimerState = WatchdogTimer_wdStates_ACTIVE;

    return (WatchdogTimer_Status_OKAY);
}

/**
 *  @b Description
 *  @n 
 *      This function is called to perform a full reset of the watchdog timer count.  It will
 *      move the watchdog timer from the Active state to the Service state then back
 *      to the Active state in order to reset the watchdog timer.
 *
 *  @retval
 *      WatchdogTimer_Status - For more information on status return values please see
 *                                           WatchdogTimer.xdc
 */
WatchdogTimer_Status WatchdogTimer_ResetTimer(void)
{
    /* Reset the timer by moving it to the service state then back to the active state */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR, 
                                   WatchdogTimer_WDTCR_WDKEY_MASK, 
                                   WatchdogTimer_WDTCR_WDKEY_SHIFT, 
                                   (uint32_t) WatchdogTimer_WDTCR_WDKEY_KEY1);            
    WatchdogTimer_module->wdTimerState = WatchdogTimer_wdStates_SERVICE;
    /* Reset timer count value */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR, 
                                   WatchdogTimer_WDTCR_WDKEY_MASK, 
                                   WatchdogTimer_WDTCR_WDKEY_SHIFT, 
                                   (uint32_t) WatchdogTimer_WDTCR_WDKEY_KEY2);            
    WatchdogTimer_module->wdTimerState = WatchdogTimer_wdStates_ACTIVE;

    return (WatchdogTimer_Status_OKAY); 
}

/**
 *  @b Description
 *  @n 
 *      This function is called to move the Watchdog timer from the Active state to
 *      the Service state.  This will NOT reset the watchdog timer count.  A call to this
 *      function must be followed by a call to the WatchdogTimer_MoveToActiveState 
 *      function in order to fully reset the watchdog timer count.
 *
 *  @retval
 *      WatchdogTimer_Status - For more information on status return values please see
 *                                           WatchdogTimer.xdc
 */
WatchdogTimer_Status WatchdogTimer_MoveToServiceState(void)
{
    if (WatchdogTimer_module->wdTimerState == WatchdogTimer_wdStates_SERVICE)
    {
        return (WatchdogTimer_Status_ERROR_ALREADY_IN_SERVICE_STATE);
    }

    /* Move the timer to the service state */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR, 
                                   WatchdogTimer_WDTCR_WDKEY_MASK, 
                                   WatchdogTimer_WDTCR_WDKEY_SHIFT, 
                                   (uint32_t) WatchdogTimer_WDTCR_WDKEY_KEY1);            
    WatchdogTimer_module->wdTimerState = WatchdogTimer_wdStates_SERVICE;

    return (WatchdogTimer_Status_OKAY);     
}

/**
 *  @b Description
 *  @n  
 *      This function is called to move the Watchdog timer from the Service state to
 *      the Active state.  This will reset the watchdog timer count as long as the watchdog
 *      timer resides in the Service state at the time of calling this API.  A call to this
 *      function must be preceeded by a call to the WatchdogTimer_MoveToServiceState 
 *      function in order to fully reset the watchdog timer count.
 *
 *  @retval
 *      Not Applicable    
 */
WatchdogTimer_Status WatchdogTimer_MoveToActiveState(void)
{
    if (WatchdogTimer_module->wdTimerState == WatchdogTimer_wdStates_ACTIVE)
    {
        return (WatchdogTimer_Status_ERROR_ALREADY_IN_ACTIVE_STATE);
    }

    /* Reset the timer by moving it back to the active state */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR, 
                                   WatchdogTimer_WDTCR_WDKEY_MASK, 
                                   WatchdogTimer_WDTCR_WDKEY_SHIFT, 
                                   (uint32_t) WatchdogTimer_WDTCR_WDKEY_KEY2);            
    WatchdogTimer_module->wdTimerState = WatchdogTimer_wdStates_ACTIVE;

    return (WatchdogTimer_Status_OKAY);     
}

/**
 *  @b Description
 *  @n  
 *      This function is called to force a timeout of the watchdog timer by writing an
 *      invalid value to the watchdog timer control registers
 */
void WatchdogTimer_ForceTimeout(void)
{
    /* Force a timeout by writing a bad key value to WDKEY.  Does't matter what state
      * the watchdog timer is in. */
    WatchdogTimer_REG_FIELD_WRITE(wdRegOverlay->WDTCR, 
                                   WatchdogTimer_WDTCR_WDKEY_MASK, 
                                   WatchdogTimer_WDTCR_WDKEY_SHIFT, 
                                   (uint32_t) 0xFFFF);
}

/**
 *  @b Description
 *  @n  
 *      This function is called to check the Watchdog Timer control registers to see if
 *      a timeout has occurred.  This function should be used to check for a timeout 
 *      when the Watchdog Timer module has not been hooked up to the BIOS
 *      Exception module.
 *
 *  @retval
 *      TRUE - A Watchdog Timer timeout has occurred.
 *  @retval
 *      FALSE - A Watchdog Timer timeout has not occurred.
 */
Bool WatchdogTimer_HasTimeoutOccurred (void)
{
    /* If the WDFLAG bit is set in WDTCR register a timeout has occurred */
    if ((wdRegOverlay->WDTCR & (WatchdogTimer_WDTCR_WDFLAG_MASK << WatchdogTimer_WDTCR_WDFLAG_SHIFT)) >>
         WatchdogTimer_WDTCR_WDFLAG_SHIFT)
    {
        return(TRUE);
    }

    /* Timeout has not occurred.  The watchdog is still running */
    return(FALSE);
}

/**
@}
*/

/**
@}
*/

