/** 
 *   @file  test_wimax.c
 *
 *   @brief  
 *      Runs WiMax Uplink (UL) and Downlink (DL) test cases using BCP driver APIs. 
 *      The test inputs are generated randomly and dont use predetermined values.
 * 
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2010, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "bcp_example.h"
#include <stdlib.h>

#define     TX_DATA_BUFFER_SIZE             2048 * 3
#define     RX_DATA_BUFFER_SIZE             2048
#define     TX_NUM_DESC                     2
#define     RX_NUM_DESC                     (BCP_EXAMPLE_NUM_HOST_DESC/4 - TX_NUM_DESC)
#define     RX_Q_NUM                        900 + CSL_chipReadReg (CSL_CHIP_DNUM)
#define     RX_FLOW_ID                      0 + CSL_chipReadReg (CSL_CHIP_DNUM)

extern UInt32   totalNumTestsPass, totalNumTestsFail;

/* When set to 1, it can create a random test vector input to run. 
 * Unfortunately since we dont have reference output for each test
 * vector combination. No validation can be done on BCP output generated
 * using random test vector data.
 */
#undef      USE_RANDOM_TEST_VECTOR

#pragma DATA_SECTION (seed, ".testData");
UInt32 seed = 200;

#pragma DATA_SECTION (numSlotsWoRpt, ".testData");
Int16       numSlotsWoRpt;
#pragma DATA_SECTION (MCR, ".testData");
UInt8       MCR;      //downlink modualtion and coding rate, 0,1,2,3,4,5,6,7
#pragma DATA_SECTION (codingRptFactor, ".testData");
UInt8       codingRptFactor;  //1,2,4,6
#pragma DATA_SECTION (numSlots, ".testData");
UInt16      numSlots;  //number of allocated data slots
#pragma DATA_SECTION (stcRate, ".testData");
UInt8       stcRate;   //1,2 for downlink. 

#pragma DATA_SECTION (numBitsPerSlot, ".testData");
UInt32      numBitsPerSlot;
#pragma DATA_SECTION (modType, ".testData");
UInt8       modType;   //modulation type
#pragma DATA_SECTION (codingRate, ".testData");
UInt8       codingRate; //coding rate
#pragma DATA_SECTION (totalNumInforBits, ".testData");
UInt32      totalNumInforBits;

//allocatedSlots * STCrate/(repetition factor * number of stc layers) 
//for vertical encoding, number of stc layers is always 1, used numSlots is STC rate * allocated Slots /repetition
//for horical encoding, number of stc rate is equal to number of stc layers, totally, there are number of layers: allocated Slots /repetition , 
#pragma DATA_SECTION (numSlotsWoRpt, ".testData");
Int16       numSlotsWoRpt;
	
#pragma DATA_SECTION (rms, ".testData");
UInt16      rms;
#pragma DATA_SECTION (tbSize, ".testData");
UInt32      tbSize;
#pragma DATA_SECTION (harqFlag, ".testData");
UInt8       harqFlag;
#pragma DATA_SECTION (awgnFlag, ".testData");
UInt8       awgnFlag;
#pragma DATA_SECTION (snr, ".testData");
Int8        snr;
#pragma DATA_SECTION (qformat, ".testData");
UInt8       qformat;

typedef struct
{
	Int16 numTotalFecBlks;
	Int16 numFecBlks1;
	Int16 numFecBlks2;
	Int16 numFecBlks3;
	Int16 fecCtcBlkLen[3];
	UInt16 P0[3];
	UInt16 P1[3];
	UInt16 P2[3];
	UInt16 P3[3];
	UInt8 M[3];
	UInt8 J[3];

}WIMAX_FECCTC_BLOCKS;

#pragma DATA_SECTION (fecBlocks, ".testData");
WIMAX_FECCTC_BLOCKS fecBlocks;

//const enum gModtype  {QPSK = 2, QAM16 = 4, QAM64 = 6};
#define QPSK 2
#define QAM16 4
#define QAM64 6

/****************************************************************
*  column 0: numBitsPerCarrier
*  column 1: rateTimes120 
*  column 2: slot concatenation parameter 

*************************************************************/
#define CTC_MCR_TABLE_NUM       32

#pragma DATA_SECTION (CTC_MCR_TABLE, ".testData");
static const Int32 CTC_MCR_TABLE[][3] = {
	{2,  60, 10},
    {2,  90, 6},
    {4,  60, 5},
    {4,  90, 3},
    {6,  60, 3},
    {6,  80, 2},
    {6,  90, 2},
    {6,  100,2}


};

/* Table 326 - Optimal CTC channel coding per Modulation */
#pragma DATA_SECTION (gCTCChCodingPerMod, ".testData");
static const UInt8 gCTCChCodingPerMod[CTC_MCR_TABLE_NUM][9] = {
/* Modulation, Data Block Size, Encoded Data Block Size, Code Rate, N, P0, P1, P2, P3 */
	{ QPSK,  6,  12,  60,  24,  5,   0,  0,   0},
	{ QPSK, 12,  24,  60,  48, 13,  24,  0,  24},
	{ QPSK, 18,  36,  60,  72, 11,   6,  0,   6},
	{ QPSK, 24,  48,  60,  96,  7,  48, 24,  72},
	{ QPSK, 30,  60,  60, 120, 13,  60,  0,  60},
	{ QPSK, 36,  72,  60, 144, 17,  74, 72,   2},
	{ QPSK, 48,  96,  60, 192, 11,  96, 48, 144},
	{ QPSK, 54, 108,  60, 216, 13, 108,  0, 108},
	{ QPSK, 60, 120,  60, 240, 13, 120, 60, 180},
	{ QPSK,  9,  12,  90,  36, 11,  18,  0,  18},
	{ QPSK, 18,  24,  90,  72, 11,   6,  0,   6},
	{ QPSK, 27,  36,  90, 108, 11,  54, 56,   2},
	{ QPSK, 36,  48,  90, 144, 17,  74, 72,   2},
	{ QPSK, 45,  60,  90, 180, 11,  90,  0,  90},
	{ QPSK, 54,  72,  90, 216, 13, 108,  0, 108},
	{QAM16, 12,  24,  60,  48, 13,  24,  0,  24},
	{QAM16, 24,  48,  60,  96,  7,  48, 24,  72},
	{QAM16, 36,  72,  60, 144, 17,  74, 72,   2},
	{QAM16, 48,  96,  60, 192, 11,  96, 48, 144},
	{QAM16, 60, 120,  60, 240, 13, 120, 60, 180},
	{QAM16, 18,  24,  90,  72, 11,   6,  0,   6},
	{QAM16, 36,  48,  90, 144, 17,  74, 72,   2},
	{QAM16, 54,  72,  90, 216, 13, 108,  0, 108},
	{QAM64, 18,  36,  60,  72, 11,   6,  0,   6},
	{QAM64, 36,  72,  60, 144, 17,  74, 72,   2},
	{QAM64, 54, 108,  60, 216, 13, 108,  0, 108},
	{QAM64, 24,  36,  80,  96,  7,  48, 24,  72},
	{QAM64, 48,  72,  80, 192, 11,  96, 48, 144},
	{QAM64, 27,  36,  90, 108, 11,  54, 56,   2},
	{QAM64, 54,  72,  90, 216, 13, 108,  0, 108},
	{QAM64, 30,  36, 100, 120, 13,  60,  0,  60},
	{QAM64, 60,  72, 100, 240, 13, 120, 60, 180}
};

#pragma DATA_SECTION (subBlockInterleaverTable, ".testData");
static const UInt8 subBlockInterleaverTable[12][3] = {
	{  24, 3, 3},
	{  36, 4, 3},
	{  48, 4, 3},
	{  72, 5, 3},
	{  96, 5, 3},
	{ 108, 5, 4},
	{ 120, 6, 2},
	{ 144, 6, 3},
	{ 180, 6, 3},
	{ 192, 6, 3},	
	{ 216, 6, 4},
	{ 240, 7, 2}
};

#pragma DATA_SECTION (prbList, ".testData");
static UInt8 prbList[45]={1, 2, 3, 4, 5, 6, 8, 9, 10,12,15,16, 18,20,24,25, 27,30,32,36, 40,45,48,50, 54,60,64,72, 75,80,
81,90,96,100,108,120,128,144,150,160,162,180,192,200,216};

#pragma DATA_SECTION (harqInput, ".testData");
static UInt32 harqInput [1024];

#pragma DATA_SECTION (harqOutput, ".testData");
static UInt32 harqOutput [1024];

/* Reference Output Data for the test */
#define WIMAX_DL_OUTPUT_PKT_1_WRD_SIZE  6144

#pragma DATA_SECTION (wimax_dl_out_packet_1, ".testData");
static UInt32 wimax_dl_out_packet_1[WIMAX_DL_OUTPUT_PKT_1_WRD_SIZE] = {
0x04020305,
0x05030402,
0x02050502,
0x03030305,
0x04050202,
0x03020205,
0x02040504,
0x02050505,
0x05040302,
0x03020405,
0x04050502,
0x04050502,
0x02020502,
0x03030204,
0x02030202,
0x03050403,
0x05020403,
0x04040403,
0x05050202,
0x05050204,
0x02050304,
0x02030402,
0x04020305,
0x03030404,
0x02030405,
0x03040405,
0x02030404,
0x04020304,
0x04030305,
0x03020405,
0x02050203,
0x04040405,
0x02050404,
0x04020402,
0x02050204,
0x02020304,
0x03030405,
0x04040303,
0x05020404,
0x04040503,
0x02020205,
0x04050204,
0x05040304,
0x03050502,
0x02050304,
0x04030205,
0x04020505,
0x02050202,
0x03040303,
0x03020402,
0x03050303,
0x02040202,
0x02040405,
0x05050503,
0x02040402,
0x03020404,
0x02020203,
0x02030205,
0x04050302,
0x05050505,
0x04030404,
0x03020502,
0x03020204,
0x05040504,
0x04030505,
0x02030404,
0x04040305,
0x04050205,
0x02030205,
0x04050205,
0x02050202,
0x04030503,
0x03040402,
0x05040502,
0x02040404,
0x03050402,
0x04040504,
0x04040402,
0x04050304,
0x05050304,
0x04020402,
0x02040503,
0x03030203,
0x04050202,
0x04050503,
0x02020505,
0x02020403,
0x02030305,
0x04040403,
0x05030305,
0x02020505,
0x02020404,
0x02020305,
0x03030402,
0x02050305,
0x03040204,
0x03020505,
0x05020402,
0x02020202,
0x02030404,
0x03030302,
0x05050505,
0x04030205,
0x02050503,
0x04020304,
0x02020205,
0x04020204,
0x04040302,
0x05050303,
0x05040505,
0x05040502,
0x05050405,
0x02020504,
0x05050203,
0x02020404,
0x03040305,
0x03050504,
0x03020405,
0x04020205,
0x02020505,
0x04050405,
0x03030402,
0x05040303,
0x04030303,
0x04050202,
0x03020502,
0x03040305,
0x05030403,
0x04030305,
0x03050402,
0x04040303,
0x02020503,
0x05050505,
0x04030405,
0x03030402,
0x03050203,
0x05020403,
0x05050403,
0x05050202,
0x05050204,
0x02050204,
0x02030402,
0x04020303,
0x05050305,
0x05030405,
0x02040405,
0x02050303,
0x04030204,
0x03020305,
0x05020502,
0x02040403,
0x04040405,
0x03040204,
0x02020402,
0x02050204,
0x02020202,
0x03030404,
0x05050205,
0x03040305,
0x03040205,
0x05050503,
0x03030503,
0x04050402,
0x04020204,
0x04020203,
0x03040302,
0x03050202,
0x05020505,
0x04030404,
0x04050305,
0x04020504,
0x05030505,
0x05030302,
0x02040504,
0x02030305,
0x04050303,
0x05030505,
0x05040402,
0x05030405,
0x02020302,
0x03030303,
0x03020304,
0x03040503,
0x03050305,
0x04030504,
0x05020205,
0x05050403,
0x04030305,
0x03050505,
0x03020504,
0x03020504,
0x04050305,
0x03030502,
0x03050503,
0x04020403,
0x02030404,
0x02020205,
0x02050504,
0x05050405,
0x03050503,
0x02050304,
0x02030203,
0x05040402,
0x04050505,
0x05050402,
0x03020203,
0x05020205,
0x05020202,
0x03020204,
0x05050304,
0x02020505,
0x05020504,
0x03050402,
0x02030305,
0x02030403,
0x02020403,
0x04030305,
0x02020503,
0x03030503,
0x02020204,
0x04050403,
0x05020505,
0x04040503,
0x02030405,
0x02040302,
0x05050204,
0x05030305,
0x03030405,
0x04040205,
0x05020503,
0x05020302,
0x05040302,
0x02020404,
0x02050302,
0x03030203,
0x04040404,
0x02040502,
0x04040304,
0x04040403,
0x04020303,
0x04050402,
0x03040302,
0x05020203,
0x04020502,
0x03020205,
0x05030502,
0x05030205,
0x03020202,
0x02030405,
0x04050302,
0x03020205,
0x03020205,
0x05050203,
0x04040503,
0x05040505,
0x03050203,
0x05030403,
0x05050403,
0x05050402,
0x03030204,
0x04050204,
0x02030402,
0x03050302,
0x04040305,
0x05030403,
0x02050302,
0x04040303,
0x03030204,
0x03020402,
0x05040302,
0x02040405,
0x04050205,
0x03040304,
0x02030202,
0x04030205,
0x04020202,
0x05050404,
0x02020204,
0x02030502,
0x05030202,
0x04040204,
0x05040302,
0x03020503,
0x03030503,
0x03040403,
0x04030405,
0x04020505,
0x02050202,
0x03040503,
0x05040402,
0x03050203,
0x02040202,
0x05040405,
0x05020203,
0x05040402,
0x03030402,
0x04050202,
0x02030305,
0x02040505,
0x05030405,
0x04030403,
0x05020202,
0x03040303,
0x02050504,
0x03020303,
0x02030403,
0x04020205,
0x02040305,
0x02050205,
0x02030304,
0x05050505,
0x02030505,
0x03020404,
0x02020302,
0x03050202,
0x05040202,
0x04050503,
0x03020402,
0x02050402,
0x04020302,
0x02020502,
0x02020402,
0x05050305,
0x02040503,
0x04030202,
0x02030503,
0x05040303,
0x05020305,
0x04050302,
0x04020405,
0x03030502,
0x03030204,
0x03040205,
0x03040305,
0x05050302,
0x04020405,
0x04030302,
0x04020402,
0x03030404,
0x04040505,
0x05050403,
0x03030303,
0x03030503,
0x03050504,
0x04020505,
0x02050502,
0x05030303,
0x05040404,
0x03040505,
0x05020302,
0x03040202,
0x03020304,
0x04030503,
0x05030304,
0x02050203,
0x02020403,
0x05050203,
0x05030504,
0x02040404,
0x04020404,
0x03050302,
0x02040305,
0x03030205,
0x04040404,
0x03020505,
0x04050505,
0x05030402,
0x02040302,
0x03040402,
0x04020305,
0x04030404,
0x05050204,
0x02020402,
0x03040302,
0x04040305,
0x05050203,
0x05030405,
0x05050405,
0x05050502,
0x03030204,
0x04050205,
0x02030202,
0x03050302,
0x04040503,
0x03040302,
0x04030302,
0x05040503,
0x03050503,
0x03050402,
0x02050302,
0x04040505,
0x04050203,
0x05040302,
0x02030304,
0x04030405,
0x05040203,
0x05050204,
0x04050204,
0x05020204,
0x02040503,
0x02020303,
0x04050404,
0x05040204,
0x03040404,
0x04030304,
0x03040302,
0x03050202,
0x05020503,
0x04030304,
0x02030305,
0x04020504,
0x05030305,
0x02030302,
0x02050502,
0x02050405,
0x02040305,
0x02020305,
0x05020402,
0x05030202,
0x04050302,
0x04030202,
0x04020205,
0x04040202,
0x03030304,
0x03050504,
0x02020405,
0x05050503,
0x04050403,
0x03020305,
0x03020504,
0x03040404,
0x05050304,
0x04050504,
0x03040203,
0x03050505,
0x04020503,
0x03030305,
0x04030204,
0x02040305,
0x03040302,
0x04040404,
0x03040403,
0x02040302,
0x05030302,
0x02030205,
0x04050205,
0x05040305,
0x04040204,
0x03030503,
0x03040305,
0x05030205,
0x04020402,
0x04020504,
0x05050504,
0x04050302,
0x03030302,
0x04040304,
0x02020204,
0x03040205,
0x03040203,
0x02050204,
0x04050202,
0x04030504,
0x05020505,
0x02030404,
0x02020304,
0x03030405,
0x03020305,
0x02050402,
0x04020503,
0x02030305,
0x02050403,
0x03040302,
0x02030305,
0x04050504,
0x02020504,
0x05030203,
0x03020302,
0x02020503,
0x05030305,
0x03020302,
0x04040405,
0x02050504,
0x03050404,
0x03050505,
0x02040205,
0x04030402,
0x04050505,
0x04040302,
0x03040405,
0x04050502,
0x04050502,
0x02020504,
0x05030205,
0x02030202,
0x05050203,
0x05030505,
0x05030405,
0x04050502,
0x03030204,
0x04040405,
0x03030202,
0x03030302,
0x02030402,
0x02040302,
0x05030303,
0x05030404,
0x05040503,
0x04050402,
0x02050405,
0x04020505,
0x02050303,
0x05020303,
0x03030304,
0x04020405,
0x05050403,
0x04050204,
0x05030404,
0x03050303,
0x03050204,
0x05050502,
0x03020503,
0x02030403,
0x04020204,
0x05020403,
0x04030405,
0x04020405,
0x02050204,
0x03040403,
0x05040402,
0x03040203,
0x03040402,
0x05020405,
0x03020204,
0x05020302,
0x04030403,
0x05050504,
0x04040305,
0x03040505,
0x03020202,
0x04020202,
0x03030203,
0x03020403,
0x03050404,
0x02020504,
0x04020405,
0x02030204,
0x02040205,
0x03020405,
0x02020202,
0x03030505,
0x05050205,
0x05050203,
0x05030305,
0x02040202,
0x04030205,
0x02020404,
0x04040504,
0x02050203,
0x05030503,
0x03050505,
0x03040402,
0x03040303,
0x02020302,
0x05020202,
0x03040405,
0x03020203,
0x02040504,
0x03020305,
0x03040504,
0x03040403,
0x05040204,
0x05050303,
0x03040405,
0x02020403,
0x03050405,
0x05030202,
0x04030502,
0x03030303,
0x05040403,
0x05030502,
0x05020503,
0x02040202,
0x02040504,
0x03050405,
0x04030405,
0x05030204,
0x04030305,
0x04050503,
0x04050502,
0x04040403,
0x02040403,
0x05020303,
0x03040404,
0x02020404,
0x04050402,
0x02050305,
0x02030303,
0x04020405,
0x02020204,
0x04020305,
0x05030405,
0x02050502,
0x03030305,
0x04050402,
0x03020205,
0x02040504,
0x05020505,
0x04030305,
0x03050402,
0x03040303,
0x03020503,
0x05050305,
0x04030405,
0x03030502,
0x05050202,
0x04050505,
0x05030405,
0x04050503,
0x03030304,
0x04040405,
0x03030204,
0x04020302,
0x03030402,
0x02040404,
0x05040405,
0x03030404,
0x04040505,
0x04030305,
0x02020405,
0x05020504,
0x03030303,
0x05020303,
0x03050305,
0x05020503,
0x05050403,
0x04040302,
0x02020404,
0x04030402,
0x04030502,
0x04020204,
0x05040204,
0x03020505,
0x03050503,
0x03050404,
0x03040303,
0x03030302,
0x05020503,
0x04030304,
0x02030205,
0x04030504,
0x04030305,
0x02040302,
0x05050503,
0x02050403,
0x03020404,
0x02020203,
0x02030404,
0x04050302,
0x04050505,
0x04040404,
0x03050305,
0x05050402,
0x05040303,
0x04040305,
0x03040303,
0x03050503,
0x05020502,
0x05050402,
0x02050305,
0x04050205,
0x05030203,
0x03020504,
0x02030402,
0x05040305,
0x04040403,
0x02040503,
0x04030402,
0x05030303,
0x02030504,
0x05040305,
0x05020503,
0x02030204,
0x05050203,
0x02020502,
0x02040504,
0x02030303,
0x03040203,
0x02040202,
0x03040203,
0x04050204,
0x05050205,
0x02040503,
0x04040405,
0x05020205,
0x02030503,
0x02040204,
0x04050202,
0x03050504,
0x04050204,
0x04050503,
0x03050202,
0x03020403,
0x02020402,
0x05050302,
0x05020203,
0x02030505,
0x04050304,
0x03020402,
0x03040305,
0x03050504,
0x04020402,
0x05050204,
0x02040305,
0x03030305,
0x04020403,
0x02030204,
0x03020503,
0x03020505,
0x05040404,
0x04020405,
0x03030402,
0x05040303,
0x04030303,
0x04050202,
0x03020502,
0x03040305,
0x03020403,
0x03030405,
0x04030304,
0x03020205,
0x03020205,
0x05030203,
0x02040502,
0x05040505,
0x05050202,
0x04050505,
0x05030205,
0x02050503,
0x03030304,
0x04040403,
0x03030205,
0x04020202,
0x03050205,
0x04030405,
0x02040405,
0x02050205,
0x04030204,
0x05020305,
0x05020405,
0x05020304,
0x03030302,
0x04030503,
0x05050305,
0x05020503,
0x05050505,
0x04040303,
0x05050502,
0x05040205,
0x02040503,
0x03030503,
0x02030405,
0x04050204,
0x02040204,
0x04030304,
0x04030404,
0x04040405,
0x02050404,
0x05040403,
0x05040502,
0x03040205,
0x03040402,
0x05030405,
0x02040404,
0x03030305,
0x04050303,
0x05030505,
0x05040402,
0x03030405,
0x02020202,
0x05040405,
0x03050405,
0x04040204,
0x03050504,
0x05040402,
0x04020305,
0x03050505,
0x03020502,
0x05040304,
0x05050504,
0x04020303,
0x04030202,
0x05050205,
0x03040504,
0x02050202,
0x04040204,
0x05030202,
0x03030202,
0x04020404,
0x02040504,
0x04050504,
0x02050505,
0x05020302,
0x03020505,
0x02040203,
0x05030205,
0x02020203,
0x02030203,
0x03020402,
0x02020205,
0x05050503,
0x02020504,
0x05050204,
0x04040203,
0x05040503,
0x02050403,
0x03030502,
0x02020202,
0x05040403,
0x03030402,
0x02020304,
0x04030504,
0x05050504,
0x05050502,
0x05040504,
0x04050303,
0x05020405,
0x04050403,
0x04040203,
0x03040402,
0x03030504,
0x03050204,
0x03050502,
0x04030503,
0x02030505,
0x05030504,
0x05040503,
0x04050203,
0x03050203,
0x02020503,
0x04050402,
0x03020302,
0x05020203,
0x04020502,
0x03020205,
0x05030502,
0x03020205,
0x02050202,
0x03040402,
0x03020305,
0x04030404,
0x04050404,
0x02020402,
0x03040302,
0x04040203,
0x05040204,
0x04050305,
0x05030305,
0x02040503,
0x03030305,
0x04040403,
0x03050205,
0x02040202,
0x04040305,
0x05030405,
0x02040502,
0x02040303,
0x02030204,
0x03020202,
0x05040302,
0x05030302,
0x03020502,
0x04030403,
0x05040505,
0x03040502,
0x03050505,
0x04020303,
0x03030503,
0x02050503,
0x03030204,
0x05050404,
0x03020503,
0x02030503,
0x04030303,
0x03040403,
0x03040303,
0x03030302,
0x05020203,
0x02020304,
0x02030205,
0x04030502,
0x04050305,
0x04040302,
0x05020203,
0x05040402,
0x03020402,
0x02050202,
0x02030305,
0x02040305,
0x05030405,
0x02030503,
0x02040405,
0x03020503,
0x03030205,
0x04040203,
0x03030505,
0x02050304,
0x04050203,
0x03020304,
0x05030502,
0x03030305,
0x04050403,
0x02020304,
0x03030203,
0x04040405,
0x02030204,
0x05020304,
0x04050503,
0x02050203,
0x02020203,
0x04020404,
0x04050504,
0x05030205,
0x05020402,
0x03050505,
0x05030303,
0x05040403,
0x03040303,
0x05040403,
0x04040203,
0x02050203,
0x03040505,
0x04050502,
0x03050204,
0x05050302,
0x05030205,
0x05020204,
0x03030504,
0x04050403,
0x04050205,
0x02030405,
0x02030304,
0x04040505,
0x03050302,
0x04030503,
0x05030502,
0x03040203,
0x02030205,
0x04030402,
0x05020202,
0x05040304,
0x05020202,
0x04030305,
0x02050403,
0x03020302,
0x03020403,
0x02050405,
0x05030504,
0x02050405,
0x05050505,
0x03050302,
0x02040305,
0x03030205,
0x04040402,
0x03020505,
0x04050505,
0x05030203,
0x05050302,
0x04040302,
0x03040403,
0x04050502,
0x04050503,
0x02040504,
0x05030205,
0x02030402,
0x05040304,
0x04050305,
0x03030305,
0x02040503,
0x03030505,
0x05020403,
0x05050205,
0x03050202,
0x04040303,
0x05020302,
0x04030302,
0x05040303,
0x03030403,
0x03040402,
0x04050302,
0x03030202,
0x03020504,
0x02030405,
0x05040405,
0x03040302,
0x02030504,
0x02020303,
0x02040503,
0x04020404,
0x04020503,
0x02020205,
0x04050204,
0x05020304,
0x03050503,
0x02050504,
0x04030504,
0x04040405,
0x02050504,
0x05050403,
0x05040502,
0x02040205,
0x05020402,
0x02030405,
0x02050502,
0x02030405,
0x02040305,
0x02020505,
0x05040402,
0x05030202,
0x02050302,
0x05050503,
0x05020202,
0x03020504,
0x03050404,
0x03030503,
0x05050503,
0x05040404,
0x04040403,
0x02020503,
0x04050202,
0x02020302,
0x05040402,
0x02030303,
0x05030503,
0x03030204,
0x05040302,
0x03030502,
0x03020402,
0x05030403,
0x05050405,
0x04020403,
0x03020302,
0x02040504,
0x03030504,
0x03030304,
0x02030502,
0x03040204,
0x03040205,
0x02030502,
0x02040205,
0x05030405,
0x04040302,
0x04050303,
0x02040403,
0x04050402,
0x05020503,
0x02030203,
0x03020202,
0x04040405,
0x02040303,
0x04050305,
0x03030403,
0x02040503,
0x04020202,
0x04050202,
0x02040502,
0x02020505,
0x04030305,
0x02050404,
0x02040205,
0x03040304,
0x03030505,
0x05030405,
0x04050505,
0x02020302,
0x04050502,
0x04030205,
0x03040503,
0x04020302,
0x03020502,
0x03020302,
0x04040405,
0x02030404,
0x03050404,
0x03050505,
0x02040205,
0x04030402,
0x05020505,
0x04030305,
0x04050202,
0x03040303,
0x03020303,
0x05050305,
0x04030505,
0x03030504,
0x05020304,
0x02050305,
0x03030304,
0x02040505,
0x03020505,
0x05020403,
0x05050205,
0x03050402,
0x02020402,
0x03040302,
0x05030302,
0x05020404,
0x03040503,
0x04050402,
0x02050303,
0x03050202,
0x05020404,
0x02030404,
0x04040403,
0x03050302,
0x02020304,
0x03020503,
0x05050303,
0x03040305,
0x02040405,
0x03050503,
0x02030503,
0x04050202,
0x04020204,
0x04020305,
0x03020203,
0x05030302,
0x05020203,
0x02020304,
0x02020205,
0x05030502,
0x02050305,
0x05040302,
0x03020204,
0x05020302,
0x04030402,
0x05050504,
0x02040305,
0x03040505,
0x03020404,
0x02040403,
0x05030203,
0x02020304,
0x03040203,
0x03020304,
0x04030405,
0x05020305,
0x05040403,
0x04020203,
0x03030305,
0x02050302,
0x05050305,
0x04050503,
0x02050503,
0x04050203,
0x05050203,
0x04050404,
0x03040404,
0x03050405,
0x05030505,
0x05020202,
0x02050405,
0x04050302,
0x03050205,
0x02030305,
0x05030403,
0x05040503,
0x03040304,
0x05040404,
0x02050403,
0x04020205,
0x04040202,
0x04030402,
0x03030305,
0x02040404,
0x02020302,
0x04040202,
0x04040304,
0x03050402,
0x02020302,
0x02050504,
0x04050504,
0x04020403,
0x03050502,
0x04020305,
0x02020203,
0x05040502,
0x03040503,
0x03020202,
0x03040305,
0x02040505,
0x04030302,
0x05030504,
0x03050405,
0x05050204,
0x03020205,
0x04030403,
0x03020405,
0x05050404,
0x05040203,
0x04020305,
0x05030405,
0x02050502,
0x03050205,
0x04050502,
0x03020205,
0x02040504,
0x02020505,
0x03030403,
0x04030304,
0x03020205,
0x03040204,
0x05030202,
0x02040502,
0x05040305,
0x04020304,
0x02050305,
0x03030304,
0x02040505,
0x02020503,
0x05020403,
0x05040305,
0x05020402,
0x03030402,
0x02040302,
0x05020405,
0x03030404,
0x04040503,
0x04050305,
0x02030405,
0x02050203,
0x04040404,
0x02050404,
0x04040402,
0x02050304,
0x02020304,
0x03030405,
0x03020303,
0x02030502,
0x05030204,
0x04040204,
0x05020303,
0x03020503,
0x05030503,
0x03040403,
0x04050504,
0x02040405,
0x02050504,
0x05050403,
0x04050504,
0x02040205,
0x05020402,
0x02040405,
0x05050503,
0x02050405,
0x03040404,
0x02020203,
0x02030402,
0x04030302,
0x04050505,
0x04030405,
0x02020305,
0x04020204,
0x02050304,
0x04050204,
0x03050502,
0x03030502,
0x05050204,
0x04030404,
0x02030502,
0x05020303,
0x04050303,
0x02020203,
0x05040302,
0x04050305,
0x03050302,
0x02020304,
0x03050502,
0x02020403,
0x04040304,
0x04020502,
0x04050402,
0x03030305,
0x05020402,
0x04030204,
0x03030305,
0x04020302,
0x02030203,
0x04030502,
0x04040502,
0x03030502,
0x04050402,
0x03040203,
0x05050204,
0x02050504,
0x02050503,
0x05050402,
0x03030202,
0x03040204,
0x05030502,
0x03030502,
0x05040303,
0x03030205,
0x04040305,
0x02030304,
0x02040405,
0x02020403,
0x03050403,
0x02030302,
0x03020204,
0x02050402,
0x03030502,
0x03030305,
0x03030305,
0x05050302,
0x04030204,
0x05040202,
0x04030403,
0x02020503,
0x05020305,
0x04020405,
0x03030402,
0x05040303,
0x04030303,
0x04050202,
0x03020202,
0x03040305,
0x02050403,
0x03040402,
0x03020505,
0x04030504,
0x04050402,
0x02020502,
0x03040202,
0x04020203,
0x04020504,
0x02050304,
0x02020304,
0x02020505,
0x02020503,
0x05020403,
0x05040305,
0x04020404,
0x03030204,
0x02030405,
0x03040405,
0x02030404,
0x04020304,
0x04020305,
0x05020405,
0x02050403,
0x04040405,
0x03040204,
0x02020402,
0x02050204,
0x02020302,
0x03030404,
0x04040205,
0x05020204,
0x04040503,
0x02020203,
0x04050204,
0x05040304,
0x03040504,
0x04050304,
0x03020203,
0x05030302,
0x04030203,
0x02040304,
0x03020203,
0x05030502,
0x02050305,
0x05030304,
0x02020404,
0x05030305,
0x04050303,
0x05050504,
0x05040502,
0x03030405,
0x02020202,
0x04050503,
0x03030502,
0x05030205,
0x04020403,
0x03040302,
0x03030403,
0x05020203,
0x03040203,
0x04040503,
0x03020504,
0x02020503,
0x05040505,
0x03020504,
0x02030204,
0x02040503,
0x04040504,
0x04030505,
0x03050304,
0x05050505,
0x04020205,
0x04040305,
0x05020205,
0x02020303,
0x02040502,
0x03020505,
0x02050504,
0x03040203,
0x03020404,
0x04050403,
0x05030202,
0x04030404,
0x03040302,
0x03040204,
0x05050203,
0x04030305,
0x05020403,
0x05040202,
0x05030504,
0x05020404,
0x02020402,
0x05030502,
0x05040304,
0x05020202,
0x03020503,
0x05050304,
0x03030202,
0x02040405,
0x05020202,
0x02040305,
0x02040303,
0x03030402,
0x03050504,
0x04030403,
0x05040504,
0x03030202,
0x02020404,
0x03030202,
0x04020205,
0x05040303,
0x05040303,
0x04050402,
0x03030302,
0x05020203,
0x04020502,
0x03020202,
0x05030502,
0x03020205,
0x05050202,
0x04040304,
0x03040403,
0x04050503,
0x04030503,
0x02040505,
0x05030405,
0x02030402,
0x04020504,
0x02050304,
0x02020304,
0x02020305,
0x04040503,
0x03020503,
0x05040305,
0x04040405,
0x05040305,
0x05030405,
0x02040405,
0x02040303,
0x02030204,
0x03020305,
0x05020502,
0x02040405,
0x04040205,
0x03040304,
0x02030202,
0x04030204,
0x04020202,
0x03050404,
0x05030204,
0x03050305,
0x03050204,
0x05050503,
0x03020503,
0x02050402,
0x04020204,
0x05020203,
0x04050504,
0x02040405,
0x03040504,
0x05030403,
0x04050504,
0x04040205,
0x05020402,
0x02040402,
0x05020203,
0x05040402,
0x03020404,
0x02050202,
0x02030305,
0x02040302,
0x05050405,
0x04040304,
0x02040305,
0x04020302,
0x03030205,
0x05020302,
0x03030503,
0x03030205,
0x03040404,
0x04040504,
0x05020505,
0x04050403,
0x03030205,
0x05030404,
0x03030502,
0x03050204,
0x04020303,
0x02030405,
0x05040502,
0x04040304,
0x02030503,
0x05030404,
0x02020403,
0x03050503,
0x04040402,
0x05030205,
0x04020302,
0x03050505,
0x02030503,
0x04020304,
0x02020505,
0x03050202,
0x03050204,
0x05040303,
0x05040502,
0x03020203,
0x02030402,
0x04030205,
0x04040503,
0x04040403,
0x05020204,
0x03050405,
0x02040502,
0x02050305,
0x02050502,
0x02040505,
0x05050302,
0x03020304,
0x05020304,
0x02030404,
0x04050304,
0x04020204,
0x02040203,
0x05040304,
0x05030205,
0x03030303,
0x02050202,
0x02030504,
0x05050204,
0x02030302,
0x05020405,
0x03050402,
0x02040305,
0x03030205,
0x04040402,
0x03020505,
0x04050505,
0x05030203,
0x05020302,
0x04030505,
0x04050202,
0x03040203,
0x03020305,
0x05050205,
0x04030505,
0x03040504,
0x02020504,
0x02040302,
0x02020302,
0x02020205,
0x04040503,
0x03020502,
0x05040305,
0x03050405,
0x04040305,
0x05030503,
0x02030302,
0x04040303,
0x03030202,
0x03040402,
0x05040302,
0x04040405,
0x04050203,
0x05040302,
0x02030302,
0x04030405,
0x05040203,
0x05050404,
0x02020204,
0x04030402,
0x05030302,
0x04020204,
0x05040202,
0x03020505,
0x03050503,
0x03040402,
0x05020203,
0x05020304,
0x04030203,
0x02040304,
0x03020203,
0x03030503,
0x02050305,
0x02030305,
0x02050504,
0x02030205,
0x04040305,
0x03020505,
0x05040402,
0x05030202,
0x02040302,
0x02040403,
0x03020203,
0x05030403,
0x04050504,
0x04030204,
0x04050402,
0x02030204,
0x02030405,
0x04030205,
0x02030203,
0x05020304,
0x02040204,
0x05020203,
0x03020503,
0x05030204,
0x02020204,
0x04030202,
0x04020403,
0x03030302,
0x04030503,
0x05050503,
0x02040505,
0x03020202,
0x03030503,
0x04030505,
0x04030304,
0x03050204,
0x02050302,
0x05050404,
0x04030202,
0x02040502,
0x05020402,
0x04030204,
0x05020403,
0x05050403,
0x02030503,
0x02020205,
0x03020304,
0x03040504,
0x04030402,
0x04040402,
0x03050505,
0x03050503,
0x03040202,
0x04050505,
0x03030402,
0x05020203,
0x05030202,
0x03030305,
0x03030202,
0x02030404,
0x03030504,
0x02020302,
0x04050504,
0x02030405,
0x02030203,
0x03020403,
0x02050405,
0x03040403,
0x03020304,
0x03020302,
0x04040305,
0x02030404,
0x03050404,
0x03050505,
0x02040205,
0x04030402,
0x02020304,
0x03050403,
0x04030305,
0x03020204,
0x03040204,
0x03020202,
0x02040302,
0x05040305,
0x02020504,
0x02040302,
0x02020302,
0x03020205,
0x04040503,
0x03030302,
0x04040505,
0x03050405,
0x04040502,
0x03040302,
0x05030302,
0x05040503,
0x03050503,
0x02050402,
0x02050302,
0x04020505,
0x02050203,
0x05040303,
0x03030304,
0x04020405,
0x05050403,
0x04050204,
0x04050204,
0x05020205,
0x02040503,
0x03030503,
0x02050404,
0x04040204,
0x02040404,
0x04030304,
0x02050504,
0x02050403,
0x03040504,
0x04030403,
0x04050504,
0x04040404,
0x05020402,
0x05040402,
0x05020204,
0x05020302,
0x04030402,
0x05050402,
0x02050305,
0x03040505,
0x03020405,
0x03030403,
0x04040502,
0x05020203,
0x02030405,
0x05040503,
0x03040204,
0x02040204,
0x03020203,
0x03040205,
0x05020204,
0x03040504,
0x04020502,
0x03020503,
0x04050205,
0x02030204,
0x04030505,
0x03030404,
0x03030202,
0x04020305,
0x05050403,
0x05030402,
0x04040504,
0x04030403,
0x02040502,
0x03020203,
0x03020503,
0x03020504,
0x03030203,
0x05040504,
0x03040504,
0x02030205,
0x03020203,
0x02040503,
0x05040505,
0x05040205,
0x04030202,
0x03040504,
0x05050503,
0x04040504,
0x03040305,
0x04040405,
0x04030204,
0x04040505,
0x03040204,
0x03020503,
0x04050504,
0x04040202,
0x05030403,
0x02020202,
0x04050403,
0x04020504,
0x03020205,
0x04020403,
0x05020305,
0x02030303,
0x03030303,
0x02050503,
0x02040504,
0x04040302,
0x02030302,
0x04020305,
0x05030405,
0x02050502,
0x03050205,
0x04050502,
0x02020205,
0x02040502,
0x02050505,
0x03040202,
0x03020405,
0x04030502,
0x04050502,
0x02020502,
0x03040202,
0x04030203,
0x02020505,
0x03020202,
0x02040302,
0x03020204,
0x04040503,
0x03030302,
0x04040505,
0x05030405,
0x03030402,
0x02040302,
0x05030303,
0x05030404,
0x05040503,
0x04050405,
0x02050405,
0x05020504,
0x03030303,
0x05020303,
0x03030305,
0x05020403,
0x05050403,
0x04040304,
0x03030404,
0x02050303,
0x03030204,
0x05050502,
0x03020503,
0x02030403,
0x04030203,
0x03020403,
0x05020203,
0x04020304,
0x04050203,
0x03040305,
0x03020203,
0x03030303,
0x02050305,
0x02050305,
0x05050503,
0x02050405,
0x03040304,
0x02020203,
0x03030402,
0x04030202,
0x04050505,
0x02030305,
0x05040402,
0x04030305,
0x03040403,
0x02050304,
0x04050302,
0x04040403,
0x03040403,
0x04040302,
0x04050402,
0x04040302,
0x04020504,
0x02020505,
0x02030205,
0x05030304,
0x03040504,
0x05040202,
0x02050305,
0x04020504,
0x02040202,
0x03020503,
0x04050203,
0x02040203,
0x02020203,
0x02020502,
0x05040205,
0x05050403,
0x03050405,
0x03050404,
0x04020203,
0x03040203,
0x05030404,
0x03040505,
0x03050303,
0x02040505,
0x03030502,
0x04050404,
0x02020502,
0x02050403,
0x03040203,
0x03030502,
0x03050203,
0x05030405,
0x05020402,
0x03040203,
0x05020403,
0x02040402,
0x05050402,
0x05020405,
0x02040204,
0x04030303,
0x05020302,
0x04030505,
0x04050303,
0x02050402,
0x03020504,
0x05040504,
0x02030502,
0x05030502,
0x04030505,
0x04020405,
0x03030402,
0x05040502,
0x04030303,
0x04050202,
0x03020202,
0x02040305,
0x05050403,
0x04020304,
0x03040402,
0x04040503,
0x04030503,
0x04050505,
0x05030405,
0x03030402,
0x02020505,
0x03020202,
0x02040302,
0x05020204,
0x04040403,
0x03030304,
0x04040502,
0x04020505,
0x03030402,
0x02050404,
0x03040405,
0x02030404,
0x04040305,
0x04030305,
0x02020405,
0x05020304,
0x03030302,
0x04030303,
0x05050305,
0x05020503,
0x05050405,
0x04040303,
0x02040502,
0x04020402,
0x04020502,
0x02020204,
0x04050204,
0x03020305,
0x03050503,
0x03050504,
0x02050504,
0x03050403,
0x03020504,
0x04030402,
0x04050504,
0x04040404,
0x05020405,
0x05030502,
0x02020204,
0x05030304,
0x04050303,
0x05050504,
0x05040503,
0x03020405,
0x03020202,
0x02030402,
0x03020302,
0x04050303,
0x04040402,
0x05040305,
0x05020303,
0x05020305,
0x04040303,
0x05040405,
0x03030204,
0x04040303,
0x03050302,
0x03050305,
0x05050504,
0x02030503,
0x03040404,
0x02020205,
0x02020405,
0x02020504,
0x02050505,
0x04040405,
0x03040405,
0x02050402,
0x03050502,
0x03040403,
0x04050302,
0x03040403,
0x02030304,
0x04030203,
0x04050405,
0x02040305,
0x05040302,
0x03030504,
0x02030402,
0x02050503,
0x04020202,
0x03020205,
0x03040403,
0x04050502,
0x04020305,
0x04030305,
0x05020204,
0x02030204,
0x02020202,
0x04020503,
0x02030402,
0x04020405,
0x05030504,
0x04040502,
0x03030303,
0x05030204,
0x03050202,
0x02030303,
0x03050403,
0x03030303,
0x04020404,
0x05030202,
0x05030504,
0x03030202,
0x03050504,
0x04050402,
0x03030302,
0x05020203,
0x04020303,
0x03020202,
0x05030502,
0x03020205,
0x05020202,
0x02030505,
0x04050302,
0x03040205,
0x03020205,
0x05050205,
0x04040505,
0x03040504,
0x02030503,
0x03020402,
0x02040402,
0x05030204,
0x04040403,
0x03030304,
0x04020502,
0x04020505,
0x03050305,
0x04030405,
0x02040405,
0x02050205,
0x04030204,
0x03020305,
0x05020404,
0x05030302,
0x03030502,
0x04030403,
0x05040305,
0x03040503,
0x03050505,
0x04020303,
0x05050503,
0x05040305,
0x02040505,
0x03050503,
0x02030405,
0x04050202,
0x04040204,
0x04030305,
0x05030205,
0x04020504,
0x04050203,
0x03040305,
0x03020203,
0x03030303,
0x02050302,
0x02040205,
0x05030203,
0x04040402,
0x03020404,
0x02040202,
0x02030305,
0x02040302,
0x05050505,
0x04040203,
0x02020203,
0x04020202,
0x05050303,
0x05020303,
0x03050503,
0x05040304,
0x03020203,
0x02040305,
0x02040303,
0x03020404,
0x05020502,
0x04040403,
0x05030405,
0x05030403,
0x02030302,
0x02030502,
0x02030503,
0x03050502,
0x03020404,
0x02050302,
0x03050202,
0x05040205,
0x03040302,
0x03020503,
0x02020402,
0x05020502,
0x04050502,
0x04030405,
0x03030404,
0x03050502,
0x04020205,
0x02040403,
0x05020205,
0x02020304,
0x02030205,
0x02020204,
0x03030204,
0x03020403,
0x03030504,
0x03050305,
0x04030305,
0x05040204,
0x02050503,
0x03030505,
0x02050404,
0x02040305,
0x05050404,
0x05020402,
0x04020404,
0x03020403,
0x02030302,
0x03050504,
0x04020405,
0x02020202,
0x03030403,
0x02050404,
0x02030304,
0x04020503,
0x05040402,
0x03050402,
0x02040305,
0x03020205,
0x04040402,
0x03020505,
0x04050502,
0x05030203,
0x02040302,
0x03050402,
0x04030305,
0x03030404,
0x05050204,
0x03020202,
0x02040302,
0x04040305,
0x02030503,
0x03020402,
0x04040402,
0x05030204,
0x04040202,
0x02050304,
0x02020502,
0x02050505,
0x04040305,
0x05030405,
0x02040502,
0x02040303,
0x03030204,
0x03020202,
0x05040302,
0x03030302,
0x03020504,
0x02030405,
0x05040405,
0x03040302,
0x02030505,
0x02020303,
0x03020503,
0x02050503,
0x05030204,
0x04050404,
0x03020303,
0x03030503,
0x05030303,
0x03040403,
0x02040502,
0x03050203,
0x05020504,
0x04030402,
0x04050305,
0x04020404,
0x05020405,
0x03030502,
0x02050504,
0x02030305,
0x04050305,
0x05020505,
0x05040402,
0x05030402,
0x02040302,
0x04050304,
0x05050203,
0x02050205,
0x03050205,
0x04050403,
0x04040503,
0x05030505,
0x03020205,
0x02040202,
0x02050305,
0x03050502,
0x05030205,
0x05030505,
0x05040503,
0x04030303,
0x03040404,
0x05040203,
0x05030304,
0x04020204,
0x02050304,
0x02050302,
0x05040405,
0x05040404,
0x05050402,
0x03040204,
0x02030404,
0x04030204,
0x02050202,
0x05030402,
0x03050505,
0x05020402,
0x04050402,
0x02020304,
0x04020303,
0x04030302,
0x05020205,
0x03050303,
0x05020402,
0x02040502,
0x03020402,
0x04020303,
0x05020505,
0x03050503,
0x02030403,
0x02030203,
0x02050204,
0x05030403,
0x05020204,
0x04030405,
0x03040302,
0x04020504,
0x03030404,
0x03020304,
0x02030505,
0x04030302,
0x02050505,
0x05050203,
0x03050202,
0x02030303,
0x04040504,
0x03020302,
0x04040305,
0x02030404,
0x03050404,
0x03020505,
0x02040205,
0x04030402,
0x02050304,
0x05040202,
0x03020405,
0x04050502,
0x04050502,
0x02020502,
0x03030204,
0x04030203,
0x02050403,
0x05020402,
0x04040403,
0x05030202,
0x04040202,
0x02050304,
0x02020502,
0x03050505,
0x04040303,
0x05040302,
0x04030302,
0x05040303,
0x03050403,
0x03040402,
0x02050302,
0x03050202,
0x05020504,
0x02030404,
0x04040403,
0x03040302,
0x02020304,
0x03020503,
0x04040503,
0x05020404,
0x04040503,
0x02020205,
0x04050204,
0x05040304,
0x03050502,
0x04050304,
0x05030205,
0x04020504,
0x02050202,
0x03040303,
0x03020402,
0x03050303,
0x02050302,
0x05040205,
0x05020205,
0x05020302,
0x05030402,
0x05050202,
0x02050305,
0x02040505,
0x03020405,
0x05030402,
0x02050405,
0x02020504,
0x05030403,
0x03050303,
0x05050302,
0x05030203,
0x04050402,
0x05030303,
0x02040302,
0x03040405,
0x04050504,
0x02050303,
0x03030305,
0x05050502,
0x02030304,
0x05030305,
0x05030504,
0x04050502,
0x02050402,
0x03040505,
0x02050504,
0x04050503,
0x03040405,
0x04050202,
0x03020503,
0x05050404,
0x03050404,
0x03040403,
0x04040403,
0x03050505,
0x02020302,
0x02030405,
0x05030304,
0x05040203,
0x03030302,
0x05020503,
0x04030402,
0x02020302,
0x05050205,
0x03030204,
0x04050403,
0x05040303,
0x05030504,
0x05020403,
0x02050205,
0x05050302,
0x03020403,
0x05030505,
0x02020202,
0x02020402,
0x05040502,
0x03030502,
0x05020305,
0x03020303,
0x05030305,
0x05040502,
0x04050502,
0x05040303,
0x03050405,
0x03020305,
0x05030405,
0x02050502,
0x03050205,
0x04050502,
0x02040205,
0x02040502,
0x05030505,
0x04020305,
0x03050402,
0x04040303,
0x02020503,
0x04050505,
0x04030405,
0x03030402,
0x02050403,
0x05020402,
0x04040403,
0x05030202,
0x05050204,
0x02050304,
0x02030402,
0x03050305,
0x02030402,
0x02040302,
0x05030302,
0x05020404,
0x03040503,
0x04050402,
0x02050205,
0x03050203,
0x04040404,
0x02050404,
0x04040402,
0x02050304,
0x02020304,
0x03030403,
0x05050303,
0x03050305,
0x03040205,
0x05050503,
0x03030503,
0x04050402,
0x04020204,
0x04020203,
0x02040502,
0x03050203,
0x05020505,
0x04030404,
0x04050305,
0x04020404,
0x05030505,
0x02030302,
0x04050503,
0x02050405,
0x03040305,
0x02020203,
0x05030402,
0x04030202,
0x04050305,
0x05020505,
0x05020305,
0x04030202,
0x03030304,
0x04040304,
0x05020304,
0x03040204,
0x05020404,
0x03050205,
0x04040404,
0x04030502,
0x02020303,
0x05040402,
0x05030205,
0x04050205,
0x04050505,
0x04020504,
0x02050503,
0x04020202,
0x04020205,
0x03050505,
0x03050202,
0x03020205,
0x03040304,
0x05030503,
0x05030404,
0x03040305,
0x02020204,
0x05020503,
0x03050403,
0x05030204,
0x02030503,
0x05020202,
0x04030503,
0x02050504,
0x03050205,
0x05040204,
0x05020305,
0x02020504,
0x02040202,
0x04040303,
0x03020404,
0x02020205,
0x04020405,
0x05040503,
0x02020204,
0x03020503,
0x03050302,
0x05030304,
0x05050404,
0x04030502,
0x05030505,
0x02050204,
0x05030304,
0x03040505,
0x02040505,
0x03040403,
0x04020404,
0x04050503,
0x04040204,
0x04020405,
0x05030402,
0x05040502,
0x04030303,
0x04050202,
0x03020202,
0x02040305,
0x03020405,
0x02030405,
0x04050302,
0x03020205,
0x03020205,
0x05050205,
0x04040503,
0x05040505,
0x03050203,
0x05020403,
0x05050403,
0x05050202,
0x05050204,
0x02050304,
0x02030402,
0x04020305,
0x03030402,
0x02040304,
0x05020405,
0x03030404,
0x04040505,
0x04050305,
0x02030405,
0x02050403,
0x04040405,
0x03040404,
0x02020402,
0x02050204,
0x02020302,
0x03030404,
0x02020203,
0x02030502,
0x05030202,
0x04040204,
0x05040302,
0x03020505,
0x03030503,
0x03040403,
0x04030405,
0x04020505,
0x02050202,
0x03040303,
0x03020402,
0x03050303,
0x02040202,
0x05030405,
0x02020204,
0x05020304,
0x04030303,
0x05050504,
0x05040303,
0x03040405,
0x03020202,
0x04030402,
0x05050205,
0x04040505,
0x02020203,
0x02030404,
0x02020404,
0x03030203,
0x04050403,
0x02050305,
0x05030505,
0x03040204,
0x05040505,
0x04020303,
0x03030203,
0x02040303,
0x03030304,
0x05020305,
0x03030505,
0x02050403,
0x05020203,
0x05040503,
0x03050202,
0x04040404,
0x03050503,
0x05020503,
0x02040203,
0x04040402,
0x04040204,
0x02030504,
0x05030205,
0x02040202,
0x04040204,
0x03020302,
0x02020404,
0x05040305,
0x03030302,
0x05020302,
0x02050204,
0x05040304,
0x04030503,
0x03040203,
0x03040302,
0x05040403,
0x02050304,
0x05030402,
0x03050304,
0x03050404,
0x05040503,
0x02050204,
0x02040404,
0x05020205,
0x05040405,
0x02020204,
0x04040505,
0x05030202,
0x05030503,
0x03020404,
0x04020404,
0x04030403,
0x02050505,
0x04050404,
0x03030302,
0x05020203,
0x04020303,
0x03020202,
0x05030502,
0x03040305,
0x02040202,
0x03040402,
0x04020305,
0x03030404,
0x05050204,
0x02020402,
0x03040302,
0x04040305,
0x03050203,
0x05020403,
0x05050403,
0x05050202,
0x03030204,
0x04050204,
0x02030402,
0x04020302,
0x03030204,
0x04030405,
0x02040405,
0x02030204,
0x04020204,
0x04020305,
0x05020405,
0x02040403,
0x04040205,
0x03040304,
0x02030402,
0x04030204,
0x04020202,
0x03030404,
0x04050204,
0x05020204,
0x02040503,
0x03020303,
0x04050404,
0x05040204,
0x03040404,
0x04030304,
0x03040302,
0x03050202,
0x05020505,
0x04030404,
0x02030305,
0x04020504,
0x05030505,
0x02040302,
0x05050303,
0x04040402,
0x03020404,
0x02020203,
0x02030305,
0x04040302,
0x05050505,
0x03020202,
0x04050504,
0x04030204,
0x03030204,
0x03050304,
0x03030504,
0x05040202,
0x04050402,
0x03040405,
0x02020202,
0x03050202,
0x04050505,
0x05050403,
0x05030404,
0x03040203,
0x02050202,
0x04050204,
0x04040303,
0x03020505,
0x03050503,
0x04050405,
0x03030304,
0x02050504,
0x05040304,
0x02050403,
0x02020403,
0x05030404,
0x04030204,
0x02020502,
0x02020402,
0x04020202,
0x04020303,
0x04040503,
0x03050504,
0x04020505,
0x04030404,
0x04050502,
0x05020503,
0x02030305,
0x04040503,
0x04040504,
0x04030305,
0x04030202,
0x03050304,
0x03040505,
0x05050404,
0x05050303,
0x03030304,
0x03050405,
0x03020405,
0x02020503,
0x02050404,
0x02020202,
0x03040202,
0x05020405,
0x05050205,
0x05030404,
0x05020502,
0x02050505,
0x03020403,
0x03050402,
0x02040305,
0x05020205,
0x04040402,
0x03020505,
0x04050502,
0x05030203,
0x04050202,
0x05040302,
0x03040405,
0x04050502,
0x04050502,
0x02020504,
0x03030204,
0x02030202,
0x05050203,
0x05030405,
0x05050405,
0x05050502,
0x03030204,
0x04050204,
0x02030402,
0x04040302,
0x04040305,
0x05030405,
0x02040404,
0x02040303,
0x02030204,
0x03020304,
0x05020302,
0x04040405,
0x04050203,
0x03040302,
0x02030302,
0x04030405,
0x05040202,
0x05050404,
0x05030204,
0x02050303,
0x03050204,
0x05050502,
0x03020503,
0x02030403,
0x04020205,
0x05020403,
0x04030405,
0x04020505,
0x02050204,
0x03040403,
0x05040402,
0x03050203,
0x02040202,
0x05030405,
0x02050504,
0x02030305,
0x04050303,
0x05020505,
0x05040402,
0x05030404,
0x02020302,
0x03030304,
0x05020304,
0x04020203,
0x03050503,
0x02050302,
0x05030503,
0x04040203,
0x03030204,
0x03040502,
0x02050305,
0x03040202,
0x04050502,
0x03040303,
0x03050403,
0x04030404,
0x05020203,
0x02040305,
0x02020303,
0x05030502,
0x02020205,
0x03020305,
0x03040303,
0x03050504,
0x03040204,
0x02040203,
0x04050504,
0x05050203,
0x05050504,
0x03020203,
0x04030404,
0x03030204,
0x05040503,
0x04020405,
0x05020302,
0x02040203,
0x04030405,
0x04020404,
0x04050305,
0x02020503,
0x02050203,
0x03040204,
0x02020404,
0x05050403,
0x03020304,
0x03050404,
0x05020503,
0x02050405,
0x04050203,
0x04040205,
0x05050505,
0x02030205,
0x03040504,
0x05040302,
0x04030505,
0x05030404,
0x05020304,
0x04030505,
0x05020502,
0x03030402,
0x04050502,
0x03020302,
0x04040305,
0x02030404,
0x03040404,
0x03020505,
0x02050205,
0x04030402,
0x05030304,
0x04030305,
0x03050402,
0x03040303,
0x02020503,
0x05050305,
0x04030405,
0x03030502,
0x05050203,
0x05030405,
0x05050405,
0x04050502,
0x03030204,
0x04050405,
0x03030202,
0x03050302,
0x04040305,
0x05030303,
0x02030302,
0x05040303,
0x03030202,
0x03040402,
0x05050302,
0x04020505,
0x02050203,
0x05040303,
0x03030304,
0x04030405,
0x05040403,
0x04050204,
0x02020204,
0x04030402,
0x04030502,
0x04020204,
0x04040204,
0x03020505,
0x03050503,
0x03050404,
0x03040302,
0x03050202,
0x05020503,
0x04030304,
0x02030305,
0x04020504,
0x04030305,
0x05040302,
0x05020203,
0x05040302,
0x03030402,
0x05050202,
0x02030305,
0x02040505,
0x05020405,
0x04040304,
0x04050202,
0x02050205,
0x03040504,
0x03030505,
0x02030403,
0x04020204,
0x02050305,
0x02040303,
0x02020504,
0x02020404,
0x04020305,
0x04050404,
0x03040504,
0x05040505,
0x03050402,
0x03020303,
0x02040304,
0x03020504,
0x03050303,
0x05040503,
0x05020405,
0x03040305,
0x05040304,
0x03050302,
0x05030205,
0x03020303,
0x02040305,
0x03040302,
0x03020504,
0x05020303,
0x02050504,
0x03030304,
0x02020305,
0x03050302,
0x04020504,
0x04020302,
0x03050502,
0x02020503,
0x05050302,
0x05030202,
0x05050205,
0x02020405,
0x03040403,
0x05050304,
0x02020202,
0x05020405,
0x02020304,
0x04030305,
0x02050302,
0x05040305,
0x03050505,
0x05020205,
0x04050202,
0x03050205,
0x03040204,
0x03030205,
0x03050404,
0x03040202,
0x04050203,
0x03020305,
0x04030405,
0x02050504,
0x03050205,
0x04050502,
0x02040205,
0x02040502,
0x03020505,
0x03030405,
0x04030304,
0x03020205,
0x03020205,
0x05050203,
0x02040502,
0x05040505,
0x05050202,
0x05050505,
0x05030405,
0x04050503,
0x03030204,
0x04040405,
0x03030202,
0x03050302,
0x04020502,
0x03040302,
0x05030302,
0x05020502,
0x03040503,
0x02050402,
0x02050302,
0x04020504,
0x02030303,
0x05020303,
0x03030305,
0x05020403,
0x05050403,
0x04040304,
0x05050404,
0x05040205,
0x02040503,
0x03030503,
0x02030405,
0x04050204,
0x02040204,
0x04030304,
0x04030404,
0x04040405,
0x02050204,
0x03040403,
0x05040402,
0x03040203,
0x03040402,
0x02030405,
0x02050503,
0x02050405,
0x03040305,
0x02020305,
0x05030402,
0x04030202,
0x04050302,
0x02040402,
0x03030402,
0x04040402,
0x02030503,
0x04030405,
0x03050305,
0x03030204,
0x04050204,
0x03030305,
0x04020302,
0x03020404,
0x04050403,
0x05050202,
0x04040304,
0x02050505,
0x05040304,
0x05020505,
0x04020504,
0x02020303,
0x03030503,
0x04050202,
0x04020204,
0x03050202,
0x04040505,
0x04040502,
0x02040403,
0x04040304,
0x05040204,
0x04020204,
0x02020303,
0x03020204,
0x02030302,
0x04050403,
0x03040502,
0x03050304,
0x05030405,
0x02020303,
0x03050302,
0x02040402,
0x05050202,
0x02020402,
0x03020505,
0x04030503,
0x04020404,
0x03030402,
0x05050503,
0x05040205,
0x04030502,
0x04040204,
0x02030405,
0x03050503,
0x03050305,
0x02050402,
0x05050405,
0x02030305,
0x02040305,
0x02050202,
0x03050203,
0x05020402,
0x03020203,
0x04020405,
0x05030402,
0x04040502,
0x03030303,
0x04050202,
0x03020202,
0x02040305,
0x02050405,
0x03040402,
0x04020305,
0x04030404,
0x04050204,
0x02020402,
0x03040302,
0x04040205,
0x05050202,
0x04050505,
0x05030405,
0x04050503,
0x03030304,
0x04040403,
0x03030205,
0x05030202,
0x03030402,
0x02040302,
0x05030205,
0x05030404,
0x05040503,
0x04050505,
0x02030405,
0x05020304,
0x03030302,
0x05030303,
0x03050305,
0x05020503,
0x05050405,
0x04040303,
0x03030504,
0x02050503,
0x05030204,
0x05050404,
0x03020303,
0x02030503,
0x04030303,
0x03040403,
0x03040303,
0x03030302,
0x05020503,
0x04030304,
0x02030205,
0x04030502,
0x04030305,
0x05020302,
0x02020204,
0x05020302,
0x04030503,
0x05050504,
0x04040305,
0x03040405,
0x03020202,
0x02020304,
0x03040405,
0x05030402,
0x04040204,
0x03030503,
0x05020503,
0x05050305,
0x04050303,
0x02030504,
0x03030203,
0x03040505,
0x02040403,
0x03040203,
0x04020302,
0x04040302,
0x05040505,
0x04040203,
0x05030503,
0x02020504,
0x03030204,
0x03050205,
0x02040202,
0x05020403,
0x02040303,
0x05040304,
0x05040404,
0x04030403,
0x05050402,
0x02030202,
0x02030403,
0x04030502,
0x05050203,
0x05050205,
0x05050305,
0x03050303,
0x02030505,
0x04020505,
0x03040502,
0x04020403,
0x04040203,
0x04050203,
0x03030302,
0x02050404,
0x02040505,
0x05040202,
0x03020402,
0x05040503,
0x02020502,
0x02040503,
0x04050405,
0x05050502,
0x04020202,
0x03050303,
0x04020403,
0x02040305,
0x04040405,
0x04020503,
0x05030404,
0x03030204,
0x05050505,
0x04050405,
0x03030302,
0x05020203,
0x04020303,
0x04020202,
0x05030502,
0x03040305,
0x04050202,
0x04040302,
0x03040403,
0x04050502,
0x04050503,
0x02040504,
0x05030205,
0x02030202,
0x05040202,
0x04050305,
0x05030305,
0x02040503,
0x03030304,
0x04040403,
0x03030205,
0x04020202,
0x03030404,
0x02050405,
0x03040405,
0x02030404,
0x04040304,
0x04030305,
0x03020405,
0x05030304,
0x03030502,
0x04030403,
0x05040305,
0x05040503,
0x03050505,
0x04040303,
0x02040503,
0x04020404,
0x04020503,
0x02020205,
0x04050204,
0x05020304,
0x03050503,
0x02050304,
0x04030404,
0x04040405,
0x02050504,
0x05050403,
0x05040502,
0x03040205,
0x03040402,
0x02040405,
0x05050503,
0x02040402,
0x03020404,
0x02020203,
0x02030205,
0x04050302,
0x05050505,
0x02050503,
0x04040203,
0x05030205,
0x03030302,
0x04020304,
0x04020304,
0x05030302,
0x02020203,
0x02040502,
0x05020404,
0x03020504,
0x05020502,
0x04020202,
0x04040503,
0x02030405,
0x04030303,
0x02050405,
0x03030503,
0x04050304,
0x04050202,
0x03030305,
0x05050205,
0x02040203,
0x04030302,
0x02050502,
0x04030302,
0x04040205,
0x02050304,
0x04030302,
0x05020405,
0x03020204,
0x02040204,
0x02030202,
0x02020202,
0x04050302,
0x02030505,
0x05050402,
0x04050305,
0x05030504,
0x05030304,
0x03030203,
0x02030203,
0x03050305,
0x03050504,
0x04020202,
0x03020305,
0x03040504,
0x02020403,
0x03020502,
0x03020304,
0x04050503,
0x05050205,
0x03040204,
0x02030303,
0x02020303,
0x03050503,
0x03020405,
0x02030204,
0x04020302,
0x02040204,
0x03050402,
0x02040302,
0x05020205,
0x04040402,
0x03020305,
0x04050502,
0x05030203,
0x05020202,
0x04030305,
0x04050402,
0x03040303,
0x03020303,
0x05050305,
0x04030505,
0x03030504,
0x05040204,
0x04050305,
0x05030305,
0x02040503,
0x03030305,
0x05020403,
0x05050205,
0x04020202,
0x05050305,
0x04030405,
0x02040405,
0x02050303,
0x04030204,
0x03020305,
0x05020404,
0x03030302,
0x03020502,
0x04030405,
0x05040405,
0x03040502,
0x02030505,
0x02020303,
0x05050503,
0x03040305,
0x02040205,
0x03050503,
0x02030503,
0x04050202,
0x04020204,
0x04020305,
0x03040303,
0x03030302,
0x05020203,
0x02020304,
0x02030205,
0x05030502,
0x02050305,
0x05030302,
0x02040504,
0x03030305,
0x04050303,
0x05030505,
0x05040402,
0x05030405,
0x02020202,
0x03030502,
0x05050303,
0x04040204,
0x04050403,
0x03020505,
0x04030505,
0x04040505,
0x04020405,
0x04050505,
0x03020505,
0x05040504,
0x03030305,
0x04020402,
0x05030305,
0x05020404,
0x03030504,
0x02030405,
0x03030405,
0x03020403,
0x03050503,
0x02050205,
0x03030505,
0x02040204,
0x03040305,
0x02030405,
0x02050305,
0x02020203,
0x04030203,
0x04050405,
0x03020203,
0x03040204,
0x03040302,
0x03030405,
0x02040304,
0x05030205,
0x02040305,
0x02030502,
0x04050504,
0x04040305,
0x05020405,
0x04050503,
0x03030405,
0x04020405,
0x05020403,
0x05050303,
0x02020405,
0x02050504,
0x04050503,
0x03050504,
0x03040205,
0x03040204,
0x03040205,
0x04050202,
0x04020503,
0x05040202,
0x04040404,
0x04020404,
0x02030302,
0x04030503,
0x02040204,
0x03050302,
0x04040305,
0x02030404,
0x03040404,
0x03020505,
0x04050205,
0x04030402,
0x02020304,
0x03030405,
0x04030304,
0x03020205,
0x03020204,
0x05030203,
0x02040502,
0x05040305,
0x05040304,
0x02050305,
0x03030304,
0x02040505,
0x03030505,
0x05020403,
0x05050205,
0x02050202,
0x04040305,
0x05030405,
0x02050302,
0x04040303,
0x03030204,
0x03020402,
0x05040302,
0x03030202,
0x05020504,
0x02030404,
0x04040403,
0x03040302,
0x02030304,
0x03020503,
0x03020503,
0x02030502,
0x05030204,
0x04040204,
0x05040303,
0x03020503,
0x05030503,
0x03040403,
0x04030504,
0x02040405,
0x02050504,
0x05050403,
0x05040502,
0x02040205,
0x05020402,
0x04040405,
0x05020203,
0x05040402,
0x03030402,
0x04050202,
0x02030305,
0x02040505,
0x05030405,
0x03040302,
0x04030304,
0x03040503,
0x05050203,
0x03040504,
0x02040304,
0x03050204,
0x03020202,
0x05020502,
0x02030204,
0x04020504,
0x02050404,
0x05020305,
0x03050204,
0x05030404,
0x03050205,
0x04030402,
0x05030403,
0x03040302,
0x05020402,
0x02030204,
0x03050504,
0x04030504,
0x04020303,
0x02020202,
0x04050505,
0x02050204,
0x05040202,
0x04030204,
0x02030304,
0x04050304,
0x04040305,
0x03030304,
0x03040405,
0x05020304,
0x02020405,
0x05020202,
0x02050405,
0x03020404,
0x02020204,
0x04020304,
0x05040404,
0x04040302,
0x02030304,
0x05030503,
0x03020405,
0x05050303,
0x05040304,
0x03030205,
0x03050303,
0x02030203,
0x03040202,
0x04030205,
0x04030205,
0x02030505,
0x04020404,
0x05050504,
0x05040405,
0x04030203,
0x03020202,
0x03020305,
0x04030405,
0x02050504,
0x03050205,
0x04050502,
0x02040205,
0x04050502,
0x02050505,
0x03040402,
0x03020505,
0x04030404,
0x04050404,
0x02020502,
0x03040202,
0x04040203,
0x05020304,
0x02050305,
0x03030304,
0x02040505,
0x02020503,
0x05020403,
0x05050305,
0x03050402,
0x04040503,
0x05040302,
0x04030302,
0x05040303,
0x03050403,
0x03050402,
0x02050302,
0x03050203,
0x05040404,
0x02050404,
0x04040402,
0x02050304,
0x02020304,
0x03020403,
0x04040303,
0x05020204,
0x04040503,
0x02020303,
0x04050204,
0x05040304,
0x03040504,
0x04030304,
0x03020203,
0x05030302,
0x05020203,
0x02020304,
0x03020203,
0x05030502,
0x02050305,
0x02030302,
0x02050502,
0x02050405,
0x02040305,
0x02020505,
0x05020402,
0x05030202,
0x04050302,
0x02020304,
0x04020404,
0x05040204,
0x03040204,
0x04020202,
0x03040405,
0x04020402,
0x03030302,
0x02050403,
0x02030304,
0x05020402,
0x05020305,
0x03050205,
0x03040304,
0x04050202,
0x04050304,
0x05030303,
0x05020404,
0x04020303,
0x05020403,
0x02040404,
0x04020502,
0x04030504,
0x04050503,
0x05050205,
0x04050403,
0x04050305,
0x04040305,
0x04050505,
0x05030503,
0x03030503,
0x05050402,
0x05020304,
0x03030304,
0x03020304,
0x03040502,
0x03030305,
0x05040302,
0x04050404,
0x04020205,
0x03020404,
0x05040403,
0x05050304,
0x04050405,
0x03030302,
0x02030402,
0x02040405,
0x05050204,
0x05030402,
0x05030402,
0x05050404,
0x02040502,
0x02030203,
0x05030504,
0x05020405,
0x04040502,
0x04050503,
0x04040403,
0x02020202,
0x05040204,
0x04020405,
0x05030402,
0x04040502,
0x03030305,
0x04050202,
0x03020202,
0x02040504,
0x05050405,
0x04040304,
0x03040403,
0x04050503,
0x04030503,
0x02040505,
0x05030205,
0x02030402,
0x04020504,
0x02050304,
0x03020304,
0x02020505,
0x02020503,
0x05020403,
0x05040305,
0x03030402,
0x02030402,
0x02040302,
0x05030302,
0x05030404,
0x05040503,
0x04050402,
0x02050205,
0x02050403,
0x04040405,
0x02040404,
0x04020402,
0x02050204,
0x02020302,
0x03030405,
0x05030203,
0x03050305,
0x03050204,
0x05050503,
0x03020503,
0x02050402,
0x04020204,
0x05020203,
0x04050504,
0x02040405,
0x02040504,
0x05030403,
0x04050504,
0x02040205,
0x05020402,
0x05020405,
0x03020204,
0x05020302,
0x04030402,
0x05050504,
0x04040305,
0x03040505,
0x03020402,
0x02040502,
0x02050202,
0x02020505,
0x02050403,
0x03050305,
0x03020303,
0x03050303,
0x04050205,
0x05030403,
0x02020404,
0x03050404,
0x05040204,
0x02030503,
0x02030202,
0x03020403,
0x02050502,
0x04020502,
0x04020202,
0x04040504,
0x05040304,
0x05020302,
0x05040505,
0x05050405,
0x05030205,
0x04020303,
0x03050404,
0x05020305,
0x02020304,
0x05050204,
0x03020202,
0x04030302,
0x04030503,
0x02030502,
0x05030505,
0x04020203,
0x02050203,
0x02030504,
0x02020502,
0x04050202,
0x02050202,
0x04020204,
0x04030502,
0x05030402,
0x02050403,
0x04050303,
0x04030204,
0x03040203,
0x03050404,
0x02020305,
0x02020504,
0x02050505,
0x05030502,
0x03050303,
0x05030202,
0x04020204,
0x02050402,
0x04020405,
0x05050202,
0x03020403,
0x02020404,
0x04050405,
0x03030302,
0x05040303,
0x04020303,
0x04020202,
0x05030502,
0x03040305,
0x05020202,
0x04030305,
0x04050202,
0x03040203,
0x03020305,
0x05050205,
0x04030505,
0x03040504,
0x04020504,
0x02050304,
0x02020304,
0x02020505,
0x02040503,
0x03020503,
0x05040305,
0x04020405,
0x03030402,
0x02040204,
0x05040405,
0x03030404,
0x04040505,
0x04030305,
0x02030405,
0x02040403,
0x04040205,
0x03040204,
0x02030402,
0x02030204,
0x04020202,
0x03030404,
0x02020204,
0x04030402,
0x05030302,
0x04020204,
0x05040202,
0x03020505,
0x03050503,
0x03040402,
0x03020203,
0x05030302,
0x04030203,
0x02040304,
0x03020203,
0x03030502,
0x02050305,
0x02040305,
0x05050503,
0x02050403,
0x03020404,
0x02020203,
0x02030404,
0x04050302,
0x04050505,
0x02040402,
0x02020405,
0x05050204,
0x05050405,
0x03050204,
0x05020403,
0x05040504,
0x02040503,
0x05020405,
0x03050404,
0x05050502,
0x02040404,
0x05020204,
0x05020203,
0x03050404,
0x05050303,
0x04040405,
0x04040305,
0x02050502,
0x02040502,
0x02050502,
0x03040504,
0x05050503,
0x02030202,
0x04040202,
0x02020503,
0x03020503,
0x03040302,
0x05030504,
0x03040503,
0x05020504,
0x02020505,
0x05040404,
0x03020405,
0x02030304,
0x02020203,
0x05020202,
0x02020204,
0x05050404,
0x05050403,
0x04020402,
0x03050505,
0x03030404,
0x03020205,
0x03020205,
0x03030305,
0x05030303,
0x05020205,
0x03040303,
0x05050404,
0x05040304,
0x03020305,
0x05030202,
0x03020204,
0x03020204,
0x05040305,
0x03020202,
0x04030305,
0x02030403,
0x03030302,
0x03050402,
0x02040302,
0x05020205,
0x04020502,
0x03020205,
0x04050502,
0x05030203,
0x02020202,
0x03030403,
0x04030305,
0x03020204,
0x03040204,
0x05030202,
0x02040302,
0x05040305,
0x02020504,
0x02040304,
0x02020302,
0x02020305,
0x04040503,
0x03020503,
0x05040305,
0x04020405,
0x03050205,
0x04030405,
0x02040405,
0x02030204,
0x04030204,
0x05020305,
0x05020405,
0x04040405,
0x04050205,
0x03040302,
0x02030202,
0x04030205,
0x05040202,
0x05050404,
0x04050204,
0x05040205,
0x02040503,
0x03030503,
0x02050404,
0x04050204,
0x02040404,
0x04030304,
0x02050504,
0x02050405,
0x03040504,
0x05030403,
0x04050504,
0x04040205,
0x05020402,
0x05030402,
0x02040404,
0x03030305,
0x04050303,
0x05050504,
0x05040402,
0x03030405,
0x02020202,
0x05050504,
0x02030405,
0x03050302,
0x02020202,
0x04030205,
0x04040404,
0x05040202,
0x04050203,
0x04040502,
0x02030403,
0x05030504,
0x02050504,
0x04040203,
0x05040204,
0x04030304,
0x02040502,
0x04040304,
0x05030302,
0x02050203,
0x02050403,
0x05030504,
0x03040503,
0x02040304,
0x02020302,
0x05030304,
0x04050303,
0x03020304,
0x05030503,
0x02030504,
0x02030205,
0x02040303,
0x03040502,
0x04020304,
0x05030405,
0x05030504,
0x03040205,
0x04040203,
0x04040304,
0x02040203,
0x02030305,
0x02020504,
0x03030303,
0x02020304,
0x05030204,
0x04050404,
0x03050404,
0x02050502,
0x05040503,
0x03030503,
0x05050202,
0x04030504,
0x02050404,
0x04030303,
0x03030504,
0x03030405,
0x05020402,
0x05030202,
0x05030304,
0x03030202,
0x02020302,
0x03050302,
0x04040305,
0x02030404,
0x03040404,
0x03020505,
0x04050505,
0x04030402,
0x02050304,
0x03040202,
0x03020505,
0x04030504,
0x04050502,
0x02020502,
0x03040202,
0x04030203,
0x02020504,
0x02040302,
0x02020302,
0x03020205,
0x04040503,
0x03020502,
0x04040505,
0x02040405,
0x04040305,
0x05030405,
0x02040402,
0x02040303,
0x02030204,
0x03020302,
0x05020302,
0x04040505,
0x02050203,
0x05040303,
0x03030304,
0x04030405,
0x05040203,
0x04050204,
0x03030204,
0x02050303,
0x03030204,
0x05050504,
0x03020503,
0x02030403,
0x04030203,
0x03020403,
0x05020203,
0x05020304,
0x04030203,
0x03040304,
0x03020203,
0x03030503,
0x02050305,
0x04040305,
0x05020203,
0x05040402,
0x03020402,
0x02050202,
0x02030305,
0x02040305,
0x05050405,
0x03020504,
0x03030204,
0x05030404,
0x02050305,
0x03020504,
0x03040405,
0x03040405,
0x04030404,
0x03040505,
0x05020503,
0x04030202,
0x03020403,
0x03030203,
0x02040204,
0x04020403,
0x04020303,
0x04040404,
0x05050502,
0x03050504,
0x02020204,
0x05020404,
0x02050505,
0x04030402,
0x03030503,
0x03020404,
0x03020305,
0x05020405,
0x05020404,
0x05020402,
0x02050305,
0x03030203,
0x04030203,
0x03050204,
0x04040205,
0x04040205,
0x04020505,
0x02020502,
0x03050302,
0x05050204,
0x03050403,
0x02020504,
0x05050504,
0x04020205,
0x04030503,
0x04040502,
0x05040305,
0x04050405,
0x02050404,
0x04020504,
0x05050503,
0x04020202,
0x03030302,
0x03040404,
0x05030505,
0x03040402,
0x03040303,
0x03030302,
0x02050305,
0x02020204,
0x02020302,
0x03020305,
0x04040405,
0x02050504,
0x03050205,
0x04050504,
0x02040205,
0x04050502,
0x05050505,
0x04020304,
0x03040402,
0x04040503,
0x04030503,
0x04050505,
0x05030405,
0x02030402,
0x02020505,
0x02040202,
0x02040302,
0x03020204,
0x04040503,
0x03030302,
0x04040505,
0x03050405,
0x04040305,
0x05020303,
0x04030302,
0x05040303,
0x03030402,
0x03040402,
0x04050302,
0x04020504,
0x02050303,
0x05020303,
0x03030305,
0x05020405,
0x05050403,
0x04050304,
0x02040404,
0x04020402,
0x04020502,
0x02020204,
0x04050204,
0x03020305,
0x03050503,
0x02050504,
0x02050504,
0x02050403,
0x03020504,
0x04030402,
0x04050504,
0x04040404,
0x05020402,
0x02030402,
0x02050504,
0x02030405,
0x02040305,
0x02020505,
0x05040402,
0x05030202,
0x02050302,
0x05040404,
0x05050303,
0x05050405,
0x05050402,
0x03020402,
0x05020205,
0x04050403,
0x03040304,
0x05050405,
0x05020203,
0x05040404,
0x05030405,
0x05030505,
0x04050305,
0x02050305,
0x03030504,
0x05020205,
0x05050202,
0x03040302,
0x02030203,
0x04020304,
0x03050503,
0x04040405,
0x04050503,
0x03040205,
0x02040402,
0x02040305,
0x03040203,
0x03030502,
0x04020203,
0x05030505,
0x03050503,
0x02040405,
0x02040302,
0x05020305,
0x05040402,
0x05040203,
0x03030303,
0x03020403,
0x05050403,
0x02030503,
0x03040505,
0x05050304,
0x03030503,
0x03050502,
0x02030203,
0x05030302,
0x04030502,
0x05050405,
0x05020503,
0x03040302,
0x03020502,
0x03020204,
0x03030502,
0x05050505,
0x04050503,
0x03040204,
0x05040205,
0x05020405,
0x02030405,
0x04020305,
0x05030402,
0x04040502,
0x03030305,
0x04050202,
0x03020202,
0x02040504,
0x05020405,
0x04030505,
0x04050302,
0x03040205,
0x03020205,
0x05050205,
0x04050505,
0x03040504,
0x02020505,
0x03020202,
0x02040302,
0x03020204,
0x04040403,
0x03030304,
0x04040503,
0x03050505,
0x02020402,
0x03040302,
0x05030302,
0x05020502,
0x03040503,
0x04050402,
0x02050303,
0x05020504,
0x03030302,
0x05030303,
0x03050305,
0x05020503,
0x05050405,
0x04040302,
0x05050504,
0x03040305,
0x02040505,
0x03050503,
0x02030405,
0x04050202,
0x04040204,
0x04030305,
0x05020203,
0x04020304,
0x04050203,
0x03040305,
0x03020203,
0x03030303,
0x02050303,
0x05040205,
0x03020204,
0x05020302,
0x04030402,
0x05050402,
0x02040305,
0x03040505,
0x03020404,
0x03020304,
0x03040405,
0x05050504,
0x04030202,
0x05050305,
0x05030203,
0x05040202,
0x03030303,
0x02040503,
0x05050403,
0x02020403,
0x04030203,
0x04030203,
0x02050202,
0x03020405,
0x05020402,
0x05050205,
0x03020305,
0x05020205,
0x02020303,
0x04050304,
0x02050505,
0x03050203,
0x04050503,
0x05040205,
0x03030403,
0x05030503,
0x03030305,
0x04020203,
0x04030205,
0x05030502,
0x05050504,
0x02030304,
0x04020302,
0x03030403,
0x03040403,
0x05030202,
0x03040302,
0x04040404,
0x02050202,
0x04050505,
0x04030202,
0x03050203,
0x03020302,
0x02030305,
0x05030304,
0x03020303,
0x04020503,
0x03040403,
0x03050205,
0x05030502,
0x03020402,
0x04030404,
0x03020505,
0x03020303,
0x04030504,
0x05040404,
0x02030505,
0x04030403,
0x02020304,
0x04050405,
0x03030502,
0x05040303,
0x04020303,
0x04020202,
0x05030502,
0x03040305,
0x02020403,
0x03050402,
0x04030305,
0x03030204,
0x03040204,
0x03020202,
0x02040302,
0x04040305,
0x02030505,
0x03020202,
0x02040502,
0x05030204,
0x04040403,
0x03030304,
0x04040502,
0x05020505,
0x03030402,
0x02040302,
0x05030405,
0x03030404,
0x04040503,
0x04050505,
0x02030405,
0x05030304,
0x03030302,
0x04030503,
0x05040305,
0x05040503,
0x03050505,
0x04040303,
0x03020502,
0x02050503,
0x05030204,
0x04050204,
0x05020303,
0x03030503,
0x05030303,
0x03040403,
0x02040504,
0x03050403,
0x03020504,
0x04030402,
0x04050504,
0x04040404,
0x05020405,
0x02040502,
0x05050503,
0x02050405,
0x03040404,
0x02020203,
0x02030402,
0x04030302,
0x04050505,
0x04050204,
0x04030502,
0x05040205,
0x05020403,
0x04020403,
0x03030205,
0x05040202,
0x05030404,
0x02040304,
0x03030204,
0x04050502,
0x04030502,
0x02040504,
0x05050205,
0x04030205,
0x05040203,
0x03020302,
0x02040403,
0x04050303,
0x03020405,
0x04050204,
0x02040502,
0x04020304,
0x05040304,
0x05020305,
0x04050204,
0x02030505,
0x04040205,
0x04020303,
0x04050205,
0x04040505,
0x02020205,
0x05030203,
0x04020402,
0x02030304,
0x02040203,
0x02030504,
0x05020504,
0x03030403,
0x02020402,
0x04030504,
0x05030202,
0x05020302,
0x02030502,
0x02030205,
0x05020202,
0x03050302,
0x05050505,
0x04020405,
0x02030303,
0x04040204,
0x04040403,
0x05040502,
0x05030404,
0x02040505,
0x02020204,
0x04040403,
0x05050505,
0x05040502,
0x02020205,
0x03050402,
0x02040302,
0x05020205,
0x04020502,
0x03020205,
0x05050502,
0x05030205,
0x02050202,
0x05040202,
0x03020405,
0x04030502,
0x04050502,
0x02020502,
0x03030204,
0x04030203,
0x02030503,
0x03020402,
0x02040402,
0x05030204,
0x04040402,
0x03050304,
0x02020502,
0x04020505,
0x03030404,
0x02030405,
0x03040405,
0x02030404,
0x04020304,
0x04030305,
0x05020405,
0x03030302,
0x03020502,
0x04030405,
0x05040505,
0x03040502,
0x03030505,
0x02020303,
0x04040503,
0x05020404,
0x04040503,
0x02020205,
0x04050204,
0x05040304,
0x03050502,
0x04050304,
0x05030205,
0x04020504,
0x04050203,
0x03040305,
0x03020202,
0x03050303,
0x02050302,
0x05030205,
0x02020204,
0x05030305,
0x04050303,
0x05050504,
0x05040502,
0x03020405,
0x02020202,
0x05050305,
0x02020203,
0x04050402,
0x03020504,
0x04030502,
0x05020405,
0x03030404,
0x04040503,
0x03050305,
0x02030305,
0x04050202,
0x05040203,
0x02050204,
0x05050504,
0x05030502,
0x02050303,
0x04050504,
0x03020304,
0x04040205,
0x04030202,
0x03030205,
0x02020403,
0x04040205,
0x02050505,
0x05030405,
0x02030305,
0x03050403,
0x03020202,
0x03020504,
0x02040302,
0x04030504,
0x05030205,
0x03030303,
0x03030303,
0x05050403,
0x05040302,
0x05050205,
0x03040403,
0x02050505,
0x04040402,
0x03020402,
0x02020303,
0x02040305,
0x03050303,
0x05020503,
0x03050504,
0x02030404,
0x03040302,
0x05020205,
0x05030304,
0x05030303,
0x02040403,
0x02040302,
0x03020505,
0x04030503,
0x04020205,
0x05050505,
0x04040304,
0x04050504,
0x02050505,
0x03050302,
0x04040305,
0x02030205,
0x03040404,
0x03020505,
0x04050505,
0x05030402,
0x05030304,
0x04020305,
0x03040402,
0x04040303,
0x02020503,
0x04050505,
0x05030405,
0x03030402,
0x02030403,
0x05020402,
0x04040402,
0x05030202,
0x04040202,
0x02050304,
0x02020502,
0x04020505,
0x05040305,
0x05030405,
0x02040405,
0x02050303,
0x04030204,
0x03020305,
0x05020502,
0x03030202,
0x05020504,
0x02030405,
0x04040403,
0x03040302,
0x02030504,
0x02020503,
0x05050503,
0x03050305,
0x03050205,
0x05050503,
0x03030503,
0x04050402,
0x04020204,
0x04020203,
0x02040502,
0x03050203,
0x05020504,
0x04030404,
0x04050305,
0x04020404,
0x05020405,
0x02040502,
0x05020203,
0x05040402,
0x03020404,
0x02040202,
0x02030305,
0x02040302,
0x05050405,
0x05020304,
0x05020405,
0x04030404,
0x04020304,
0x03040204,
0x03020305,
0x05030305,
0x04050204,
0x03050503,
0x04040405,
0x03040502,
0x04040503,
0x03040403,
0x02020502,
0x03040204,
0x03040304,
0x03040204,
0x04040404,
0x05040405,
0x03030204,
0x02020205,
0x03030204,
0x05020404,
0x04050505,
0x02020505,
0x02030504,
0x02050203,
0x02040303,
0x02050503,
0x05030204,
0x04020402,
0x02050405,
0x04020203,
0x02020304,
0x02050205,
0x04040402,
0x03050203,
0x02040503,
0x03050304,
0x02030404,
0x04020504,
0x05030205,
0x04050404,
0x02040404,
0x04040303,
0x02040302,
0x04050502,
0x04040502,
0x04030202,
0x03020204,
0x02040304,
0x04030302,
0x03050502,
0x05040204,
0x03030204,
0x04020202,
0x05030403,
0x05040403,
0x05040505,
0x04040404,
0x03020305,
0x04040405,
0x02050504,
0x03050404,
0x04050505,
0x02040205,
0x04050502,
0x05020505,
0x02030505,
0x04050302,
0x03020205,
0x03020205,
0x05050205,
0x04040503,
0x05040504,
0x02050403,
0x05020402,
0x04040403,
0x05030202,
0x05050202,
0x02050304,
0x02020402,
0x03050305,
0x04040305,
0x05030403,
0x02050302,
0x04040303,
0x03030202,
0x03020402,
0x05040302,
0x03050203,
0x05020404,
0x02050404,
0x04040403,
0x02050302,
0x02020304,
0x03020403,
0x02020303,
0x02030402,
0x05030202,
0x04040204,
0x05040302,
0x03020505,
0x03030503,
0x03040402,
0x05030205,
0x04020504,
0x02050202,
0x03040303,
0x03020402,
0x03050303,
0x02050202,
0x02030405,
0x02050504,
0x02030305,
0x04040305,
0x03020505,
0x05040402,
0x05030202,
0x02040302,
0x03050203,
0x04020304,
0x04030302,
0x05040505,
0x04020303,
0x05030205,
0x02030303,
0x04030402,
0x03040502,
0x05020205,
0x03040503,
0x04040505,
0x04030405,
0x03050203,
0x04030503,
0x02020302,
0x05030404,
0x05040204,
0x02050205,
0x05030502,
0x02050404,
0x02040405,
0x03030202,
0x03050405,
0x02030503,
0x02050402,
0x05020505,
0x03020404,
0x03050205,
0x04040404,
0x05040405,
0x02020305,
0x02030504,
0x03040502,
0x03050302,
0x04020205,
0x02030205,
0x04050502,
0x02030405,
0x03020303,
0x04040502,
0x04030405,
0x03030502,
0x05050403,
0x03050505,
0x03020204,
0x04040502,
0x04030502,
0x03050403,
0x05050403,
0x05030403,
0x04040304,
0x05040502,
0x02020503,
0x03020405,
0x02040403,
0x02050505,
0x04030503,
0x05020205,
0x04020205,
0x04020305,
0x05030402,
0x04040502,
0x03030305,
0x04050202,
0x03020205,
0x02040504,
0x02040405,
0x03040402,
0x04020305,
0x03030404,
0x05050204,
0x03020202,
0x03040302,
0x04040305,
0x03050403,
0x05020403,
0x04050403,
0x05050202,
0x05050204,
0x02050304,
0x02030402,
0x03050305,
0x04040503,
0x03040302,
0x05030302,
0x05040503,
0x03050503,
0x03050402,
0x02050302,
0x02050203,
0x04040405,
0x02040404,
0x04020402,
0x02050204,
0x02020304,
0x03030405,
0x04050203,
0x05020204,
0x02040503,
0x03020303,
0x04050404,
0x04040204,
0x03040404,
0x04030304,
0x03040502,
0x03050202,
0x05020505,
0x04030404,
0x04050305,
0x04020404,
0x05030505,
0x05040302,
0x05020205,
0x05020302,
0x04030402,
0x05050402,
0x02050305,
0x02040505,
0x03020405,
0x04020505,
0x04050402,
0x03050505,
0x02050404,
0x02050304,
0x04040302,
0x05020205,
0x04050203,
0x03030203,
0x03040304,
0x02020202,
0x02050205,
0x03050304,
0x03050305,
0x03030305,
0x03020503,
0x04040504,
0x05040202,
0x02030404,
0x02040402,
0x04020304,
0x04040305,
0x02030505,
0x03020502,
0x02040302,
0x02050404,
0x02030405,
0x02050304,
0x03020503,
0x04040503,
0x04050402,
0x05020302,
0x04040203,
0x05020403,
0x04030303,
0x03050404,
0x04040203,
0x04030405,
0x03050505,
0x05050505,
0x05050304,
0x05020505,
0x03030303,
0x04040204,
0x02040205,
0x04020505,
0x05020202,
0x04020505,
0x02050503,
0x02040404,
0x05050202,
0x05050503,
0x03040504,
0x05040303,
0x05020305,
0x05040303,
0x02050405,
0x02030503,
0x02050502,
0x03040202,
0x04050405,
0x03030402,
0x05040303,
0x04020303,
0x04050202,
0x05030502,
0x03040305,
0x04050403,
0x05040302,
0x03020405,
0x04050502,
0x04050502,
0x02020502,
0x03030204,
0x02030202,
0x03050203,
0x05020403,
0x05050403,
0x05050202,
0x05050204,
0x04050204,
0x02030402,
0x03030303,
0x03030402,
0x02040302,
0x05030303,
0x05030404,
0x05040503,
0x04050403,
0x02050405,
0x02040403,
0x04040405,
0x03040204,
0x02030402,
0x02050204,
0x04020202,
0x03030404,
0x05030205,
0x02050303,
0x03030204,
0x05050502,
0x03020503,
0x02030403,
0x04020205,
0x05020403,
0x04030405,
0x04020505,
0x02050202,
0x03040303,
0x03020402,
0x03050203,
0x02040202,
0x02050405,
0x05050503,
0x02050405,
0x03040304,
0x02020203,
0x03030402,
0x04030202,
0x04050505,
0x05020302,
0x05050404,
0x02030403,
0x03040405,
0x03030202,
0x05050504,
0x03050202,
0x05050505,
0x05050203,
0x03040302,
0x05020205,
0x03030504,
0x03030204,
0x05040503,
0x02020404,
0x04050503,
0x05020503,
0x04040504,
0x04050404,
0x04050203,
0x05050505,
0x03040303,
0x04040402,
0x05030502,
0x02040504,
0x02040503,
0x02040402,
0x03020204,
0x02030304,
0x02020304,
0x05040302,
0x05040202,
0x05050205,
0x04050304,
0x02030202,
0x02040304,
0x05040302,
0x02030505,
0x02020404,
0x03030302,
0x03030405,
0x05020403,
0x03020404,
0x05030203,
0x03030504,
0x02050304,
0x04040403,
0x02030304,
0x03030204,
0x05030304,
0x03050505,
0x04050202,
0x05020202,
0x03050303,
0x05050303,
0x02040205,
0x02050402,
0x04050404,
0x03030205,
0x03030403,
0x04050402,
0x02040302,
0x05020205,
0x04020502,
0x03020205,
0x05050502,
0x05030205,
0x05030202,
0x04030305,
0x03050402,
0x02040303,
0x02020503,
0x05050305,
0x04030405,
0x03030402,
0x03050203,
0x05030403,
0x05050405,
0x05050402,
0x03030204,
0x04050204,
0x02030402,
0x04020302,
0x03030402,
0x02040404,
0x05040405,
0x02030404,
0x04040505,
0x04030305,
0x02020405,
0x02040405,
0x04050205,
0x03040302,
0x02030202,
0x04030205,
0x04040202,
0x05050404,
0x02020204,
0x04030402,
0x04030502,
0x04020204,
0x04040204,
0x03020505,
0x03050503,
0x03050504,
0x03040302,
0x03050202,
0x05020503,
0x04030204,
0x02030305,
0x04020504,
0x05030505,
0x05030302,
0x02020204,
0x05020304,
0x04050303,
0x05050504,
0x05040303,
0x03020405,
0x03020202,
0x05030404,
0x05020202,
0x02050404,
0x02050503,
0x05050202,
0x05040405,
0x03020403,
0x05030205,
0x05030303,
0x05020203,
0x02030403,
0x02040505,
0x04020405,
0x02020505,
0x02040303,
0x05040502,
0x03050304,
0x04020404,
0x02050502,
0x05030404,
0x03040402,
0x02050304,
0x03050504,
0x03040402,
0x03050403,
0x04050205,
0x03040204,
0x05030405,
0x02040304,
0x05040305,
0x04050505,
0x05050403,
0x02020402,
0x02020202,
0x02020204,
0x05020502,
0x02030504,
0x05020503,
0x04050505,
0x02030304,
0x03040402,
0x05050204,
0x03050505,
0x04020203,
0x02050504,
0x03050403,
0x05030303,
0x04040304,
0x02030404,
0x04040403,
0x03020504,
0x02030202,
0x05030303,
0x05050202,
0x02020403,
0x05040505,
0x04020205,
0x04020202,
0x02030204,
0x04020405,
0x03050302,
0x02040305,
0x02030205,
0x03040404,
0x03020505,
0x04050505,
0x05030402,
0x03020302,
0x02030405,
0x04030302,
0x03020205,
0x03020205,
0x05050203,
0x02040503,
0x05040505,
0x05050203,
0x05030405,
0x05050405,
0x04050502,
0x03030204,
0x04050205,
0x02030202,
0x04020302,
0x03050205,
0x04030405,
0x02040405,
0x02050205,
0x04030204,
0x05020305,
0x05020405,
0x04040505,
0x04050203,
0x05040302,
0x03030304,
0x04030405,
0x05040203,
0x05050204,
0x05050204,
0x05040205,
0x02040505,
0x03030503,
0x02030405,
0x04050204,
0x02040204,
0x04030304,
0x04030405,
0x04020505,
0x02050204,
0x03040403,
0x05040402,
0x03050203,
0x02040402,
0x02040405,
0x05030303,
0x04040402,
0x03020404,
0x02040202,
0x02030305,
0x04040302,
0x05050505,
0x02050504,
0x04040402,
0x03050502,
0x04040403,
0x04020302,
0x03030403,
0x05050402,
0x04030503,
0x02020203,
0x03050403,
0x04050504,
0x05020402,
0x03050202,
0x03030405,
0x02020202,
0x05020303,
0x05040305,
0x04050304,
0x04020302,
0x03040304,
0x05050202,
0x03040204,
0x02020504,
0x05020403,
0x03040403,
0x04040504,
0x03040505,
0x03030302,
0x04050205,
0x03050304,
0x02050403,
0x04030504,
0x03020304,
0x05050405,
0x03030503,
0x04050305,
0x05030503,
0x03050302,
0x03030203,
0x05020405,
0x04050305,
0x05050202,
0x04040304,
0x03050405,
0x05040304,
0x03050505,
0x05040402,
0x02030503,
0x03040402,
0x02030403,
0x03030304,
0x05050403,
0x02020202,
0x02050504,
0x03020403,
0x05050505,
0x05020305,
0x04040204,
0x05050205,
0x03050503,
0x03020303,
0x04040405,
0x02050504,
0x03050404,
0x04050505,
0x02040205,
0x04030402,
0x02050505,
0x03040402,
0x04020305,
0x04030404,
0x05050204,
0x02020402,
0x03040302,
0x04040205,
0x05050202,
0x05030505,
0x05030405,
0x04050502,
0x03030204,
0x04040405,
0x03030202,
0x02040302,
0x04040305,
0x05030405,
0x02040502,
0x02040303,
0x02030204,
0x03020202,
0x05040302,
0x04020504,
0x02050303,
0x05020303,
0x03030304,
0x05020405,
0x05050403,
0x04050204,
0x03030404,
0x02050503,
0x05030204,
0x05050404,
0x03020303,
0x02030503,
0x04030303,
0x03040403,
0x03040303,
0x03050302,
0x05020503,
0x04030304,
0x02030305,
0x04030504,
0x04030305,
0x03030302,
0x02050504,
0x02030305,
0x04050305,
0x05020505,
0x05040402,
0x05030402,
0x02040302,
0x04050402,
0x02040405,
0x02040202,
0x05050503,
0x02050302,
0x05050204,
0x03020205,
0x03050303,
0x02030502,
0x02040305,
0x04050202,
0x04050504,
0x04030305,
0x02040205,
0x05030402,
0x04050403,
0x02050505,
0x03030304,
0x05020504,
0x03020302,
0x05040503,
0x05020204,
0x04050403,
0x03040304,
0x03040305,
0x02030503,
0x02030505,
0x04020503,
0x04050303,
0x02020505,
0x05050202,
0x04020305,
0x03030504,
0x05030304,
0x04030305,
0x03040502,
0x03030502,
0x05050302,
0x02030202,
0x02050405,
0x02040505,
0x04020202,
0x02050503,
0x03040302,
0x04050402,
0x04040303,
0x03020503,
0x04020402,
0x05030505,
0x04030302,
0x05050302,
0x05020202,
0x03040204,
0x02030303,
0x04020303,
0x05040203,
0x05050505,
0x05050204,
0x03020403,
0x03050402,
0x04020305,
0x05030402,
0x02050502,
0x03030305,
0x04050202,
0x03020205,
0x02040504,
0x04050505,
0x04040302,
0x03040403,
0x04050502,
0x04050502,
0x02020504,
0x05030205,
0x02030202,
0x05050202,
0x04050505,
0x05030405,
0x04050503,
0x03030304,
0x04040405,
0x03030204,
0x03050302,
0x04040303,
0x05020302,
0x04030302,
0x05040303,
0x03050403,
0x03040402,
0x04050302,
0x05020504,
0x03030302,
0x05020303,
0x03050305,
0x05020503,
0x05050403,
0x04040302,
0x02040404,
0x04020404,
0x04020503,
0x02020205,
0x04050204,
0x05040304,
0x03050503,
0x02050304,
0x04030404,
0x04040405,
0x02050204,
0x03040403,
0x05040502,
0x03040203,
0x03040402,
0x05040405,
0x05020205,
0x05040302,
0x05030402,
0x05050202,
0x02050305,
0x02040505,
0x05020405,
0x04050203,
0x03030505,
0x05030502,
0x02020402,
0x02040303,
0x02050404,
0x03020205,
0x02020402,
0x05050302,
0x02030405,
0x04020202,
0x03020204,
0x04040404,
0x02030204,
0x03050203,
0x02020502,
0x02030204,
0x05020304,
0x03050504,
0x03040504,
0x04030404,
0x02020402,
0x04020502,
0x04030402,
0x03020503,
0x04030304,
0x03030305,
0x03020205,
0x04030403,
0x04050505,
0x05050303,
0x05020302,
0x04030205,
0x03050303,
0x04020305,
0x02020205,
0x05020404,
0x04050203,
0x03020404,
0x04040304,
0x02040504,
0x05040304,
0x02030302,
0x02020205,
0x05020305,
0x04030502,
0x04040404,
0x05030502,
0x04040205,
0x03040503,
0x03030305,
0x03020404,
0x04040202,
0x05030302,
0x05030204,
0x03040303,
0x05040503,
0x04050305,
0x05020503,
0x02050403,
0x04050405,
0x03030402,
0x05040303,
0x04030303,
0x04050202,
0x05020502,
0x03040305,
0x05020403,
0x04030305,
0x05050402,
0x03040303,
0x03020303,
0x05050305,
0x04030405,
0x03030502,
0x05040202,
0x04050505,
0x05030205,
0x02050503,
0x03030304,
0x04040403,
0x03030205,
0x03050202,
0x02020402,
0x02040302,
0x05030302,
0x05020404,
0x03040503,
0x04050402,
0x02050305,
0x05030304,
0x03030302,
0x04030503,
0x05050305,
0x05020503,
0x03050505,
0x04040303,
0x05050502,
0x03040305,
0x03040205,
0x03050503,
0x02030503,
0x04050202,
0x04020204,
0x04020303,
0x03040303,
0x03030302,
0x05020303,
0x02030304,
0x02030205,
0x04030502,
0x04030305,
0x02030302,
0x04050503,
0x02050405,
0x03040305,
0x02020203,
0x05030402,
0x04030202,
0x04050305,
0x04030302,
0x04030205,
0x03030205,
0x02040402,
0x04020304,
0x05040402,
0x05050503,
0x05030302,
0x05030502,
0x02040502,
0x03040303,
0x03020405,
0x03030403,
0x02030305,
0x04020205,
0x03040204,
0x05050305,
0x03040504,
0x03030403,
0x03030504,
0x02020202,
0x02030503,
0x04050204,
0x04050505,
0x04040204,
0x03050303,
0x04020303,
0x03030403,
0x05040405,
0x02040303,
0x03050202,
0x02050404,
0x02020504,
0x05030304,
0x04050205,
0x03020505,
0x02050304,
0x05020505,
0x02030205,
0x05050505,
0x05030403,
0x03050305,
0x05050305,
0x03040304,
0x03040303,
0x04040405,
0x05030203,
0x02030505,
0x03040304,
0x05050303,
0x05030403,
0x03050502,
0x02040202,
0x05050505,
0x03020403,
0x04030204,
0x04020502,
0x05050202,
0x03040303,
0x05050404,
0x04050402,
0x03040302,
0x05020203,
0x04020502,
0x03020205,
0x05030502,
0x05030205,
0x03020202,
0x03030405,
0x04030304,
0x03020205,
0x03020204,
0x05030203,
0x02040502,
0x05040305,
0x05040204,
0x04050305,
0x05030305,
0x02040503,
0x03030305,
0x04040403,
0x05050205,
0x04020202,
0x03030402,
0x02040302,
0x05020405,
0x03030404,
0x04040503,
0x04050305,
0x02030405,
0x05030302,
0x03020502,
0x04030403,
0x05040505,
0x03040502,
0x03050505,
0x02020303,
0x02020503,
0x02030502,
0x05030204,
0x04040204,
0x05040303,
0x03020503,
0x05030503,
0x03040403,
0x04030404,
0x04040405,
0x02050504,
0x05050403,
0x05040502,
0x03040205,
0x05020402,
0x05030405,
0x02020204,
0x05020302,
0x04030303,
0x05050504,
0x05040305,
0x03040405,
0x03020202,
0x04020305,
0x05040504,
0x05050404,
0x02020404,
0x02050303,
0x04040502,
0x05050403,
0x02030302,
0x04050404,
0x02030503,
0x05040503,
0x05030203,
0x05040502,
0x03040302,
0x05040305,
0x03020305,
0x03050202,
0x05040303,
0x04030402,
0x02030202,
0x05020304,
0x04050305,
0x03020204,
0x04040403,
0x02020505,
0x03040505,
0x02050505,
0x04050403,
0x02040304,
0x03020302,
0x05030503,
0x04050305,
0x03020404,
0x05040503,
0x05030203,
0x03030303,
0x05030403,
0x05040303,
0x04040302,
0x02040503,
0x04040405,
0x04030405,
0x05020503,
0x05040502,
0x04040403,
0x05020503,
0x02030202,
0x03020404,
0x05020404,
0x02040205,
0x05050303,
0x03050405,
0x02040203,
0x02030205,
0x04040303,
0x05040404,
0x03040303,
0x04040502,
0x05050204,
0x03040404,
0x03050302,
0x02040305,
0x02030205,
0x04040404,
0x03020505,
0x04050505,
0x05030402,
0x02050302,
0x03040402,
0x03020305,
0x04030404,
0x04050404,
0x02020402,
0x03040202,
0x04040203,
0x05040304,
0x04050305,
0x03030305,
0x02040505,
0x03030505,
0x05020403,
0x05050205,
0x04020202,
0x03030204,
0x02030405,
0x03040405,
0x02030404,
0x04020204,
0x04020305,
0x05020405,
0x03030202,
0x03020504,
0x02030405,
0x05040403,
0x03040302,
0x02030504,
0x02020503,
0x04040503,
0x05020204,
0x04040503,
0x02020303,
0x04050204,
0x05040204,
0x03040504,
0x04030304,
0x03040203,
0x03030302,
0x05020203,
0x02020304,
0x02030205,
0x05030502,
0x02050305,
0x02040302,
0x05050303,
0x02040402,
0x03020404,
0x02020203,
0x02030305,
0x04040302,
0x05050505,
0x05050202,
0x02030505,
0x03030504,
0x05030402,
0x04040305,
0x05030204,
0x04020202,
0x02020402,
0x04020503,
0x02040303,
0x04030304,
0x02020303,
0x02050204,
0x02050403,
0x03030204,
0x04030205,
0x04040404,
0x04050303,
0x04050502,
0x03020204,
0x05030202,
0x04030503,
0x03040503,
0x05050203,
0x05030305,
0x02050504,
0x05030504,
0x02020302,
0x02050403,
0x04040202,
0x05040202,
0x05040404,
0x02050404,
0x03040404,
0x05050505,
0x03030202,
0x02020302,
0x02040305,
0x04020303,
0x02040204,
0x04050305,
0x05040503,
0x02050202,
0x05050404,
0x03020202,
0x03050503,
0x02030404,
0x02030504,
0x04040305,
0x03050405,
0x04050504,
0x03040402,
0x05020505,
0x05030203,
0x04050405,
0x05040304,
0x03030403,
0x04040504,
0x02030505,
0x02040503,
0x05030202,
0x02050304,
0x04050303,
0x02020404,
0x05050504,
0x03040403,
0x05030504,
0x05040505,
0x04020502,
0x03050204,
0x05050302,
0x02030304,
0x04050203,
0x03050205,
0x04040503,
0x03030204,
0x05020404,
0x03030304,
0x04030302,
0x03030305,
0x04020402,
0x05020204,
0x05030503,
0x05020303,
0x04050504,
0x04040304,
0x03040202,
0x05050305,
0x03050302,
0x05050403,
0x02020404,
0x05030403,
0x04030305,
0x04030503,
0x05050305,
0x02030503,
0x04030403,
0x05050204,
0x02050503,
0x05040403,
0x04030203,
0x05020503,
0x02030403,
0x03030205,
0x05040502,
0x05020302,
0x03020503,
0x05050503,
0x04030203,
0x05020305,
0x02020305,
0x03040304,
0x04040204,
0x03020205,
0x05020404,
0x03040305,
0x03030504,
0x03040202,
0x03030302,
0x05050303,
0x02040402,
0x02040303,
0x02050202,
0x05030404,
0x05050303,
0x03040204,
0x04020305,
0x05020505,
0x03020403,
0x02020503,
0x02020203,
0x05020302,
0x04040405,
0x05040205,
0x05020205,
0x05040202,
0x04050402,
0x03040403,
0x05020504,
0x05040505,
0x04020502,
0x05050302,
0x05030404,
0x02020505,
0x04030403,
0x04020205,
0x02030204,
0x04040503,
0x02050303,
0x04040402,
0x02040204,
0x03050503,
0x03050205,
0x05040505,
0x04030302,
0x04020303,
0x04050504,
0x04040304,
0x03040202,
0x05050205,
0x03030204,
0x05050202,
0x02050404,
0x03030403,
0x02030202,
0x05020305,
0x05050405,
0x03020304,
0x03050504,
0x04020302,
0x05040502,
0x02050504,
0x02020305,
0x04050203,
0x04020504,
0x03040202,
0x04020205,
0x02050205,
0x04040502,
0x02020205,
0x03040505,
0x04030305,
0x04030403,
0x05040505,
0x03050302,
0x04050303,
0x04020404,
0x02050504,
0x05020302,
0x03030202,
0x02050203,
0x02040403,
0x02040404,
0x03050404,
0x04020202,
0x02050504,
0x04020204,
0x03030502,
0x05030502,
0x04020205,
0x04030503,
0x02030205,
0x05020203,
0x05020203};

/* Packet header on RD output */
#define WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE  8

#pragma DATA_SECTION (wimax_ul_out_packet_1, ".testData");
static UInt32 wimax_ul_out_packet_1[WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE] = {
0x00000000,
0x00000000,
0x00000000,
0x00000000,
0x000001E0,
0x00000000,
0x00000000,
0x00000000
};

/* Reference Output Data for the test */
#define WIMAX_UL_OUTPUT_PKT_2_WRD_SIZE  396

#pragma DATA_SECTION (wimax_ul_out_packet_2, ".testData");
static UInt32 wimax_ul_out_packet_2[WIMAX_UL_OUTPUT_PKT_2_WRD_SIZE] = {
0xe1e11fe1, 0x1f1f1f1f, 0x1f1fe1e1, 0x1f1f1fe1,
0x1f1fe1e1, 0x1f1f1f1f, 0x1fe11fe1, 0x1fe1e1e1,
0xe11fe11f, 0xe11fe11f, 0xe1e11fe1, 0xe11fe11f,
0x1f1f1fe1, 0xe11f1fe1, 0x1f1f1f1f, 0x1fe1e11f,
0x1f1fe1e1, 0x1fe11fe1, 0xe1e11f1f, 0x1f1f1f1f,
0x1fe11fe1, 0x1f1fe1e1, 0x1fe1e1e1, 0xe11fe1e1,
0x1fe11f1f, 0xe1e1e11f, 0x1f1f1fe1, 0xe1e11fe1,
0x1f1f1fe1, 0x1fe1e1e1, 0xe1e1e1e1, 0xe11f1fe1,
0xe1e11fe1, 0x1fe11fe1, 0x1f1f1f1f, 0x1f1f1f1f,
0x1f1f1fe1, 0xe1e1e1e1, 0xe1e11fe1, 0x1fe1e11f,
0xe11fe1e1, 0x1f1f1fe1, 0x1f1fe11f, 0x1f1f1fe1,
0xe1e1e1e1, 0x1f1fe11f, 0xe11fe1e1, 0xe11f1fe1,
0xe11f1fe1, 0xe11fe11f, 0xe1e1e1e1, 0x1fe11fe1,
0xe11fe11f, 0xe1e11fe1, 0x1fe1e1e1, 0xe11f1f1f,
0xe11fe11f, 0xe1e11fe1, 0xe11f1f1f, 0x1fe11fe1,
0x1f1f1f1f, 0xe11fe1e1, 0x1f1fe11f, 0x1fe1e1e1,
0xe11f1fe1, 0xe11f1fe1, 0xe1e11f1f, 0xe11fe1e1,
0xe11fe11f, 0xe11f1fe1, 0x1f1f1fe1, 0x1f1f1fe1,
0x00e1001f, 0x001f00e1, 0x00e100e1, 0x00e1001f,
0x00e100e1, 0x00e1001f, 0x001f00e1, 0x00e1001f,
0x001f00e1, 0x001f00e1, 0x00e1001f, 0x001f00e1,
0x00e1001f, 0x001f001f, 0x00e100e1, 0x001f00e1,
0x001f001f, 0x00e1001f, 0x001f001f, 0x00e100e1,
0x001f001f, 0x00e1001f, 0x001f001f, 0x001f00e1,
0x00e1001f, 0x00e100e1, 0x001f001f, 0x001f00e1,
0x00e100e1, 0x001f001f, 0x00e100e1, 0x001f001f,
0x001f001f, 0x001f001f, 0x00e100e1, 0x00e1001f,
0x00e100e1, 0x001f00e1, 0x001f00e1, 0x00e100e1,
0x00e1001f, 0x001f00e1, 0x00e1001f, 0x001f00e1,
0x00e1001f, 0x001f00e1, 0x001f001f, 0x001f001f,
0x00e1001f, 0x00e100e1, 0x00e100e1, 0x00e100e1,
0x00e1001f, 0x001f001f, 0x00e1001f, 0x00e100e1,
0x00e100e1, 0x00e1001f, 0x00e100e1, 0x00e1001f,
0x001f001f, 0x001f001f, 0x001f001f, 0x00e1001f,
0x001f001f, 0x001f00e1, 0x001f00e1, 0x00e1001f,
0x001f00e1, 0x00e100e1, 0x001f00e1, 0x001f001f,
0x001f00e1, 0x00e1001f, 0x00e100e1, 0x00e1001f,
0x001f00e1, 0x001f001f, 0x00e1001f, 0x001f001f,
0x00e1001f, 0x001f00e1, 0x00e100e1, 0x001f001f,
0x001f00e1, 0x001f00e1, 0x001f001f, 0x00e1001f,
0x001f00e1, 0x00e100e1, 0x00e100e1, 0x001f00e1,
0x00e1001f, 0x00e100e1, 0x00e1001f, 0x001f001f,
0x001f00e1, 0x001f001f, 0x001f001f, 0x001f001f,
0x001f001f, 0x001f00e1, 0x00e100e1, 0x001f001f,
0x00e100e1, 0x00e1001f, 0x001f001f, 0x00e100e1,
0x001f00e1, 0x00e1001f, 0x00e1001f, 0x00e100e1,
0x00e100e1, 0x001f001f, 0x001f001f, 0x00e1001f,
0x00e1001f, 0x00e1001f, 0x001f001f, 0x00e100e1,
0x001f00e1, 0x001f00e1, 0x00e1001f, 0x00e100e1,
0x001f001f, 0x001f001f, 0x00e1001f, 0x001f00e1,
0x001f00e1, 0x001f001f, 0x00e100e1, 0x001f001f,
0x001f001f, 0x001f00e1, 0x001f001f, 0x001f00e1,
0x001f00e1, 0x00e1001f, 0x00e1001f, 0x00e100e1,
0x00e1001f, 0x001f00e1, 0x00e1001f, 0x001f00e1,
0xe1e1e11f, 0x1fe11f1f, 0x1f1f1fe1, 0xe1e11fe1,
0x1f1fe11f, 0xe1e11f1f, 0xe11fe1e1, 0xe11fe1e1,
0xe1e11fe1, 0x1fe1e11f, 0x1f1fe11f, 0xe1e1e1e1,
0x1fe1e1e1, 0xe1e11fe1, 0x1f1f1fe1, 0x1f1fe11f,
0x1f1f1fe1, 0xe11f1fe1, 0xe1e1e1e1, 0xe1e11f1f,
0x1fe1e11f, 0xe1e1e1e1, 0x1fe11fe1, 0xe1e1e1e1,
0x1fe11fe1, 0xe11fe11f, 0x1fe1e1e1, 0xe11f1fe1,
0xe1e1e11f, 0x1fe11f1f, 0xe11f1f1f, 0x1fe11fe1,
0x1f1fe1e1, 0xe11f1fe1, 0x1f1f1fe1, 0xe1e11f1f,
0x1f1fe1e1, 0xe11fe1e1, 0xe1e1e11f, 0x1f1fe11f,
0xe11f1f1f, 0x1fe11fe1, 0x1f1f1f1f, 0xe1e11fe1,
0xe1e1e11f, 0xe1e1e11f, 0x1fe11fe1, 0x1fe11f1f,
0xe1e1e11f, 0xe1e1e11f, 0x1f1f1f1f, 0x1f1f1fe1,
0xe11fe1e1, 0xe11f1fe1, 0x1fe1e11f, 0xe1e11f1f,
0xe11fe1e1, 0x1fe11f1f, 0xe1e11fe1, 0xe1e1e1e1,
0x001f00e1, 0x001f00e1, 0x00e100e1, 0x001f001f,
0x001f001f, 0x001f00e1, 0x001f00e1, 0x00e100e1,
0x001f001f, 0x001f00e1, 0x001f001f, 0x001f001f,
0x00e100e1, 0x001f001f, 0x001f00e1, 0x00e1001f,
0x001f00e1, 0x001f00e1, 0x001f001f, 0x001f001f,
0x001f001f, 0x001f00e1, 0x00e100e1, 0x001f001f,
0x00e1001f, 0x00e100e1, 0x001f00e1, 0x001f00e1,
0x001f001f, 0x00e100e1, 0x001f00e1, 0x00e1001f,
0x001f00e1, 0x00e100e1, 0x00e1001f, 0x00e1001f,
0x001f00e1, 0x001f00e1, 0x00e1001f, 0x00e100e1,
0x001f00e1, 0x001f00e1, 0x00e1001f, 0x00e100e1,
0x001f00e1, 0x00e1001f, 0x001f001f, 0x00e1001f,
0x00e100e1, 0x00e1001f, 0x00e1001f, 0x00e100e1,
0x00e100e1, 0x00e1001f, 0x001f001f, 0x00e100e1,
0x001f00e1, 0x00e1001f, 0x001f00e1, 0x00e1001f,
0x00e100e1, 0x00e1001f, 0x00e1001f, 0x00e1001f,
0x00e1001f, 0x00e100e1, 0x001f00e1, 0x00e100e1,
0x00e1001f, 0x00e1001f, 0x001f00e1, 0x00e1001f,
0x00e1001f, 0x001f001f, 0x001f00e1, 0x001f00e1,
0x001f00e1, 0x00e1001f, 0x00e1001f, 0x00e100e1,
0x00e1001f, 0x00e100e1, 0x00e100e1, 0x00e100e1,
0x00e100e1, 0x001f001f, 0x001f00e1, 0x001f00e1,
0x00e100e1, 0x001f001f, 0x00e100e1, 0x00e1001f,
0x001f00e1, 0x001f00e1, 0x00e100e1, 0x001f00e1,
0x00e100e1, 0x00e1001f, 0x00e100e1, 0x00e100e1,
0x00e1001f, 0x00e100e1, 0x00e100e1, 0x001f001f,
0x001f00e1, 0x001f00e1, 0x001f001f, 0x00e1001f,
0x00e100e1, 0x001f001f, 0x001f001f, 0x00e100e1,
0x001f001f, 0x00e1001f, 0x00e100e1, 0x00e100e1,
0x00e100e1, 0x001f001f, 0x00e100e1, 0x00e100e1};

#ifdef USE_RANDOM_TEST_VECTOR
/** ============================================================================
 *   @n@b gen_rand
 *
 *   @b Description
 *   @n Random number generator function.
 *
 *   @param[in]  
 *   @n None
 *
 *   @return        
 *   @n None. 
 * =============================================================================
 */
static UInt16 gen_rand ()
{
	UInt16 temp;

	srand(seed);

	temp = (UInt16) (((float)rand()/RAND_MAX) * 32768);
	seed += 3;

	return temp;
}
#endif

/** ============================================================================
 *   @n@b prepare_wimax_enchdr_cfg
 *
 *   @b Description
 *   @n Sets up the Encoder engine header for the test. 
 *
 *   @param[out]  
 *   @n pEncHdrCfg      Encoder engine Header configuration thus populated for 
 *                      the test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_wimax_enchdr_cfg 
(
    Bcp_EncHdrCfg*              pEncHdrCfg
)
{
    /* Setup the Encoder header as per inputs passed. */        
    pEncHdrCfg->local_hdr_len       =   9;    

    pEncHdrCfg->turbo_conv_sel      =   1;
    pEncHdrCfg->code_rate_flag      =   0;
    pEncHdrCfg->scr_crc_en          =   3;  
    
	if (fecBlocks.numFecBlks1)
	{
        pEncHdrCfg->blockCfg[0].block_size      =   fecBlocks.fecCtcBlkLen[0] * numBitsPerSlot;
        pEncHdrCfg->blockCfg[0].num_code_blks   =   fecBlocks.numFecBlks1;

        pEncHdrCfg->blockCfg[0].intvpar0        =   fecBlocks.P0[0];
        pEncHdrCfg->blockCfg[0].intvpar1        =   fecBlocks.P1[0];
        pEncHdrCfg->blockCfg[0].intvpar2        =   fecBlocks.P2[0];
        pEncHdrCfg->blockCfg[0].intvpar3        =   fecBlocks.P3[0];

        pEncHdrCfg->blockCfg[1].block_size      =   fecBlocks.fecCtcBlkLen[1] * numBitsPerSlot;
        pEncHdrCfg->blockCfg[1].num_code_blks   =   fecBlocks.numFecBlks2;
    
        pEncHdrCfg->blockCfg[1].intvpar0        =   fecBlocks.P0[1];
        pEncHdrCfg->blockCfg[1].intvpar1        =   fecBlocks.P1[1];
        pEncHdrCfg->blockCfg[1].intvpar2        =   fecBlocks.P2[1];
        pEncHdrCfg->blockCfg[1].intvpar3        =   fecBlocks.P3[1];
        
        pEncHdrCfg->blockCfg[2].block_size      =   fecBlocks.fecCtcBlkLen[2] * numBitsPerSlot;
        pEncHdrCfg->blockCfg[2].num_code_blks   =   fecBlocks.numFecBlks3;

        pEncHdrCfg->blockCfg[2].intvpar0        =   fecBlocks.P0[2];
        pEncHdrCfg->blockCfg[2].intvpar1        =   fecBlocks.P1[2];
        pEncHdrCfg->blockCfg[2].intvpar2        =   fecBlocks.P2[2];
        pEncHdrCfg->blockCfg[2].intvpar3        =   fecBlocks.P3[2];
	}
	else
	{
        pEncHdrCfg->blockCfg[0].block_size      =   fecBlocks.fecCtcBlkLen[1] * numBitsPerSlot;
        pEncHdrCfg->blockCfg[0].num_code_blks   =   fecBlocks.numFecBlks2;

        pEncHdrCfg->blockCfg[0].intvpar0        =   fecBlocks.P0[0];
        pEncHdrCfg->blockCfg[0].intvpar1        =   fecBlocks.P1[0];
        pEncHdrCfg->blockCfg[0].intvpar2        =   fecBlocks.P2[0];
        pEncHdrCfg->blockCfg[0].intvpar3        =   fecBlocks.P3[0];

        pEncHdrCfg->blockCfg[1].block_size      =   fecBlocks.fecCtcBlkLen[2] * numBitsPerSlot;
        pEncHdrCfg->blockCfg[1].num_code_blks   =   fecBlocks.numFecBlks3;
    
        pEncHdrCfg->blockCfg[1].intvpar0        =   fecBlocks.P0[1];
        pEncHdrCfg->blockCfg[1].intvpar1        =   fecBlocks.P1[1];
        pEncHdrCfg->blockCfg[1].intvpar2        =   fecBlocks.P2[1];
        pEncHdrCfg->blockCfg[1].intvpar3        =   fecBlocks.P3[1];
        
        pEncHdrCfg->blockCfg[2].block_size      =   0;
        pEncHdrCfg->blockCfg[2].num_code_blks   =   0;

        pEncHdrCfg->blockCfg[2].intvpar0        =   0;
        pEncHdrCfg->blockCfg[2].intvpar1        =   0;
        pEncHdrCfg->blockCfg[2].intvpar2        =   0;
        pEncHdrCfg->blockCfg[2].intvpar3        =   0;
	}
    
    return;        
}

/** ============================================================================
 *   @n@b prepare_wimax_rmhdr_cfg
 *
 *   @b Description
 *   @n Sets up the WiMax Rate modulation header for the test. 
 *
 *   @param[out]  
 *   @n pRmHdrCfg       Rate modulation Header configuration thus populated for 
 *                      the test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_wimax_rmhdr_cfg 
(
    Bcp_RmHdr_WiMaxCfg*         pRmHdrCfg
)
{
    /* Setup the WiMax RM header as per the inputs specified */        
    pRmHdrCfg->local_hdr_len            =   6;        

	if (fecBlocks.numFecBlks1)
	{
        pRmHdrCfg->num_code_blocks_c1   =   fecBlocks.numFecBlks1;        
        pRmHdrCfg->num_code_blocks_c2   =   fecBlocks.numFecBlks2;        
        pRmHdrCfg->num_code_blocks_c3   =   fecBlocks.numFecBlks3;        

        pRmHdrCfg->block_size_k1        =   fecBlocks.fecCtcBlkLen[0]*numBitsPerSlot;        
        pRmHdrCfg->block_size_k2        =   fecBlocks.fecCtcBlkLen[1]*numBitsPerSlot;        
        pRmHdrCfg->block_size_k3        =   fecBlocks.fecCtcBlkLen[2]*numBitsPerSlot;        

        pRmHdrCfg->block_size_e1        =   fecBlocks.fecCtcBlkLen[0] * 48 * modType;
        pRmHdrCfg->block_size_e2        =   fecBlocks.fecCtcBlkLen[1] * 48 * modType;
        pRmHdrCfg->block_size_e3        =   fecBlocks.fecCtcBlkLen[2] * 48 * modType;

        pRmHdrCfg->param_m1             =   fecBlocks.M[0];
        pRmHdrCfg->param_m2             =   fecBlocks.M[1];
        pRmHdrCfg->param_m3             =   fecBlocks.M[2];

        pRmHdrCfg->param_j1             =   fecBlocks.J[0];
        pRmHdrCfg->param_j2             =   fecBlocks.J[1];
        pRmHdrCfg->param_j3             =   fecBlocks.J[2];
	}
	else
	{
        pRmHdrCfg->num_code_blocks_c1   =   fecBlocks.numFecBlks2;        
        pRmHdrCfg->num_code_blocks_c2   =   fecBlocks.numFecBlks3;        
        pRmHdrCfg->num_code_blocks_c3   =   fecBlocks.numFecBlks1;        

        pRmHdrCfg->block_size_k1        =   fecBlocks.fecCtcBlkLen[1]*numBitsPerSlot;        
        pRmHdrCfg->block_size_k2        =   fecBlocks.fecCtcBlkLen[2]*numBitsPerSlot;        
        pRmHdrCfg->block_size_k3        =   0;        

        pRmHdrCfg->block_size_e1        =   fecBlocks.fecCtcBlkLen[1] * 48 * modType;
        pRmHdrCfg->block_size_e2        =   fecBlocks.fecCtcBlkLen[2] * 48 * modType;
        pRmHdrCfg->block_size_e3        =   0;

        pRmHdrCfg->param_m1             =   fecBlocks.M[1];
        pRmHdrCfg->param_m2             =   fecBlocks.M[2];
        pRmHdrCfg->param_m3             =   0;

        pRmHdrCfg->param_j1             =   fecBlocks.J[1];
        pRmHdrCfg->param_j2             =   fecBlocks.J[2];
        pRmHdrCfg->param_j3             =   0;
	}

    return;        
}

/** ============================================================================
 *   @n@b prepare_modhdr_cfg
 *
 *   @b Description
 *   @n Sets up the Modulation engine header for the test. 
 *
 *   @param[out]  
 *   @n pModHdrCfg      Modulation engine header configuration thus populated for the test.
 * 
 *   @param[in]  
 *   @n radioStd        Radio standard to use.
 * 
 *   @param[in]  
 *   @n Rprime          Number of allocation subcarriers.
 * 
 *   @param[in]  
 *   @n mod             Modulation type.
 * 
 *   @param[in]  
 *   @n operationMode   Modulation mode, i.e., hard/soft.
 * 
 *   @param[in]  
 *   @n rms             
 * 
 *   @param[in]  
 *   @n numOFDMsymPerSubfrm     Number of SC-FDMA symbols per sub-frame
 * 
 *   @param[in]  
 *   @n numCqiSubcs     Number of CQI sub-carriers per sub-frame
 * 
 *   @param[in]  
 *   @n numRiSubcs      Number of RI sub-carriers per sub-frame   
 * 
 *   @param[in]  
 *   @n numAckSubcs     Number of ACK/NACK sub-carriers per sub-frame   
 * 
 *   @param[in]  
 *   @n cinit           Scrambler second shift register initial value
 * 
 *   @param[in]  
 *   @n qFmt            Input LLR Q format
 * 
 *   @return        
 *   @n None
 * =============================================================================
 */
static Void prepare_modhdr_cfg 
(
    Bcp_ModHdrCfg*      pModHdrCfg,
    Bcp_RadioStd        radioStd,
    UInt32              Rprime, 
    UInt8               mod,   
    UInt8               operationMode, 
    UInt16              rms,
    UInt8               numOFDMsymPerSubfrm,
    UInt16              numCqiSubcs,
    UInt16              numRiSubcs,
    UInt16              numAckSubcs,
    UInt32              cinit,
    UInt8               qFmt
)
{
	UInt32              Rprime1;

    /* Prepare modulation header as per inputs provided */        
    pModHdrCfg->local_hdr_len   =   4;        
    pModHdrCfg->sh_mod_sel      =   (Bcp_ModulationMode)operationMode;        
    pModHdrCfg->jack_bit        =   0;        
    if (mod == 1)
        pModHdrCfg->mod_type_sel=   Bcp_ModulationType_BPSK;
    else
        pModHdrCfg->mod_type_sel=   (Bcp_ModulationType)(mod/2);

    if (operationMode == 2 || operationMode == 3) // soft modulator
    {
	    pModHdrCfg->uva_val     =	rms;          
    }
    else
    {
        if (mod == 1 || mod == 2) // BPSK
            pModHdrCfg->uva_val =   (rms * 23170 + 0x4000) >> 15;        
        else if (mod == 4) //16 QAM
            pModHdrCfg->uva_val =   (rms * 10362 + 0x4000) >> 15;        
        else if (mod == 6) //64 QAM 
            pModHdrCfg->uva_val =   (rms * 5056 + 0x4000) >> 15;        
        else //256 QAM
            pModHdrCfg->uva_val =   (rms * 2513 + 0x4000) >> 15;
    }

    if (radioStd == Bcp_RadioStd_LTE)
        pModHdrCfg->scr_en      =   1;        
    else
        pModHdrCfg->scr_en      =   0;        

    if (operationMode == 2 || operationMode == 3)
    {
        pModHdrCfg->rmux_ln     =   Rprime;        

		if (operationMode == 2) //single layer
			Rprime1 = 12 * prbList[Rprime];
		else //2 layer
			Rprime1 = 12 * prbList[Rprime] * 2; 

        pModHdrCfg->b_table_index=  gind_row_index(Rprime1 * mod);        
        pModHdrCfg->cmux_ln     =   (Bcp_CmuxLength) (numOFDMsymPerSubfrm-9);        
        pModHdrCfg->q_format    =   (Bcp_QFormat) qFmt;        
        pModHdrCfg->cqi_ln      =   numCqiSubcs;        
        pModHdrCfg->ri_ln       =   numRiSubcs;        
        pModHdrCfg->ack_ln      =   numAckSubcs;        
    }
    else
    {
        pModHdrCfg->rmux_ln     =   0;        
        pModHdrCfg->b_table_index=  0;        
        pModHdrCfg->cmux_ln     =   (Bcp_CmuxLength) 0;        
        pModHdrCfg->q_format    =   (Bcp_QFormat) 0;        
        pModHdrCfg->cqi_ln      =   0;        
        pModHdrCfg->ri_ln       =   0;        
        pModHdrCfg->ack_ln      =   0;        
    }
    
    pModHdrCfg->cinit_p2        =   cinit;        
    pModHdrCfg->split_mode_en   =   0;        

    return;
}

/** ============================================================================
 *   @n@b prepare_tmhdr_cfg
 *
 *   @b Description
 *   @n Sets up the Traffic Manager header for the test. 
 *
 *   @param[out]  
 *   @n pTmHdrCfg   TM Header configuration thus populated for the test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_tmhdr_cfg (Bcp_TmHdrCfg* pTmHdrCfg)
{
    pTmHdrCfg->ps_data_size     =   0;
    pTmHdrCfg->info_data_size   =   0;

    return;
}

/** ============================================================================
 *   @n@b gen_ctc_fec_block
 *
 *   @b Description
 *   @n Based on test parameters, generates rest of the code block params for 
 *      the test. 
 * =============================================================================
 */
Void gen_ctc_fec_block (Void)
{
	Int8  j = 0; /* Parameter dependent on the modulation and FEC rate */
	Int16 k = 0; 
	Int16 m = 0;
	Int16 n = 0;
	UInt16 N;
	UInt8 i;

	j = CTC_MCR_TABLE[MCR][2];
	n = numSlotsWoRpt;

	k = n / j;
	m = n % j;

	if( n <= j )
	{
		if ( n != 7)
		{
			fecBlocks.fecCtcBlkLen[0] = n;
			fecBlocks.fecCtcBlkLen[1] = 0;
			fecBlocks.fecCtcBlkLen[2] = 0;
			fecBlocks.numFecBlks1 = 1;
			fecBlocks.numFecBlks2 = 0;
			fecBlocks.numFecBlks3 = 0;

		}
		else if( n == 7 )
		{
			fecBlocks.fecCtcBlkLen[0] = 4;
			fecBlocks.fecCtcBlkLen[1] = 3;
			fecBlocks.fecCtcBlkLen[2] = 0;
			fecBlocks.numFecBlks1 = 1;
			fecBlocks.numFecBlks2 = 1;
			fecBlocks.numFecBlks3 = 0;
		}
	}
	else
	{
		if ( m == 0 )  //n % j
		{
		
			fecBlocks.fecCtcBlkLen[0] = j;
			fecBlocks.fecCtcBlkLen[1] = 0;
			fecBlocks.fecCtcBlkLen[2] = 0;
			
			fecBlocks.numFecBlks1 = k;
			fecBlocks.numFecBlks2 = 0;
			fecBlocks.numFecBlks3 = 0;
		}
		else
		{
			fecBlocks.fecCtcBlkLen[0] = j;
			fecBlocks.fecCtcBlkLen[1] = ( m + j ) / 2 + ( m + j ) % 2;
			fecBlocks.fecCtcBlkLen[2] = ( m + j ) / 2;
			fecBlocks.numFecBlks1 = k-1;
			fecBlocks.numFecBlks2 = 1;
			fecBlocks.numFecBlks3 = 1;
			if ( fecBlocks.fecCtcBlkLen[1] == 7 || fecBlocks.fecCtcBlkLen[2] == 7 )
			{
				fecBlocks.fecCtcBlkLen[1] = fecBlocks.fecCtcBlkLen[1] + 1;
				fecBlocks.fecCtcBlkLen[2] = fecBlocks.fecCtcBlkLen[2] - 1;
			}		
		}
	}
	fecBlocks.numTotalFecBlks = fecBlocks.numFecBlks1 + fecBlocks.numFecBlks2 + fecBlocks.numFecBlks3;


	N = fecBlocks.fecCtcBlkLen[0] * numBitsPerSlot / 2  ; 

	for ( i = 0; i < CTC_MCR_TABLE_NUM; i++ )
	{
		if( modType == gCTCChCodingPerMod[i][0] && codingRate == gCTCChCodingPerMod[i][3] 
				&& N == gCTCChCodingPerMod[i][4])
		{
			fecBlocks.P0[0] = gCTCChCodingPerMod[i][5];
			fecBlocks.P1[0] = gCTCChCodingPerMod[i][6];
			fecBlocks.P2[0] = gCTCChCodingPerMod[i][7];
			fecBlocks.P3[0] = gCTCChCodingPerMod[i][8];
			break;
		}
	}

	for ( i = 0; i < 12; i++ )
	{
		if ( N == subBlockInterleaverTable[i][0] )
		{	
			fecBlocks.M[0] = subBlockInterleaverTable[i][1];
			fecBlocks.J[0] = subBlockInterleaverTable[i][2];
			break;
		}
	}
	

	if (fecBlocks.numFecBlks2)
	{
		N = fecBlocks.fecCtcBlkLen[1] * numBitsPerSlot / 2;

		for ( i = 0; i < CTC_MCR_TABLE_NUM; i++ )
		{
			if( modType == gCTCChCodingPerMod[i][0] && codingRate == gCTCChCodingPerMod[i][3] 
					&& N == gCTCChCodingPerMod[i][4])
			{
				fecBlocks.P0[1] = gCTCChCodingPerMod[i][5];
				fecBlocks.P1[1] = gCTCChCodingPerMod[i][6];
				fecBlocks.P2[1] = gCTCChCodingPerMod[i][7];
				fecBlocks.P3[1] = gCTCChCodingPerMod[i][8];
				break;
			}
		}

		for ( i = 0; i < 12; i++ )
		{	
			if ( N == subBlockInterleaverTable[i][0] )
			{	
				fecBlocks.M[1] = subBlockInterleaverTable[i][1];
				fecBlocks.J[1] = subBlockInterleaverTable[i][2];
				break;
			}
		}
	}

	else
	{
		fecBlocks.P0[1] = 0;
		fecBlocks.P1[1] = 0;
		fecBlocks.P2[1] = 0;
		fecBlocks.P3[1] = 0;
		fecBlocks.M[1] = 0;
		fecBlocks.J[1] = 0;

	}

	if (fecBlocks.numFecBlks3)
	{
		N = fecBlocks.fecCtcBlkLen[2] * numBitsPerSlot / 2;

		for ( i = 0; i < CTC_MCR_TABLE_NUM; i++ )
		{
			if( modType == gCTCChCodingPerMod[i][0] && codingRate == gCTCChCodingPerMod[i][3] 
					&& N == gCTCChCodingPerMod[i][4])
			{
				fecBlocks.P0[2] = gCTCChCodingPerMod[i][5];
				fecBlocks.P1[2] = gCTCChCodingPerMod[i][6];
				fecBlocks.P2[2] = gCTCChCodingPerMod[i][7];
				fecBlocks.P3[2] = gCTCChCodingPerMod[i][8];
				break;
			}
		}

		for ( i = 0; i < 12; i++ )
		{	
			if ( N == subBlockInterleaverTable[i][0] )
			{	
				fecBlocks.M[2] = subBlockInterleaverTable[i][1];
				fecBlocks.J[2] = subBlockInterleaverTable[i][2];
				break;
			}
		}
	}

	else
	{
		fecBlocks.P0[2] = 0;
		fecBlocks.P1[2] = 0;
		fecBlocks.P2[2] = 0;
		fecBlocks.P3[2] = 0;
		fecBlocks.M[2] = 0;
		fecBlocks.J[2] = 0;

	}

    return;
}

/** ============================================================================
 *   @n@b generate_wimax_dl_testvector
 *
 *   @b Description
 *   @n Utility API to generate test parameters. 
 *
 *   @param[in]  
 *   @n maxBurstSize    Burst size
 *
 *   @param[in]  
 *   @n mcrInitVal      MCR Initial value.
 * 
 *   @return        None
 * =============================================================================
 */
Void generate_wimax_dl_testvector (UInt32 maxBurstSize, UInt32 mcrInitVal)
{
	UInt8       numK;
        
    memset (&fecBlocks, 0, sizeof (fecBlocks));
    numSlotsWoRpt = maxBurstSize + 1;

    if (numSlotsWoRpt == 1024) //maximum number of code block is 511
        numK    =   5;
    else
	    numK    =   8;

    /* not used */
    ((Void) (numK));

    MCR         =   mcrInitVal;
    codingRate  =   CTC_MCR_TABLE[MCR][1];
    modType     =   CTC_MCR_TABLE[MCR][0];
			
    // 48 data subcarriers * modualtion * coding rate */
    numBitsPerSlot  =   48 * modType * codingRate/120; //must be multiple of 8
			
    /* calculate the concatenation rule */
    gen_ctc_fec_block();

    totalNumInforBits = numSlotsWoRpt * numBitsPerSlot;
    tbSize = totalNumInforBits - 16 ; //16 bits crc

#ifdef USE_RANDOM_TEST_VECTOR
    rms = SIGRMS[gen_rand()%3];//0,1,2
#else
    rms = 1024;
#endif

    return;
}

/** ============================================================================
 *   @n@b generate_wimax_ul_testvector
 *
 *   @b Description
 *   @n Utility API to generate test parameters. 
 *
 *   @param[in]  
 *   @n maxBurstSize    Burst size
 *
 *   @param[in]  
 *   @n mcrInitVal      MCR Initial value.
 * 
 *   @return        None
 * =============================================================================
 */
Void generate_wimax_ul_testvector (UInt32 maxBurstSize, UInt32 mcrInitVal)
{
	UInt8       numK;

    memset (&fecBlocks, 0, sizeof (fecBlocks));
        
    numSlotsWoRpt = maxBurstSize + 1;

    if (numSlotsWoRpt == 1024) //maximum number of code block is 511
        numK    =   5;
    else
	    numK    =   8;

    /* not used */
    ((Void) (numK));

    MCR         =   mcrInitVal;
    codingRate  =   CTC_MCR_TABLE[MCR][1];
    modType     =   CTC_MCR_TABLE[MCR][0];
			
    // 48 data subcarriers * modualtion * coding rate */
    numBitsPerSlot  =   48 * modType * codingRate/120; //must be multiple of 8
			
    /* calculate the concatenation rule */
    gen_ctc_fec_block();

#ifdef USE_RANDOM_TEST_VECTOR
    rms = SIGRMS[gen_rand()%3];//0,1,2
#else
    rms = 1024;
#endif

#ifdef USE_RANDOM_TEST_VECTOR
    harqFlag = gen_rand()&0x1;
	awgnFlag = gen_rand()&0x1;
#else
    harqFlag = 0;
    awgnFlag = 0;
#endif

#ifdef USE_RANDOM_TEST_VECTOR
    if (modType == 2) //QPSK
        snr = gen_rand()% 21 + 7  ; //7-27dB
    else if (modType == 4) //16QAM
        snr = gen_rand()%21 + 14; //14-34 db
    else if (modType == 6)
        snr = gen_rand()%21 + 20; //20-40db
#else
    snr =   26;
#endif
				
			
    /* add payload size */
    {
        UInt32 tempPayload;
				
        tempPayload = fecBlocks.numFecBlks1 * fecBlocks.fecCtcBlkLen[0] * 48;
        tempPayload += (fecBlocks.numFecBlks2 * fecBlocks.fecCtcBlkLen[1] * 48 );
        tempPayload += (fecBlocks.numFecBlks3 * fecBlocks.fecCtcBlkLen[2] * 48);
    }

#ifdef USE_RANDOM_TEST_VECTOR
	qformat = gen_rand()%4;//0,1,2,3
#else
    qformat = 0;
#endif

    return;
}

/** ============================================================================
 *   @n@b build_dl_packet
 *
 *   @b Description
 *   @n Given a data buffer, this API sets up the packet contents for the test.
 *      It adds all the BCP configuration params (header) and the payload to the 
 *      data buffer. On success, returns the number of bytes of configuration 
 *      and payload data added to the data buffer passed. 
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n pDataBuffer Data Buffer handle to which the payload and BCP configuration
 *                  params need to be added.
 * 
 *   @return        Int32
 *   @n >0      -   Number of bytes of data (payload + header configuration) added 
 *                  to the data buffer.
 *
 *   @n -1      -   Error populating the data buffer.
 * =============================================================================
 */
static Int32 build_dl_packet (Bcp_DrvHandle hBcp, UInt8*  pDataBuffer)
{
	FILE*                   pTestCfgFile;
    UInt32                  maxBurstSize, mcrInitVal, dataBufferLen, tmpLen;        
    Bcp_GlobalHdrCfg        bcpGlblHdrCfg;
    Bcp_TmHdrCfg            tmHdrCfg;
    Bcp_CrcHdrCfg           crcHdrCfg;
    Bcp_EncHdrCfg           encHdrCfg;
    Bcp_RmHdr_WiMaxCfg      rmHdrCfg;
    Bcp_ModHdrCfg           modHdrCfg;
    Bcp_RadioStd            radioStd;

    /* Generate a DL test vector parameter set for us to use */
    maxBurstSize    =   511;
    mcrInitVal      =   0;
    generate_wimax_dl_testvector (maxBurstSize, mcrInitVal);
    
    /* Start building the BCP packet  */
    radioStd        =   Bcp_RadioStd_WIMAX_802_16E;
    
    /* Initialize our data buffer length running counter */
    dataBufferLen   =   0;

    /* Start adding BCP Packet headers based on the test configuration we read. */
    tmpLen			=	0;

    /* Header 1: Global Header */
    bcpGlblHdrCfg.pkt_type          =   Bcp_PacketType_Normal;
    bcpGlblHdrCfg.flush             =   0;
    bcpGlblHdrCfg.drop              =   0;
    bcpGlblHdrCfg.halt              =   0;
    bcpGlblHdrCfg.radio_standard    =   radioStd;
    bcpGlblHdrCfg.hdr_end_ptr       =   30;   // 2 + 4 + 10 + 7 + 5 + 2
    bcpGlblHdrCfg.flow_id           =   RX_FLOW_ID;
    bcpGlblHdrCfg.destn_tag         =   0xDEAD; 
    if (Bcp_addGlobalHeader (&bcpGlblHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add Global Header to packet \n");
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;
   
    /* Header 2: CRC Header */
    prepare_crchdr_cfg (&crcHdrCfg, radioStd, tbSize, 0, 0, 0, 0, 0, 0, 0, 0, NULL);
    if (Bcp_addCRCHeader (&crcHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add CRC Header to packet \n");
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;
   
    /* Header 3: Encoder Header */
    prepare_wimax_enchdr_cfg (&encHdrCfg);
    if (Bcp_addEncoderHeader (&encHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add Encoder Header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 4: Rate matching header */
    prepare_wimax_rmhdr_cfg (&rmHdrCfg);
    if (Bcp_addWiMax_RMHeader (&rmHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add WiMax rate modulation header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 5: Modulation header */
    prepare_modhdr_cfg (&modHdrCfg, 
                        radioStd, 
				        0, // number of subcarrier, not used,
				        modType,
#ifdef USE_RANDOM_TEST_VECTOR
				        gen_rand()&0x1, //uncompressed mode
#else
                        1,
#endif
				        rms,
				        0, //numOFDMsymPerSubfrm, not used
				        0, //numCqiSubcs
				        0, //numRiSubcs
				        0, //numAckSubcs
				        0, //cInit
				        0); //qformat
    if (Bcp_addModulatorHeader (&modHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add Modulation header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 6: Traffic Manager header */
    prepare_tmhdr_cfg (&tmHdrCfg);
    if (Bcp_addTMHeader (&tmHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add Traffic Manager header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* No Info words/PS to be added for now */

	/* Add padding to align data on 128 bits */    
    pDataBuffer 	+=	8;
    dataBufferLen	+=	8;

    /* Finally add the data to the packet in the same order as the headers */
    if ((pTestCfgFile = fopen("..\\..\\testcases\\wimax\\wimaxTx_inforBits.dat","r")) == NULL)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Cannot open data input file:  ..\..\testcases\wimax\wimaxTx_inforBits.dat\n");
#endif
        return -1;
    }
    read_data_from_file (pTestCfgFile, pDataBuffer, &dataBufferLen);
    fclose (pTestCfgFile);
       
    /* Successfully read the test configuration */        
    return dataBufferLen;
}

/** ============================================================================
 *   @n@b prepare_wimax_sslhdr_cfg
 *
 *   @b Description
 *   @n Sets up the Soft Slicer (SSL) header for the test based on inputs passed.
 * =============================================================================
 */
static Void prepare_wimax_sslhdr_cfg
(
    Bcp_SslHdr_WiMaxCfg* pSslHdrCfg, 
    Bcp_RadioStd        radioStd,
    UInt32              Rprime, //number of subcarrier
    UInt8               mod, 
    UInt8               numOFDMSymPerSubfrm, //lte only
    UInt8               numLayers,
    UInt16              rms,
    UInt8               Qfmt,
    UInt32              cinit,   //lte scrambling only
    UInt16              numRi,
    UInt16              numAck,
    Int32               scaleEven,    //noise scale for even slot
    Int32               scaleOdd      //noise scale for odd slot
)
{
    /* Setup the SSL header as per params passed */
    memset (pSslHdrCfg, 0, sizeof (Bcp_SslHdr_WiMaxCfg));        
    pSslHdrCfg->local_hdr_len = 1;

    pSslHdrCfg->modeSelCfg.split_mode_en       =   0;
    pSslHdrCfg->modeSelCfg.jack_bit            =   0;
    pSslHdrCfg->modeSelCfg.lte_descrambler_en  =   0;
    pSslHdrCfg->modeSelCfg.mod_type_sel        =   (Bcp_ModulationType) (mod/2);
    pSslHdrCfg->modeSelCfg.cmux_ln             =   (Bcp_CmuxLength) (0);
    pSslHdrCfg->modeSelCfg.q_format            =   (Bcp_QFormat) (Qfmt);
    pSslHdrCfg->modeSelCfg.b_matrix_sel        =   0;
    pSslHdrCfg->modeSelCfg.rmux_ln_index       =   0;

    if (mod == 1 || mod == 2)   // BPSK or QPSK
        pSslHdrCfg->uva                 =   ((rms * 23170 + 0x4000) >> 15);
    else if (mod == 4)  // 16 QAM
        pSslHdrCfg->uva                 =   ((rms * 10362 + 0x4000) >> 15);
    else if (mod == 6)  // 64 QAM
        pSslHdrCfg->uva                 =   ((rms * 5056 + 0x4000) >> 15);
    else    // 256 QAM
        pSslHdrCfg->uva                 =   ((rms * 2513 + 0x4000) >> 15);

    return;
}

/** ============================================================================
 *   @n@b prepare_wimax_rdhdr_cfg
 *
 *   @b Description
 *   @n Sets up the WiMax Rate Dematching (RD) header for the test. 
 * =============================================================================
 */
static Void prepare_wimax_rdhdr_cfg 
(
    Bcp_RdHdr_WiMaxCfg*         pRdHdrCfg
)
{
    /* Setup the WiMax Rate dematching header as per inputs */
    memset (pRdHdrCfg, 0, sizeof (Bcp_RdHdr_WiMaxCfg));        
    pRdHdrCfg->local_hdr_len                =   8;        

	if (fecBlocks.numFecBlks1)
	{
        pRdHdrCfg->num_code_blocks_c1       =   fecBlocks.numFecBlks1;        
        pRdHdrCfg->num_code_blocks_c2       =   fecBlocks.numFecBlks2;        
        pRdHdrCfg->num_code_blocks_c3       =   fecBlocks.numFecBlks3;        

        pRdHdrCfg->block_size_k1            =   fecBlocks.fecCtcBlkLen[0]*numBitsPerSlot;        
        pRdHdrCfg->block_size_k2            =   fecBlocks.fecCtcBlkLen[1]*numBitsPerSlot;        
        pRdHdrCfg->block_size_k3            =   fecBlocks.fecCtcBlkLen[2]*numBitsPerSlot;        
        
        pRdHdrCfg->block_size_e1            =   fecBlocks.fecCtcBlkLen[0] * 48 * modType;        
        pRdHdrCfg->block_size_e2            =   fecBlocks.fecCtcBlkLen[1] * 48 * modType;        
        pRdHdrCfg->block_size_e3            =   fecBlocks.fecCtcBlkLen[2] * 48 * modType;        
        
        pRdHdrCfg->param_m1                 =   fecBlocks.M[0];        
        pRdHdrCfg->param_m2                 =   fecBlocks.M[1];        
        pRdHdrCfg->param_m3                 =   fecBlocks.M[2];        

        pRdHdrCfg->param_j1                 =   fecBlocks.J[0];        
        pRdHdrCfg->param_j2                 =   fecBlocks.J[1];        
        pRdHdrCfg->param_j3                 =   fecBlocks.J[2];        
	}
	else
	{
        pRdHdrCfg->num_code_blocks_c1       =   fecBlocks.numFecBlks2;        
        pRdHdrCfg->num_code_blocks_c2       =   fecBlocks.numFecBlks3;        
        pRdHdrCfg->num_code_blocks_c3       =   fecBlocks.numFecBlks1;        

        pRdHdrCfg->block_size_k1            =   fecBlocks.fecCtcBlkLen[1]*numBitsPerSlot;        
        pRdHdrCfg->block_size_k2            =   fecBlocks.fecCtcBlkLen[2]*numBitsPerSlot;        
        pRdHdrCfg->block_size_k3            =   0;        
        
        pRdHdrCfg->block_size_e1            =   fecBlocks.fecCtcBlkLen[1] * 48 * modType;        
        pRdHdrCfg->block_size_e2            =   fecBlocks.fecCtcBlkLen[2] * 48 * modType;        
        pRdHdrCfg->block_size_e3            =   0;        
        
        pRdHdrCfg->param_m1                 =   fecBlocks.M[1];        
        pRdHdrCfg->param_m2                 =   fecBlocks.M[2];        
        pRdHdrCfg->param_m3                 =   0;        

        pRdHdrCfg->param_j1                 =   fecBlocks.J[1];        
        pRdHdrCfg->param_j2                 =   fecBlocks.J[2];        
        pRdHdrCfg->param_j3                 =   0;        
	}
    
	if (harqFlag)
        pRdHdrCfg->enable_harq_input        =   1;
	else
        pRdHdrCfg->enable_harq_input        =   0;

    pRdHdrCfg->enable_harq_output           =   1;
    pRdHdrCfg->harq_input_address           =   convert_coreLocalToGlobalAddr ((UInt32)harqInput);
    pRdHdrCfg->harq_output_address          =   convert_coreLocalToGlobalAddr ((UInt32)harqOutput);
    pRdHdrCfg->tcp3d_scale_factor           =   16;
    pRdHdrCfg->tcp3d_dyn_range              =   0;
    pRdHdrCfg->flowId_starting_offset       =   0;
    pRdHdrCfg->flowId_ending_offset         =   0;

    return;
}	

/** ============================================================================
 *   @n@b build_ul_packet
 *
 *   @b Description
 *   @n Given a data buffer, this API sets up the packet contents for the test.
 *      It adds all the BCP configuration params (header) and the payload to the 
 *      data buffer. On success, returns the number of bytes of configuration 
 *      and payload data added to the data buffer passed. 
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n pDataBuffer Data Buffer handle to which the payload and BCP configuration
 *                  params need to be added.
 * 
 *   @return        Int32
 *   @n >0      -   Number of bytes of data (payload + header configuration) added 
 *                  to the data buffer.
 *
 *   @n -1      -   Error populating the data buffer.
 * =============================================================================
 */
static Int32 build_ul_packet (Bcp_DrvHandle hBcp, UInt8*  pDataBuffer)
{
	FILE*                   pTestCfgFile;
    UInt32                  maxBurstSize, mcrInitVal, dataBufferLen, tmpLen;        
    Bcp_GlobalHdrCfg        bcpGlblHdrCfg;
    Bcp_TmHdrCfg            tmHdrCfg;
    Bcp_RadioStd            radioStd;
    Bcp_RdHdr_WiMaxCfg      rdHdrCfg;
    Bcp_SslHdr_WiMaxCfg     sslHdrCfg;

    /* Generate a DL test vector parameter set for us to use */
    maxBurstSize    =   10;
    mcrInitVal      =   0;
    generate_wimax_ul_testvector (maxBurstSize, mcrInitVal);
    
    /* Start building the BCP packet  */
    radioStd        =   Bcp_RadioStd_WIMAX_802_16E;
    
    /* Initialize our data buffer length running counter */
    dataBufferLen   =   0;

    /* Start adding BCP Packet headers based on the test configuration we read. */
    tmpLen			=	0;

    /* Header 1: Global Header */
    bcpGlblHdrCfg.pkt_type          =   Bcp_PacketType_Normal;
    bcpGlblHdrCfg.flush             =   0;
    bcpGlblHdrCfg.drop              =   0;
    bcpGlblHdrCfg.halt              =   0;
    bcpGlblHdrCfg.radio_standard    =   radioStd;
    bcpGlblHdrCfg.hdr_end_ptr       =   16;   
    bcpGlblHdrCfg.flow_id           =   RX_FLOW_ID;
    bcpGlblHdrCfg.destn_tag         =   0xDEAD; 
    if (Bcp_addGlobalHeader (&bcpGlblHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add Global Header to packet \n");
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 2: Soft slicer header */
    prepare_wimax_sslhdr_cfg (&sslHdrCfg, radioStd, 0, modType, 0, 0, rms, qformat, 
                        0, 0, 0, 0, 0);
    if (Bcp_addWiMax_SSLHeader (&sslHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add SSL Header to packet \n");
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 3: Rate de-matching header */
    prepare_wimax_rdhdr_cfg (&rdHdrCfg);
    if (Bcp_addWiMax_RDHeader (&rdHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add lte rate modulation header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 4: Traffic Manager header */
    prepare_tmhdr_cfg (&tmHdrCfg);
    if (Bcp_addTMHeader (&tmHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add Traffic Manager header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* No Info words/PS to be added for now */

	/* Add padding to align data on 128 bits */    
    pDataBuffer 	+=	4;
    dataBufferLen	+=	4;

    /* Finally add the data to the packet in the same order as the headers */
    if ((pTestCfgFile = fopen("..\\..\\testcases\\wimax\\wimaxRx_sslIn.dat","r")) == NULL)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Cannot open data input file: ..\..\testcases\wimax\wimaxRx_sslIn.dat\n");
#endif
        return -1;
    }
    read_data_from_file (pTestCfgFile, pDataBuffer, &dataBufferLen);
    fclose (pTestCfgFile);
       
    /* Successfully read the test configuration */        
    return dataBufferLen;
}

/** ============================================================================
 *   @n@b test_wimax_dl
 *
 *   @b Description
 *   @n Sets up the parameters required and runs an WiMax DL test case.
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n hGlblFDQ    Global free descriptor queue handle. The test will allocate all its
 *                  free descriptors from this queue for its run and release them
 *                  back once done to this queue.
 * 
 *   @return
 *   @n None
 *
 * =============================================================================
 */
Void test_wimax_dl (Bcp_DrvHandle hBcp, Qmss_QueueHnd hGlblFDQ)
{
    Bcp_TxCfg           txCfg;
    Bcp_TxHandle        hTx = NULL;
    Bcp_RxCfg           rxCfg;
    Bcp_RxHandle        hRx = NULL;
    Qmss_QueueHnd       hTxFDQ, hRxFDQ;
    UInt8               rxFlowId, rxSrcId;
    UInt32              i, dataBufferLen, rxDataBufferLen, rxPsInfoLen, rxDataTotalLen, numRxPackets, testFail = 0, numTestPkts;
    UInt16              rxDestnTag;
    Cppi_Desc*          pCppiDesc;
    UInt8*              pDataBuffer;
    UInt8*              pRxDataBuffer;
    UInt8*              pRxPsInfo;
    Bcp_DrvBufferHandle hRxDrvBuffer;
    Bcp_DrvBufferHandle hVoid;
    Bcp_DrvBufferHandle hTmp;
    Int32               dataBufferLenUsed;
    Qmss_Queue          tmpQ;

    /* Setup Rx side:
     *  -   Open BCP Rx queue on which BCP results are to be received
     *  -   Setup a Rx FDQ, Rx flow to receive data
     *  -   Set the BCP Tunnel Rx endpoint configuration to NULL since we
     *      are processing BCP packets locally.
     */
    if (allocate_fdq (hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, 0, &hRxFDQ, NULL) < 0)
    {
        System_printf ("Error opening Rx FDQ \n");            
        return;
    }
    else
    {
        System_printf ("Rx FDQ %d successfully setup with %d descriptors\n", hRxFDQ, RX_NUM_DESC);
    }
    tmpQ = Qmss_getQueueNumber(hRxFDQ);
    
    rxCfg.rxQNum                        =   RX_Q_NUM;       
    rxCfg.bUseInterrupts                =   0;              // Use polling 
    memset (&rxCfg.flowCfg, 0, sizeof(Cppi_RxFlowCfg));
    rxCfg.flowCfg.flowIdNum             =   RX_FLOW_ID;  
    rxCfg.flowCfg.rx_dest_qmgr          =   0;    
    rxCfg.flowCfg.rx_dest_qnum          =   RX_Q_NUM;  
    rxCfg.flowCfg.rx_desc_type          =   Cppi_DescType_HOST; 
    rxCfg.flowCfg.rx_ps_location        =   Cppi_PSLoc_PS_IN_DESC;  
    rxCfg.flowCfg.rx_psinfo_present     =   0;           
    rxCfg.flowCfg.rx_error_handling     =   0;              //  Drop the packet, do not retry on starvation
    rxCfg.flowCfg.rx_einfo_present      =   0;              //  By default no EPIB info
    
    rxCfg.flowCfg.rx_dest_tag_lo_sel    =   4;              //  Pick dest tag 7:0 bits from the PD dest_tag        
    rxCfg.flowCfg.rx_dest_tag_hi_sel    =   5;              //  Pick the dest tag 15:8 bits from the PD dest_tag
    rxCfg.flowCfg.rx_src_tag_lo_sel     =   2;              //  Pick the src tag 7:0 bits from the PD flow_id 7:0
    rxCfg.flowCfg.rx_src_tag_hi_sel     =   4;              //  Pick the src tag 15:8 bits from the PD src_tag 7:0 

    rxCfg.flowCfg.rx_size_thresh0_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh1_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh2_en    =   0;    

    rxCfg.flowCfg.rx_size_thresh0       =   0x0;
    rxCfg.flowCfg.rx_size_thresh1       =   0x0;
    rxCfg.flowCfg.rx_size_thresh2       =   0x0;

    rxCfg.flowCfg.rx_fdq0_sz0_qmgr      =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq0_sz0_qnum      =   tmpQ.qNum;   
    rxCfg.flowCfg.rx_fdq0_sz1_qnum      =   0x0; 
    rxCfg.flowCfg.rx_fdq0_sz1_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qmgr      =   0x0;

    rxCfg.flowCfg.rx_fdq1_qnum          =   tmpQ.qNum;  
    rxCfg.flowCfg.rx_fdq1_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq2_qnum          =   tmpQ.qNum; 
    rxCfg.flowCfg.rx_fdq2_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq3_qnum          =   tmpQ.qNum; 
    rxCfg.flowCfg.rx_fdq3_qmgr          =   tmpQ.qMgr;

    /* Setup TM flow entry configuration for BCP */
#ifdef SIMULATOR_SUPPORT
    /* Simulator bug workaround: BCP TM module endian handling not correct. */
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_128;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_128;
#else
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_32;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_32;
#endif
    rxCfg.tmFlowCfg.format_in           =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.pkt_type            =   0;      // No special pkt assignments
    rxCfg.tmFlowCfg.dsp_int_sel         =   0;      // Interrupt core 0
    rxCfg.tmFlowCfg.format_out          =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.qfifo_out           =   RX_FLOW_ID;      // Use Rx QFIFO 6
    rxCfg.tmFlowCfg.ps_flags            =   0;      // No PS for now

    if ((hRx = Bcp_rxOpen (hBcp, &rxCfg, NULL)) == NULL)
    {
        System_printf ("BCP Rx Open failed \n");
		goto cleanup_and_return;
    }
    else
    {
        System_printf ("Flow %d opened to send data to RxQ: %d \n", RX_FLOW_ID, RX_Q_NUM);
    }

    /* Setup Tx side:
     *  -   Open BCP Tx queue using which data would be sent 
     *  -   Setup a Tx FDQ and initialize it with some Tx FDs
     *  -   Set the BCP Tunnel Tx endpoint configuration to NULL since we
     *      are processing BCP packets locally.
     */
    txCfg.txQNum    =   Bcp_QueueId_0;
    if ((hTx = Bcp_txOpen (hBcp, &txCfg, NULL)) == NULL)
    {
        System_printf ("BCP Tx Open failed \n");
        return;
    }

    if (allocate_fdq (hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, 0, &hTxFDQ, NULL) < 0)
    {
        System_printf ("Error opening Tx FDQ \n");            
        return;
    }
    else
    {
        System_printf ("Tx FDQ %d successfully setup with %d descriptors\n", hTxFDQ, TX_NUM_DESC);
    }

    /* Build and Send a packet with WiMax DL parameters for BCP Processing */
    for (numTestPkts = 0; numTestPkts < BCP_TEST_NUM_PACKETS; numTestPkts ++)
    {
        if ((pCppiDesc = (Cppi_Desc*) Qmss_queuePop (hTxFDQ)) == NULL)
        {
#ifdef BCP_EXAMPLE_DEBUG
            System_printf ("Out of Tx FDs! \n");
#endif
            testFail ++;
		    goto cleanup_and_return;
        }
        pCppiDesc = (Cppi_Desc *) (QMSS_DESC_PTR (pCppiDesc));    
        Cppi_getData (Cppi_DescType_HOST, pCppiDesc, &pDataBuffer, &dataBufferLen);
        
	    memset (pDataBuffer, 0, dataBufferLen); 

        /* Read test configuration */
        if ((dataBufferLenUsed = build_dl_packet (hBcp, pDataBuffer)) <= 0)
        {
            System_printf ("Error reading test vectors/populating packet \n");
            testFail ++;
            goto cleanup_and_return;
        }

#ifdef BCP_EXAMPLE_DEBUG
   	    System_printf ("Sending a packet of len %d to BCP ...\n", dataBufferLenUsed);
#endif

        /* Setup the data length in the descriptor */
        Cppi_setDataLen (Cppi_DescType_HOST, pCppiDesc, dataBufferLenUsed);
        Cppi_setPacketLen (Cppi_DescType_HOST, pCppiDesc, dataBufferLenUsed);

        /* Since BCP is local to the device, set destination address to NULL */
        Bcp_send (hTx, (Bcp_DrvBufferHandle) pCppiDesc, BCP_EXAMPLE_SIZE_HOST_DESC, NULL);

        /* Wait on data to be received from BCP and validate it */
        for (i = 0; i < 100; i ++);
        while ((numRxPackets = Bcp_rxGetNumOutputEntries (hRx)) == 0);

#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Got %d packet(s) from BCP \n", numRxPackets);
#endif

        for (i = 0; i < numRxPackets; i ++)
        {
            /* Data could arrive scattered across multiple linked descriptors.
             * Collect data from all linked descriptors and validate it.
             */
            rxDataTotalLen  =   0;

            Bcp_recv (hRx,
                    &hRxDrvBuffer,
                    &pRxDataBuffer,
                    &rxDataBufferLen,
                    &pRxPsInfo,
                    &rxPsInfoLen,
                    &rxFlowId,
                    &rxSrcId,
                    &rxDestnTag);            

            if (validate_rxdata ((UInt8 *)wimax_dl_out_packet_1, 
                                WIMAX_DL_OUTPUT_PKT_1_WRD_SIZE*4, 
                                pRxDataBuffer, 
                                rxDataBufferLen, 
                                rxDataTotalLen) != 0)
                testFail ++;                    

            rxDataTotalLen  +=  rxDataBufferLen;

            /* Check if there are any descriptors linked to this Rx desc */
            while (hRxDrvBuffer)
            {
                /* Save original Rx desc handle. */
                hTmp = hRxDrvBuffer;

                if ((hRxDrvBuffer = Cppi_getNextBD (Cppi_getDescType (hRxDrvBuffer), hRxDrvBuffer)))
                {
                    Bcp_rxProcessDesc  (hRx,
                                        hRxDrvBuffer,
                                        &hVoid,
                                        &pRxDataBuffer,
                                        &rxDataBufferLen,
                                        &pRxPsInfo,
                                        &rxPsInfoLen,
                                        &rxFlowId,
                                        &rxSrcId,
                                        &rxDestnTag);            

                    if (validate_rxdata ((UInt8 *)wimax_dl_out_packet_1, 
                                        WIMAX_DL_OUTPUT_PKT_1_WRD_SIZE*4, 
                                        pRxDataBuffer, 
                                        rxDataBufferLen, 
                                        rxDataTotalLen) != 0)
                        testFail ++;                           

                    rxDataTotalLen  +=  rxDataBufferLen;

                }
       
                Bcp_rxFreeRecvBuffer (hRx, hTmp, BCP_EXAMPLE_SIZE_HOST_DESC);
            }

#ifdef BCP_EXAMPLE_DEBUG
            System_printf ("[Pkt %d]: Total Len: %d DestnTag: 0x%x SrcId: 0x%x \n", i, rxDataTotalLen, rxDestnTag, rxSrcId);
#endif

            if (rxDataTotalLen != WIMAX_DL_OUTPUT_PKT_1_WRD_SIZE * 4)
                testFail ++;                
        }
    }
    
cleanup_and_return:
    if (testFail > 0)
    {
        System_printf ("WiMAX DL Test:    FAILED\n");                
        totalNumTestsFail ++;
    }
    else
    {
        System_printf ("WiMAX DL Test:    PASS\n");                
        totalNumTestsPass ++;
    }    

    if (hRx)
        Bcp_rxClose (hRx);

    deallocate_fdq (hRxFDQ, hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, NULL);

    if (hTx)
        Bcp_txClose (hTx);

    deallocate_fdq (hTxFDQ, hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, NULL);

    return;
}

/** ============================================================================
 *   @n@b test_wimax_ul
 *
 *   @b Description
 *   @n Sets up the parameters required and runs an WiMax UL test case.
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n hGlblFDQ    Global free descriptor queue handle. The test will allocate all its
 *                  free descriptors from this queue for its run and release them
 *                  back once done to this queue.
 * 
 *   @return
 *   @n None
 *
 * =============================================================================
 */
Void test_wimax_ul (Bcp_DrvHandle hBcp, Qmss_QueueHnd hGlblFDQ)
{
    Bcp_TxCfg           txCfg;
    Bcp_TxHandle        hTx = NULL;
    Bcp_RxCfg           rxCfg;
    Bcp_RxHandle        hRx = NULL;
    Qmss_QueueHnd       hTxFDQ, hRxFDQ;
    UInt8               rxFlowId, rxSrcId;
    UInt32              i, dataBufferLen, rxDataBufferLen, rxPsInfoLen, rxDataTotalLen, testFail = 0, numTestPkts;
    UInt16              rxDestnTag;
    Cppi_Desc*          pCppiDesc;
    UInt8*              pDataBuffer;
    UInt8*              pRxDataBuffer;
    UInt8*              pRxPsInfo;
    Bcp_DrvBufferHandle hRxDrvBuffer;
    Bcp_DrvBufferHandle hVoid;
    Bcp_DrvBufferHandle hTmp;
    Int32               dataBufferLenUsed;
    Qmss_Queue          tmpQ;

    /* Setup Rx side:
     *  -   Open BCP Rx queue on which BCP results are to be received
     *  -   Setup a Rx FDQ, Rx flow to receive data
     *  -   Set the BCP Tunnel Rx endpoint configuration to NULL since we
     *      are processing BCP packets locally.
     */
    if (allocate_fdq (hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, 0, &hRxFDQ, NULL) < 0)
    {
        System_printf ("Error opening Rx FDQ \n");            
        return;
    }
    else
    {
        System_printf ("Rx FDQ %d successfully setup with %d descriptors\n", hRxFDQ, RX_NUM_DESC);
    }
    tmpQ = Qmss_getQueueNumber(hRxFDQ);
    
    rxCfg.rxQNum                        =   RX_Q_NUM;       
    rxCfg.bUseInterrupts                =   0;              // Use polling 
    memset (&rxCfg.flowCfg, 0, sizeof(Cppi_RxFlowCfg));
    rxCfg.flowCfg.flowIdNum             =   RX_FLOW_ID;  
    rxCfg.flowCfg.rx_dest_qmgr          =   0;    
    rxCfg.flowCfg.rx_dest_qnum          =   RX_Q_NUM;  
    rxCfg.flowCfg.rx_desc_type          =   Cppi_DescType_HOST; 
    rxCfg.flowCfg.rx_ps_location        =   Cppi_PSLoc_PS_IN_DESC;  
    rxCfg.flowCfg.rx_psinfo_present     =   0;           
    rxCfg.flowCfg.rx_error_handling     =   0;              //  Drop the packet, do not retry on starvation
    rxCfg.flowCfg.rx_einfo_present      =   0;              //  By default no EPIB info
    
    rxCfg.flowCfg.rx_dest_tag_lo_sel    =   4;              //  Pick dest tag 7:0 bits from the PD dest_tag        
    rxCfg.flowCfg.rx_dest_tag_hi_sel    =   5;              //  Pick the dest tag 15:8 bits from the PD dest_tag
    rxCfg.flowCfg.rx_src_tag_lo_sel     =   2;              //  Pick the src tag 7:0 bits from the PD flow_id 7:0
    rxCfg.flowCfg.rx_src_tag_hi_sel     =   4;              //  Pick the src tag 15:8 bits from the PD src_tag 7:0 

    rxCfg.flowCfg.rx_size_thresh0_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh1_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh2_en    =   0;    

    rxCfg.flowCfg.rx_size_thresh0       =   0x0;
    rxCfg.flowCfg.rx_size_thresh1       =   0x0;
    rxCfg.flowCfg.rx_size_thresh2       =   0x0;

    rxCfg.flowCfg.rx_fdq0_sz0_qmgr      =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq0_sz0_qnum      =   tmpQ.qNum;   
    rxCfg.flowCfg.rx_fdq0_sz1_qnum      =   0x0; 
    rxCfg.flowCfg.rx_fdq0_sz1_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qmgr      =   0x0;

    rxCfg.flowCfg.rx_fdq1_qnum          =   tmpQ.qNum;
    rxCfg.flowCfg.rx_fdq1_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq2_qnum          =   tmpQ.qNum; 
    rxCfg.flowCfg.rx_fdq2_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq3_qnum          =   tmpQ.qNum;
    rxCfg.flowCfg.rx_fdq3_qmgr          =   tmpQ.qMgr;

    /* Setup TM flow entry configuration for BCP */
#ifdef SIMULATOR_SUPPORT
    /* Simulator bug workaround: BCP TM module endian handling not correct. */
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_128;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_128;
#else
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_32;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_32;
#endif
    rxCfg.tmFlowCfg.format_in           =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.pkt_type            =   0;      // No special pkt assignments
    rxCfg.tmFlowCfg.dsp_int_sel         =   0;      // Interrupt core 0
    rxCfg.tmFlowCfg.format_out          =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.qfifo_out           =   RX_FLOW_ID;      // Use Rx QFIFO 6
    rxCfg.tmFlowCfg.ps_flags            =   0;      // No PS for now

    if ((hRx = Bcp_rxOpen (hBcp, &rxCfg, NULL)) == NULL)
    {
        System_printf ("BCP Rx Open failed \n");
		goto cleanup_and_return;
    }
    else
    {
        System_printf ("Flow %d opened to send data to RxQ: %d \n", RX_FLOW_ID, RX_Q_NUM);
    }

    /* Setup Tx side:
     *  -   Open BCP Tx queue using which data would be sent 
     *  -   Setup a Tx FDQ and initialize it with some Tx FDs
     *  -   Set the BCP Tunnel Tx endpoint configuration to NULL since we
     *      are processing BCP packets locally.
     */
    txCfg.txQNum    =   Bcp_QueueId_0;
    if ((hTx = Bcp_txOpen (hBcp, &txCfg, NULL)) == NULL)
    {
        System_printf ("BCP Tx Open failed \n");
        return;
    }

    if (allocate_fdq (hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, 0, &hTxFDQ, NULL) < 0)
    {
        System_printf ("Error opening Tx FDQ \n");            
        return;
    }
    else
    {
        System_printf ("Tx FDQ %d successfully setup with %d descriptors\n", hTxFDQ, TX_NUM_DESC);
    }

    /* Build and Send a packet with WiMax DL parameters for BCP Processing */
    for (numTestPkts = 0; numTestPkts < BCP_TEST_NUM_PACKETS; numTestPkts ++)
    {
        if ((pCppiDesc = (Cppi_Desc*) Qmss_queuePop (hTxFDQ)) == NULL)
        {
#ifdef BCP_EXAMPLE_DEBUG
            System_printf ("Out of Tx FDs! \n");
#endif
            testFail ++;
		    goto cleanup_and_return;
        }
        pCppiDesc = (Cppi_Desc *) (QMSS_DESC_PTR (pCppiDesc));    
        Cppi_getData (Cppi_DescType_HOST, pCppiDesc, &pDataBuffer, &dataBufferLen);
        
	    memset (pDataBuffer, 0, dataBufferLen); 

        /* Read test configuration */
        if ((dataBufferLenUsed = build_ul_packet (hBcp, pDataBuffer)) <= 0)
        {
            System_printf ("Error reading test vectors/populating packet \n");
            testFail ++;
            goto cleanup_and_return;
        }

#ifdef BCP_EXAMPLE_DEBUG
   	    System_printf ("Sending a packet of len %d to BCP ...\n", dataBufferLenUsed);
#endif

        /* Setup the data length in the descriptor */
        Cppi_setDataLen (Cppi_DescType_HOST, pCppiDesc, dataBufferLenUsed);
        Cppi_setPacketLen (Cppi_DescType_HOST, pCppiDesc, dataBufferLenUsed);

        /* Since BCP is local to the device, set destination address to NULL */
        Bcp_send (hTx, (Bcp_DrvBufferHandle) pCppiDesc, BCP_EXAMPLE_SIZE_HOST_DESC, NULL);

        /* Wait on data to be received from BCP and validate it */
        for (i = 0; i < 100; i ++);
        while (Bcp_rxGetNumOutputEntries (hRx) == 0);

#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Got %d packet(s) from BCP \n", Bcp_rxGetNumOutputEntries (hRx));
#endif

        /* RD output is one per code block per packet. Get one code
         * block output at a time and validate it until we are done receiving from BCP.
         */
        i = 0;
        rxDataTotalLen  =   0;
        while (1)
        {
            Bcp_recv (hRx,
                    &hRxDrvBuffer,
                    &pRxDataBuffer,
                    &rxDataBufferLen,
                    &pRxPsInfo,
                    &rxPsInfoLen,
                    &rxFlowId,
                    &rxSrcId,
                    &rxDestnTag); 
            /* No more BCP output. Done receiving. */
		    if (!hRxDrvBuffer)
			    break;                           

/* Validate against reference data only if input is not random test vector */
#ifndef USE_RANDOM_TEST_VECTOR
            /* Ignore validating the packet header */
            validate_rxdata ((UInt8 *)wimax_ul_out_packet_1, 
                            WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4, 
                            pRxDataBuffer, 
                            WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4, 
                            rxDataTotalLen);

            /* Validate the RD output payload against reference data */
            if (validate_rxdata ((UInt8 *)wimax_ul_out_packet_2, 
                                WIMAX_UL_OUTPUT_PKT_2_WRD_SIZE*4, 
                                (pRxDataBuffer + WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4), 
                                (rxDataBufferLen - WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4),
                                rxDataTotalLen) != 0)
                testFail ++;                    

            rxDataTotalLen  +=  (rxDataBufferLen - WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4);
#endif

            /* Check if there are any descriptors linked to this Rx desc */
            while (hRxDrvBuffer)
            {
                /* Save original Rx desc handle. */
                hTmp = hRxDrvBuffer;

                if ((hRxDrvBuffer = Cppi_getNextBD (Cppi_getDescType (hRxDrvBuffer), hRxDrvBuffer)))
                {
                    Bcp_rxProcessDesc  (hRx,
                                        hRxDrvBuffer,
                                        &hVoid,
                                        &pRxDataBuffer,
                                        &rxDataBufferLen,
                                        &pRxPsInfo,
                                        &rxPsInfoLen,
                                        &rxFlowId,
                                        &rxSrcId,
                                        &rxDestnTag);            

/* Validate against reference data only if input is not random test vector */
#ifndef USE_RANDOM_TEST_VECTOR
                    /* Ignore validating the packet header */
                    validate_rxdata ((UInt8 *)wimax_ul_out_packet_1, 
                                    WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4, 
                                    pRxDataBuffer, 
                                    WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4, 
                                    rxDataTotalLen);

                    /* Validate the RD output payload against reference data */
                    if (validate_rxdata ((UInt8 *)wimax_ul_out_packet_2, 
                                    WIMAX_UL_OUTPUT_PKT_2_WRD_SIZE*4, 
                                    (pRxDataBuffer + WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4), 
                                    (rxDataBufferLen - WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4), 
                                    rxDataTotalLen) != 0)
                        testFail ++;                    

                    rxDataTotalLen  +=  (rxDataBufferLen - WIMAX_UL_OUTPUT_PKT_1_WRD_SIZE*4);
#endif
                }
        
                Bcp_rxFreeRecvBuffer (hRx, hTmp, BCP_EXAMPLE_SIZE_HOST_DESC);
            }

#ifdef BCP_EXAMPLE_DEBUG
            System_printf ("[Pkt %d]: Total Len: %d DestnTag: 0x%x SrcId: 0x%x \n", i, rxDataTotalLen, rxDestnTag, rxSrcId);
#endif

		    i++;
        }

#ifndef USE_RANDOM_TEST_VECTOR
        if (rxDataTotalLen != (WIMAX_UL_OUTPUT_PKT_2_WRD_SIZE * 4))
            testFail ++;                
#endif
    }
    
cleanup_and_return:
    if (testFail > 0)
    {
        System_printf ("WiMAX UL Test:    FAILED\n");                
        totalNumTestsFail ++;
    }
    else
    {
        System_printf ("WiMAX UL Test:    PASS\n");                
        totalNumTestsPass ++;
    }    

    if (hRx)
        Bcp_rxClose (hRx);

    deallocate_fdq (hRxFDQ, hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, NULL);

    if (hTx)
        Bcp_txClose (hTx);

    deallocate_fdq (hTxFDQ, hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, NULL);

    return;
}
